<?php
require __DIR__.'/../datos/Conexion.clase.php';
require __DIR__.'/../util/definiciones.php';
date_default_timezone_set("America/Lima");

class Fundo extends Conexion {

    public function listarFundos($codigo_empresa){
        try {
            $sql = "SELECT
                        f.codigo_entrada_fundo,
                        to_char(f.fecha_entrada_fundo,'DD/MM/YYYY') AS fecha_entrada,
                        coalesce(to_char(f.fecha_manual,'DD/MM/YYYY'),to_char(f.fecha_entrada_fundo,'DD/MM/YYYY')) AS fecha_manual,
                        to_char(f.hora_entrada_fundo, 'HH24:MI') AS hora_entrada,
                        coalesce(f.dni_conductor, 'sin-dni') as dni,
                        coalesce(f.nombre_conductor, 'sin-nombre') as conductor,
                        coalesce(f.codigo_categoria_personal, 0) as codigo_categoria_personal,
                        UPPER(coalesce(f.nombre_empresa, 'sin-nombre')) as nombre_empresa,
                        coalesce(f.codigo_tipo_vehiculo, 0) as codigo_tipo_vehiculo,
                        UPPER(coalesce(f.placa_entrada_fundo, 'sin-placa')) as placa,
                        upper(coalesce(f.procedencia_fundo,'sin-procedencia')) as procedencia_fundo,
                        (case coalesce(f.numero_asientos, '0') when '' then '0' else coalesce(f.numero_asientos, '0') end) as numero_asientos,
                        (case coalesce(f.numero_ocupantes, '0') when '' then '0' else coalesce(f.numero_ocupantes, '0') end) as numero_ocupantes,
                        upper(coalesce(f.motivo_fundo, '')) as motivo_fundo,
                        coalesce(f.contenedor, '') as contenedor,
                        f.entrada_salida_fundo,
                        f.codigo_empresa,
                        f.codigo_usuario,
                        f.estado_entrada_fundo,
                        to_char(f.hora_manual, 'HH24:MI') AS hora_manual,
                        coalesce(f.codigo_ruta, 0) as codigo_ruta,
                        coalesce(f.codigo_entrada, 0) as codigo_entrada,
                        coalesce(f.codigo_caseta, 0) as codigo_caseta,
                        coalesce(f.codigo_off_line, '') as codigo_off,
                        coalesce(f.orientacion, '') as orientacion,
                        coalesce(f.pin, '') as pin,
                        coalesce(f.mano_obra, '') as mano_obra,
                        coalesce(f.codigo_proveedor, 0) as codigo_proveedor,
                        coalesce(f.codigo_ruta_transporte, 0) as codigo_ruta_transporte,
                        coalesce(f.codigo_proveedor_empresa, 0) as codigo_proveedor_empresa,
                        (select count(*) from visita_fundos as v where v.codigo_entrada_fundo = f.codigo_entrada_fundo) as visitas,
                        (select count(*) from fundo_entrada_control_tipo_mano_obra as fectmo where fectmo.codigo_entrada_fundo = f.codigo_entrada_fundo) as num_manos_obra
                    from
                        fundo_entrada_control as f
                    where
                        f.fecha_entrada_fundo >= CURRENT_DATE - INTERVAL '30 DAYS' and f.codigo_empresa = :codigo_empresa and f.estado_entrada_fundo='E'
                    order by 2 desc;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":codigo_empresa", $codigo_empresa);
            $sentencia->execute();

            if (!$sentencia->rowCount()) {
                return [];
            }

            $resultado = $sentencia->fetchAll(PDO::FETCH_ASSOC);

            foreach ($resultado as $item){
                $id_entrada = $item["codigo_entrada_fundo"];
                $visitas = $item["visitas"];
                if($visitas > 0 ){
                    $sql = "SELECT
                                codigo_visita_fundo,
                                codigo_entrada_fundo,
                                dni,
                                coalesce(nombre, '') as nombre,
                                coalesce(comentario, '') as comentario,
                                coalesce(codigo_off_line, '') as codigo_off_line,
                                coalesce(codigo_off_line_fundo, '') as codigo_off_line_fundo
                            FROM visita_fundos where codigo_entrada_fundo = :p_codigo_entrada_fundo";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_codigo_entrada_fundo", $id_entrada);
                    $sentencia->execute();
                    $item["visitas_lista"] = $sentencia->fetchAll(PDO::FETCH_ASSOC);
                }else{
                    $item["visitas_lista"] = [];
                }

                $num_manos_obra = $item["num_manos_obra"];
                if($num_manos_obra > 0 ){
                    $sql = "SELECT
                                codigo_entrada_fundo as codigo_entrada,
                                codigo_tipo_mano_obra as codigo_mano_obra,
                                numero_trabajadores,
                                coalesce(codigo_off_line, '') as off_line,
                                coalesce(codigo_off_line_fundo, '') as off_line_fundo
                            FROM fundo_entrada_control_tipo_mano_obra where codigo_entrada_fundo = :p_codigo_entrada_fundo";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_codigo_entrada_fundo", $id_entrada);
                    $sentencia->execute();
                    $item["manos_obra_lista"] = $sentencia->fetchAll(PDO::FETCH_ASSOC);
                }else{
                    $item["manos_obra_lista"] = [];
                }

                $arrayResultado[] = $item;
            }

            return $arrayResultado;
        } catch (Exception $e) {
            throw $e;
        }
    }

    public function datosEntradaFundo($dni){
        $this->dblink->beginTransaction();
        try{
            // búsqueda de entradas en el fundo
            $sql = " SELECT
                            codigo_entrada_fundo,
                            codigo_empresa,
                            dni_conductor,
                            upper(placa_entrada_fundo) as placa,
                            codigo_categoria_personal,
                            codigo_tipo_vehiculo,
                            nombre_empresa,
                            procedencia_fundo,
                            (coalesce (fecha_manual, fecha_entrada_fundo)) as fecha_entrada_fundo,
                            hora_manual,
                            (case coalesce(numero_asientos, '0') when '' then '0' else coalesce(numero_asientos, '0') end) as numero_asientos
                        from
                            fundo_entrada_control
                        WHERE
                            lower(trim(dni_conductor)) = lower(trim(:p_dni)) and entrada_salida_fundo='E'
                        order by
                           codigo_entrada_fundo desc
                        limit 1";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $dni);
            $sentencia->execute();
            if (!$sentencia->rowCount()) {
                $resultado["codigo_entrada_fundo"] = 0;
                $resultado["dni_conductor"] = $dni;
                $resultado["placa"] = "";
                $resultado["codigo_categoria_personal"] = 0;
                $resultado["codigo_tipo_vehiculo"] = 0;
                $resultado["nombre_empresa"] = "";
                $resultado["procedencia_fundo"] = "";
                $resultado["fecha_entrada_fundo"] = "";
                $resultado["numero_asientos"] = 0;
                $resultado["codigo_empresa"] = 0;
            } else {
                $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
            }
            // búsqueda de entradas en el fundo

            // búsqueda en la caseta en el intervalo de un día
            $sql = "SELECT
                            coalesce(codigo_caseta,0) as codigo_caseta
                        from
                            garita_entrada_salida_control
                        where
                            lower(trim(dni_conductor)) = lower(trim(:p_dni)) and tipo = 'E'  and fecha_entrada_salida >= CURRENT_DATE - INTERVAL '1 DAY'
                        order by
                            fecha_entrada_salida desc
                        limit 1;";

            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $dni);
            $sentencia->execute();
            if (!$sentencia->rowCount()) {
                $resultado["codigo_caseta"] = 0;
            } else {
                $resultadoCaseta = $sentencia->fetch(PDO::FETCH_ASSOC);
                $resultado["codigo_caseta"] = $resultadoCaseta["codigo_caseta"];
            }
            // búsqueda en la caseta en el intervalo de un día

            // búsqueda en conductor
            $sql = "SELECT
                        (apellidos_paternos_conductor ||' ' || apellidos_maternos_conductor ||' ' || nombre_conductor ) as nombre_conductor,
                        (case when estado_conductor ='A' then 'ACTIVO' ELSE 'INACTIVO' end)::character varying(15) as estado,
                        codigo_conductor
                    FROM
                        conductor_control
                    where
                        lower(trim(dni_conductor)) = lower(trim(:p_dni))
                    limit 1;";

            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $dni);
            $sentencia->execute();

            if ($sentencia->rowCount()) {
                //si encontró datos en la tabla conductor, entonces guardamos los resultados
                $resultadoPersona = $sentencia->fetch(PDO::FETCH_ASSOC);
                $resultado["nombre_conductor"] =  $resultadoPersona["nombre_conductor"];
                $resultado["estado"] = $resultadoPersona["estado"];
                $resultado["codigo_conductor"] = $resultadoPersona["codigo_conductor"];
                $resultado["sql"] = "BD pro olmos";
            } else {
                //no encontrado en nuestra BD, entonces se consulta con reniec
                $json = '{CODDNI: "' . $dni . '"}';
                $ch = curl_init();
                curl_setopt($ch, CURLOPT_URL, CONSULTA_JNE);
                curl_setopt($ch, CURLOPT_POST, 1);
                curl_setopt($ch, CURLOPT_POSTFIELDS, $json);
                curl_setopt($ch, CURLOPT_HEADER, TRUE);
                curl_setopt($ch, CURLOPT_HTTPHEADER, array("Content-Type: application/json", "RequestVerificationToken: " . TOKEN_JNE));
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);

                $head = curl_exec($ch);
                curl_close($ch);

                $porciones = explode("GMT", $head);
                $ultimo_elemento = count($porciones) - 1;
                $respuesta = json_decode($porciones[$ultimo_elemento], TRUE);

                if (isset($respuesta["data"])) {
                    $respuesta = explode("|", $respuesta["data"]);
                    /* Si no fue encontrado en RENIEC se regresa un array vacío */
                    if ($respuesta[0] == "" && $respuesta[1] == "" && $respuesta[2] == "") {
                        $resultado["nombre_conductor"] =  "CONDUCTOR NO ENCONTRADO";
                        $resultado["estado"] = 'ACTIVO';
                        $resultado["codigo_conductor"] = 0;
                        $resultado["sql"] = "JNE no encontrado";
                    } else {
                        $resultado["nombre_conductor"] = $respuesta[0] . " " . $respuesta[1] . ", " . $respuesta[2];
                        $resultado["estado"] = 'ACTIVO';
                        $resultado["codigo_conductor"] = 0;
                        $resultado["sql"] = "JNE";
                    }
                } else {
                    $resultado["nombre_conductor"] =  "CONDUCTOR NO ENCONTRADO";
                    $resultado["estado"] = 'ACTIVO';
                    $resultado["codigo_conductor"] = 0;
                    $resultado["sql"] = "else -> no data en reniec";
                }
            }
            // búsqueda en conductor
            return $resultado;
        } catch (Exception $exc) {
            $this->dblink->rollBack();
            throw $exc;
        }
    }

    public function obtenerQRConductorPorDni($dni){
        try{
            $sql = "SELECT
                        upper(trim(c.dni_conductor)) as dni_conductor,
                        upper(v.placa) as placa,
                        c.codigo_tipo_personal as codigo_categoria_personal,
                        v.codigo_tipo_vehiculo,
                        v.capacidad as numero_asientos,
                        e.razon_social as nombre_empresa,
                        upper(c.apellidos_paternos_conductor||' '||c.apellidos_maternos_conductor||', '||c.nombre_conductor) as nombre_conductor,
                        rt.nombre_ruta as procedencia_fundo,
                        dt.codigo_ruta_transporte,
                        dt.codigo_proveedor,
                        dt.codigo_proveedor_empresa,
                        c.estado_conductor as estado
                    FROM
                        proveedor_empresa as dt inner join conductor_control as c on(dt.codigo_conductor = c.codigo_conductor)
                        inner join vehiculo as v on (dt.codigo_vehiculo = v.codigo_vehiculo)
                        inner join proveedor as e on(dt.codigo_proveedor = e.codigo_proveedor)
                        inner join ruta_transporte as rt on(dt.codigo_ruta_transporte = rt.codigo_ruta_transporte)
                    WHERE
                        trim(c.dni_conductor) = trim(?)
                    ORDER BY
                        dt.codigo_proveedor_empresa desc
                    LIMIT
                        1;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(1, $dni);
            $sentencia->execute();
            if (!$sentencia->rowCount()) {
                return "";
            } else {
                $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
                //Búsqueda de datos de fundo_entrada_control previos
                $sql= "SELECT
                            (coalesce (f.fecha_manual, f.fecha_entrada_fundo)) as fecha_entrada_fundo,
                            f.hora_manual
                        From
                            fundo_entrada_control as f
                        WHERE
                            f.dni_conductor = :p_dni
                        ORDER BY
                            1 desc, 2 desc;";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_dni", $resultado["dni_conductor"]);
                $sentencia->execute();
                if (!$sentencia->rowCount()) {
                    $resultado["fecha_entrada_fundo"] = "";
                    $resultado["hora_manual"] = "";
                }else{
                    $resultadoFundo = $sentencia->fetch(PDO::FETCH_ASSOC);
                    $resultado["fecha_entrada_fundo"] = $resultadoFundo["fecha_entrada_fundo"];
                    $resultado["hora_manual"] = $resultadoFundo["hora_manual"];
                }
                //Búsqueda de datos de fundo_entrada_control previos

                // búsqueda en la caseta en el intervalo de un día
                $sql = "SELECT
                            coalesce(codigo_caseta,0) as codigo_caseta
                        from
                            garita_entrada_salida_control
                        where
                            lower(trim(dni_conductor)) = lower(trim(:p_dni)) and tipo = 'E'  and fecha_entrada_salida >= CURRENT_DATE - INTERVAL '1 DAY'
                        order by
                            fecha_entrada_salida desc
                        limit 1;";

                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_dni", $resultado["dni_conductor"]);
                $sentencia->execute();
                if (!$sentencia->rowCount()) {
                    $resultado["codigo_caseta"] = 0;
                } else {
                    $resultadoCaseta = $sentencia->fetch(PDO::FETCH_ASSOC);
                    $resultado["codigo_caseta"] = $resultadoCaseta["codigo_caseta"];
                }
                // búsqueda en la caseta en el intervalo de un día
            }
            $resultado["tipo_entrada"] = "P";

            return $resultado;
        } catch (Exception $exc) {
            throw $exc;
        }
    }

    public function obtenerDatosQrEntradaConductor($codigo, $codigo_empresa, $tipo_busqueda){
        $this->dblink->beginTransaction();
        try{
            $sql = "SELECT
                        upper(trim(c.dni_conductor)) as dni_conductor,
                        upper(v.placa) as placa,
                        c.codigo_tipo_personal as codigo_categoria_personal,
                        v.codigo_tipo_vehiculo,
                        e.nombre_empresa,
                        upper(c.apellidos_paternos_conductor||' '||c.apellidos_maternos_conductor||', '||c.nombre_conductor) as nombre_conductor,
                        dt.estado as estado
                    FROM
                        detalle_transporte as dt inner join conductor_control as c on(dt.codigo_conductor = c.codigo_conductor)
                        inner join vehiculo as v on (dt.codigo_vehiculo = v.codigo_vehiculo)
                        inner join empresa_control as e on(dt.codigo_empresa = e.codigo_empresa)
                    WHERE ";
            if ($tipo_busqueda == 'C'){
                $sql .= " dt.codigo_conductor = :codigo ";
            }else{
                $sql .= " dt.codigo_vehiculo = :codigo ";
            }
            $sql .= " and dt.codigo_empresa = :empresa
                    ORDER BY
                        dt.codigo_detalle_transporte desc
                    LIMIT
                        1;";
            $sentencia = $this->dblink->prepare($sql);
            if ($tipo_busqueda == 'C') {
                $sentencia->bindParam(":codigo", $codigo);
            } else {
                $sentencia->bindParam(":codigo", $codigo);
            }
            $sentencia->bindParam(":empresa", $codigo_empresa);
            $sentencia->execute();
            if (!$sentencia->rowCount()) {
                throw new Exception("Este código QR no es válido o ha sido revocado");
            } else {
                $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
                //Búsqueda de datos de fundo_entrada_control previos
                $sql= "SELECT
                            f.procedencia_fundo,
                            (coalesce (f.fecha_manual, f.fecha_entrada_fundo)) as fecha_entrada_fundo,
                            f.hora_manual,
                            (case coalesce(f.numero_asientos, '0') when '' then '0' else coalesce(f.numero_asientos, '0') end) as numero_asientos
                        From
                            fundo_entrada_control as f
                        WHERE
                            f.dni_conductor = :p_dni
                        ORDER BY
                            2 desc, 3 desc;";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_dni", $resultado["dni_conductor"]);
                $sentencia->execute();
                if (!$sentencia->rowCount()) {
                    $resultado["procedencia_fundo"] = "";
                    $resultado["fecha_entrada_fundo"] = "";
                    $resultado["hora_manual"] = "";
                    $resultado["numero_asientos"] = 0;
                }else{
                    $resultadoFundo = $sentencia->fetch(PDO::FETCH_ASSOC);
                    $resultado["procedencia_fundo"] = $resultadoFundo["procedencia_fundo"];
                    $resultado["fecha_entrada_fundo"] = $resultadoFundo["fecha_entrada_fundo"];
                    $resultado["hora_manual"] = $resultadoFundo["hora_manual"];
                    $resultado["numero_asientos"] = $resultadoFundo["numero_asientos"];
                }
                //Búsqueda de datos de fundo_entrada_control previos

                // búsqueda en la caseta en el intervalo de un día
                $sql = "SELECT
                            coalesce(codigo_caseta,0) as codigo_caseta
                        from
                            garita_entrada_salida_control
                        where
                            lower(trim(dni_conductor)) = lower(trim(:p_dni)) and tipo = 'E'  and fecha_entrada_salida >= CURRENT_DATE - INTERVAL '1 DAY'
                        order by
                            fecha_entrada_salida desc
                        limit 1;";

                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_dni", $resultado["dni_conductor"]);
                $sentencia->execute();
                if (!$sentencia->rowCount()) {
                    $resultado["codigo_caseta"] = 0;
                } else {
                    $resultadoCaseta = $sentencia->fetch(PDO::FETCH_ASSOC);
                    $resultado["codigo_caseta"] = $resultadoCaseta["codigo_caseta"];
                }
                // búsqueda en la caseta en el intervalo de un día
                $this->dblink->commit();
            }

            return $resultado;
        } catch (Exception $exc) {
            $this->dblink->rollBack();
            throw $exc;
        }
    }

    public function obtenerDatosQRProveedorEmpresa($codigo_conductor, $codigo_proveedor, $codigo_empresa, $codigo_vehiculo){
        $this->dblink->beginTransaction();
        try{
            $sql = "SELECT
                        upper(trim(c.dni_conductor)) as dni_conductor,
                        upper(v.placa) as placa,
                        c.codigo_tipo_personal as codigo_categoria_personal,
                        v.codigo_tipo_vehiculo,
                        v.capacidad as numero_asientos,
                        e.razon_social as nombre_empresa,
                        upper(c.apellidos_paternos_conductor||' '||c.apellidos_maternos_conductor||', '||c.nombre_conductor) as nombre_conductor,
                        rt.nombre_ruta as procedencia_fundo,
                        dt.codigo_ruta_transporte,
                        dt.codigo_proveedor,
                        dt.codigo_proveedor_empresa,
                        dt.estado as estado
                    FROM
                        proveedor_empresa as dt inner join conductor_control as c on(dt.codigo_conductor = c.codigo_conductor)
                        inner join vehiculo as v on (dt.codigo_vehiculo = v.codigo_vehiculo)
                        inner join proveedor as e on(dt.codigo_proveedor = e.codigo_proveedor)
                        inner join ruta_transporte as rt on(dt.codigo_ruta_transporte = rt.codigo_ruta_transporte)
                    WHERE
                        dt.codigo_conductor = ? and
                        dt.codigo_proveedor = ? and
                        dt.codigo_empresa = ? and
                        dt.codigo_vehiculo = ?
                    ORDER BY
                        dt.codigo_proveedor_empresa desc
                    LIMIT
                        1;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(1, $codigo_conductor);
            $sentencia->bindParam(2, $codigo_proveedor);
            $sentencia->bindParam(3, $codigo_empresa);
            $sentencia->bindParam(4, $codigo_vehiculo);
            $sentencia->execute();
            if (!$sentencia->rowCount()) {
                throw new Exception("Este código QR no es válido o ha sido revocado");
            } else {
                $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
                //Búsqueda de datos de fundo_entrada_control previos
                $sql= "SELECT
                            (coalesce (f.fecha_manual, f.fecha_entrada_fundo)) as fecha_entrada_fundo,
                            f.hora_manual
                        From
                            fundo_entrada_control as f
                        WHERE
                            f.dni_conductor = :p_dni
                        ORDER BY
                            1 desc, 2 desc;";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_dni", $resultado["dni_conductor"]);
                $sentencia->execute();
                if (!$sentencia->rowCount()) {
                    $resultado["fecha_entrada_fundo"] = "";
                    $resultado["hora_manual"] = "";
                }else{
                    $resultadoFundo = $sentencia->fetch(PDO::FETCH_ASSOC);
                    $resultado["fecha_entrada_fundo"] = $resultadoFundo["fecha_entrada_fundo"];
                    $resultado["hora_manual"] = $resultadoFundo["hora_manual"];
                }
                //Búsqueda de datos de fundo_entrada_control previos

                // búsqueda en la caseta en el intervalo de un día
                $sql = "SELECT
                            coalesce(codigo_caseta,0) as codigo_caseta
                        from
                            garita_entrada_salida_control
                        where
                            lower(trim(dni_conductor)) = lower(trim(:p_dni)) and tipo = 'E'  and fecha_entrada_salida >= CURRENT_DATE - INTERVAL '1 DAY'
                        order by
                            fecha_entrada_salida desc
                        limit 1;";

                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_dni", $resultado["dni_conductor"]);
                $sentencia->execute();
                if (!$sentencia->rowCount()) {
                    $resultado["codigo_caseta"] = 0;
                } else {
                    $resultadoCaseta = $sentencia->fetch(PDO::FETCH_ASSOC);
                    $resultado["codigo_caseta"] = $resultadoCaseta["codigo_caseta"];
                }
                // búsqueda en la caseta en el intervalo de un día
                $this->dblink->commit();
            }
            $resultado["tipo_entrada"] = "P";
            return $resultado;
        } catch (Exception $exc) {
            $this->dblink->rollBack();
            throw $exc;
        }
    }

    public function obtenerDatosQrEntradaProveedor($codigo, $codigo_proveedor){
        $this->dblink->beginTransaction();
        try{
            $sql = "SELECT
                        upper(trim(c.dni_conductor)) as dni_conductor,
                        upper(v.placa) as placa,
                        c.codigo_tipo_personal as codigo_categoria_personal,
                        v.codigo_tipo_vehiculo,
                        e.razon_social as nombre_empresa,
                        upper(c.apellidos_paternos_conductor||' '||c.apellidos_maternos_conductor||', '||c.nombre_conductor) as nombre_conductor,
                        dt.codigo_proveedor,
                        dt.estado as estado
                    FROM
                        proveedor_entrada as dt inner join conductor_control as c on(dt.codigo_conductor = c.codigo_conductor)
                        inner join vehiculo as v on (dt.codigo_vehiculo = v.codigo_vehiculo)
                        inner join proveedor as e on(dt.codigo_proveedor = e.codigo_proveedor)
                    WHERE
                        dt.codigo_conductor = ? and
                        dt.codigo_proveedor = ?
                    ORDER BY
                        dt.codigo_proveedor_entrada desc
                    LIMIT
                        1;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(1, $codigo);
            $sentencia->bindParam(2, $codigo_proveedor);
            $sentencia->execute();
            if (!$sentencia->rowCount()) {
                throw new Exception("Este código QR no es válido o ha sido revocado");
            } else {
                $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
                //Búsqueda de datos de fundo_entrada_control previos
                $sql= "SELECT
                            (coalesce (f.fecha_manual, f.fecha_entrada_fundo)) as fecha_entrada_fundo,
                            f.hora_manual
                        From
                            fundo_entrada_control as f
                        WHERE
                            f.dni_conductor = :p_dni
                        ORDER BY
                            1 desc, 2 desc;";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_dni", $resultado["dni_conductor"]);
                $sentencia->execute();
                if (!$sentencia->rowCount()) {
                    $resultado["fecha_entrada_fundo"] = "";
                    $resultado["hora_manual"] = "";
                }else{
                    $resultadoFundo = $sentencia->fetch(PDO::FETCH_ASSOC);
                    $resultado["fecha_entrada_fundo"] = $resultadoFundo["fecha_entrada_fundo"];
                    $resultado["hora_manual"] = $resultadoFundo["hora_manual"];
                }
                //Búsqueda de datos de fundo_entrada_control previos

                // búsqueda en la caseta en el intervalo de un día
                $sql = "SELECT
                            coalesce(codigo_caseta,0) as codigo_caseta
                        from
                            garita_entrada_salida_control
                        where
                            lower(trim(dni_conductor)) = lower(trim(:p_dni)) and tipo = 'E'  and fecha_entrada_salida >= CURRENT_DATE - INTERVAL '1 DAY'
                        order by
                            fecha_entrada_salida desc
                        limit 1;";

                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_dni", $resultado["dni_conductor"]);
                $sentencia->execute();
                if (!$sentencia->rowCount()) {
                    $resultado["codigo_caseta"] = 0;
                } else {
                    $resultadoCaseta = $sentencia->fetch(PDO::FETCH_ASSOC);
                    $resultado["codigo_caseta"] = $resultadoCaseta["codigo_caseta"];
                }
                // búsqueda en la caseta en el intervalo de un día
                $this->dblink->commit();
            }
            $resultado["tipo_entrada"] = "P";
            return $resultado;
        } catch (Exception $exc) {
            $this->dblink->rollBack();
            throw $exc;
        }
    }

    public function registrarEntrada($dni, $placa, $hora_servidor, $hora_manual, $nombre_conductor, $numero_ocupantes, $numero_asientos, $codigo_empresa_sesion, $codigo_usuario_sesion, $codigo_tipo_vehiculo, $codigo_tipo_persona, $procedencia_fundo, $motivo_fundo, $nombre_empresa, $fecha_entrada_fundo, $codigo_caseta, $fecha_manual, $codigo_off_line, $detalle_visitas, $orientacion, $pin, $codigo_maximo, $detalle_manos_obra, $mano_obra, $codigo_proveedor, $codigo_ruta_transporte, $codigo_proveedor_empresa){
        $this->dblink->beginTransaction();
        try {
            $holderDni = trim($dni);
            // Aqui se agrega el trace de la ruta
            $sql = "SELECT codigo_ruta as id FROM ruta WHERE dni = :p_dni and estado = 'A';";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $holderDni);
            $sentencia->execute();
            $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);

            $codigo_ruta = "";

            if ($sentencia->rowCount() === 0) {
                $inicioRuta = "F#".$codigo_empresa_sesion;
                $sql = "INSERT INTO ruta(dni, lugar_creacion) values(:p_dni, :p_lugar);";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_dni", $holderDni);
                $sentencia->bindParam(":p_lugar", $inicioRuta);
                $sentencia->execute();

                $sql = "SELECT codigo_ruta as id FROM ruta WHERE dni = :p_dni and estado = 'A';";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_dni", $holderDni);
                $sentencia->execute();
                $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
            }

            $codigo_ruta = $resultado['id'];

            $sql = "SELECT * FROM f_generar_correlativo('fundo_entrada_control') AS nc";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->execute();
            $resultado = $sentencia->fetch();

            if ($sentencia->rowCount()) {
                $nuevoNumeroFundo = $resultado["nc"];

                $sql = "INSERT INTO fundo_entrada_control(
                                    codigo_entrada_fundo,
                                    placa_entrada_fundo,
                                    hora_entrada_fundo,
                                    hora_manual,
                                    dni_conductor,
                                    nombre_conductor,
                                    numero_ocupantes,
                                    codigo_empresa,
                                    codigo_usuario,
                                    codigo_tipo_vehiculo,
                                    codigo_categoria_personal,
                                    entrada_salida_fundo,
                                    numero_asientos,
                                    procedencia_fundo,
                                    motivo_fundo,
                                    nombre_empresa,
                                    fecha_entrada_fundo,
                                    codigo_ruta,
                                    codigo_caseta,
                                    fecha_manual,
                                    codigo_off_line,
                                    orientacion,
                                    pin,
                                    mano_obra,
                                    codigo_proveedor,
                                    codigo_ruta_transporte,
                                    codigo_proveedor_empresa
                        )VALUES (
                                    :p_codigo_entrada_fundo,
                                    :p_placa_entrada_fundo,
                                    :p_hora_entrada_fundo,
                                    :p_hora_manual,
                                    :p_dni_conductor,
                                    :p_nombre_conductor,
                                    :p_numero_ocupantes,
                                    :p_codigo_empresa,
                                    :p_codigo_usuario,
                                    :p_codigo_tipo_vehiculo,
                                    :p_codigo_categoria_personal,
                                    'E',
                                    :p_numero_asientos,
                                    :p_procedencia_fundo,
                                    :p_motivo_fundo,
                                    :p_nombre_empresa,
                                    :p_fecha_entrada_fundo,
                                    :p_codigo_ruta,
                                    :p_codigo_caseta,
                                    :fecha_manual,
                                    :codigo_off_line,
                                    :orientacion,
                                    :pin,
                                    :mano_obra,
                                    :codigo_proveedor,
                                    :codigo_ruta_transporte,
                                    :codigo_proveedor_empresa);";

                //Preparar la sentencia
                $sentencia = $this->dblink->prepare($sql);

                //Asignar un valor a cada parametro
                $sentencia->bindParam(":p_codigo_entrada_fundo", $nuevoNumeroFundo);
                $sentencia->bindParam(":p_placa_entrada_fundo", $placa);
                $sentencia->bindParam(":p_hora_entrada_fundo", $hora_servidor);
                $sentencia->bindParam(":p_hora_manual", $hora_manual);
                $sentencia->bindParam(":p_dni_conductor", $dni);
                $sentencia->bindParam(":p_nombre_conductor", $nombre_conductor);
                $sentencia->bindParam(":p_numero_ocupantes", $numero_ocupantes);
                $sentencia->bindParam(":p_codigo_empresa",  $codigo_empresa_sesion);
                $sentencia->bindParam(":p_codigo_usuario",  $codigo_usuario_sesion);
                $sentencia->bindParam(":p_codigo_tipo_vehiculo", $codigo_tipo_vehiculo);
                $sentencia->bindParam(":p_codigo_categoria_personal", $codigo_tipo_persona);
                $sentencia->bindParam(":p_numero_asientos", $numero_asientos);
                $sentencia->bindParam(":p_procedencia_fundo", $procedencia_fundo);
                $sentencia->bindParam(":p_motivo_fundo", $motivo_fundo);
                $sentencia->bindParam(":p_nombre_empresa", $nombre_empresa);
                $sentencia->bindParam(":p_fecha_entrada_fundo", $fecha_entrada_fundo);
                $sentencia->bindParam(":p_codigo_ruta", $codigo_ruta);
                $sentencia->bindParam(":p_codigo_caseta", $codigo_caseta);
                $sentencia->bindParam(":fecha_manual", $fecha_manual);
                $sentencia->bindParam(":codigo_off_line", $codigo_off_line);
                $sentencia->bindParam(":orientacion", $orientacion);
                $sentencia->bindParam(":pin", $pin);
                $sentencia->bindParam(":mano_obra", $mano_obra);
                $sentencia->bindParam(":codigo_proveedor", $codigo_proveedor);
                $sentencia->bindParam(":codigo_ruta_transporte", $codigo_ruta_transporte);
                $sentencia->bindParam(":codigo_proveedor_empresa", $codigo_proveedor_empresa);
                //Ejecutar la sentencia preparada
                $sentencia->execute();

                foreach( $detalle_manos_obra as $manoObraEntrada){
                    $codigoTipoManoObra = $manoObraEntrada["p_codigo_mano_obra"];
                    $codigo_off_line = $manoObraEntrada["p_codigo_off"];
                    $codigo_off_line_fundo = $manoObraEntrada["p_codigo_off_fundo"];

                    $sql = "INSERT INTO
                                    fundo_entrada_control_tipo_mano_obra
                            values(:codigo_entrada_fundo,
                                    :codigo_tipo_mano_obra, :numero_trabajadores, :codigo_offline,
                                    :codigo_offline_fundo);";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":codigo_entrada_fundo", $nuevoNumeroFundo);
                    $sentencia->bindParam(":codigo_tipo_mano_obra", $codigoTipoManoObra);
                    $sentencia->bindParam(":numero_trabajadores", $numero_ocupantes);
                    $sentencia->bindParam(":codigo_offline", $codigo_off_line);
                    $sentencia->bindParam(":codigo_offline_fundo", $codigo_off_line_fundo);
                    $sentencia->execute();
                }

                foreach ($detalle_visitas as $visita) {
                    $sql = "SELECT * FROM f_generar_correlativo('visita_fundos') AS nc";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->execute();
                    $resultado = $sentencia->fetch();
                    $codigo_visita = $resultado["nc"];
                    $dni = $visita["p_dni_visita"];
                    $codigo_off_line_visita = $visita["p_codigo_off"];
                    $comentario = "";

                    $sql = "SELECT
                                (nombre_conductor||','||apellidos_maternos_conductor||' '||apellidos_paternos_conductor) as nombre_completo
                            FROM
                                conductor_control
                            WHERE
                                lower(dni_conductor) = :p_dni;";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_dni", $dni);
                    $sentencia->execute();
                    $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
                    $nombre_visita = "";
                    if(isset($resultado["nombre_completo"])){
                        $nombre_visita = $resultado["nombre_completo"];
                    }else{
                        if(strlen($dni) == 8){
                            $json = '{CODDNI: "' . $dni . '"}';
                            $ch = curl_init();
                            curl_setopt($ch, CURLOPT_URL, CONSULTA_JNE);
                            curl_setopt($ch, CURLOPT_POST, 1);
                            curl_setopt($ch, CURLOPT_POSTFIELDS, $json);
                            curl_setopt($ch, CURLOPT_HEADER, TRUE);
                            curl_setopt($ch, CURLOPT_HTTPHEADER, array("Content-Type: application/json", "RequestVerificationToken: " . TOKEN_JNE));
                            curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);

                            $head = curl_exec($ch);
                            $codigoHttpd = curl_getinfo($ch, CURLINFO_HTTP_CODE);
                            curl_close($ch);

                            $porciones = explode("GMT", $head);
                            $ultimo_elemento = count($porciones) - 1;
                            $respuesta = json_decode($porciones[$ultimo_elemento], TRUE);

                            if ($codigoHttpd == 200) {
                                $respuesta = explode("|", $respuesta["data"]);
                                if ($respuesta[0] == "" && $respuesta[1] == "" && $respuesta[2] == "") {
                                    $nombre_visita = "";
                                }else{
                                    $nombre_visita = $respuesta[0] . " " . $respuesta[1] . ", " . $respuesta[2];
                                }
                            } else {
                                $nombre_visita = "";
                            }
                        }else{
                            $nombre_visita = "";
                        }
                    }

                    $sql = "INSERT INTO
                                    visita_fundos(codigo_visita_fundo, codigo_entrada_fundo, dni, nombre, comentario, codigo_off_line, codigo_off_line_fundo)
                                    values(:p_cod_visita, :p_cod_entrada, :p_dni, :p_nombre, :p_comentario, :codigo_off_line, :codigo_off_line_fundo);";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_cod_visita", $codigo_visita);
                    $sentencia->bindParam(":p_cod_entrada", $nuevoNumeroFundo);
                    $sentencia->bindParam(":p_dni", $dni);
                    $sentencia->bindParam(":p_nombre", $nombre_visita);
                    $sentencia->bindParam(":p_comentario", $comentario);
                    $sentencia->bindParam(":codigo_off_line", $codigo_off_line_visita);
                    $sentencia->bindParam(":codigo_off_line_fundo", $codigo_off_line);
                    $sentencia->execute();

                    $sql = "UPDATE correlativo_control SET numero = numero + 1 WHERE tabla = 'visita_fundos'";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->execute();
                }

                $sql = "UPDATE correlativo_control SET numero = numero + 1 WHERE tabla = 'fundo_entrada_control'";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->execute();

                $sql = "SELECT * from alertas_empresa where dni = :p_dni and codigo_empresa = :p_codigo_empresa;";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_dni", $holderDni);
                $sentencia->bindParam(":p_codigo_empresa", $codigo_empresa_sesion);
                $sentencia->execute();

                $resultado = $sentencia->fetchAll();

                if ($sentencia->rowCount()) {
                    foreach ($resultado as $key => $value) {
                        $telefono = $value['telefono'];
                        $mensaje = date('h:i A') . ' ' . $value['mensaje'];

                        $url_holder_sms = str_replace(' ', '%20', "http://servicio.smsmasivos.com.ar/enviar_sms.asp?api=1&usuario=PROOLMOS&clave=PROOLMOS352&tos=" . $telefono . "&texto=" . $mensaje);
                        $ch = curl_init();
                        curl_setopt($ch, CURLOPT_URL, $url_holder_sms);
                        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
                        $data = curl_exec($ch);
                        curl_close($ch);
                    }
                }


                $this->dblink->commit();

                $sql = "SELECT
                        f.codigo_entrada_fundo,
                        to_char(f.fecha_entrada_fundo,'DD/MM/YYYY') AS fecha_entrada,
                        coalesce(to_char(f.fecha_manual,'DD/MM/YYYY'),to_char(f.fecha_entrada_fundo,'DD/MM/YYYY')) AS fecha_manual,
                        to_char(f.hora_entrada_fundo, 'HH24:MI') AS hora_entrada,
                        coalesce(f.dni_conductor, 'sin-dni') as dni,
                        coalesce(f.nombre_conductor, 'sin-nombre') as conductor,
                        coalesce(f.codigo_categoria_personal, 0) as codigo_categoria_personal,
                        UPPER(coalesce(f.nombre_empresa, 'sin-nombre')) as nombre_empresa,
                        coalesce(f.codigo_tipo_vehiculo, 0) as codigo_tipo_vehiculo,
                        UPPER(coalesce(f.placa_entrada_fundo, 'sin-placa')) as placa,
                        upper(coalesce(f.procedencia_fundo,'sin-procedencia')) as procedencia_fundo,
                        (case coalesce(f.numero_asientos, '0') when '' then '0' else coalesce(f.numero_asientos, '0') end) as numero_asientos,
                        (case coalesce(f.numero_ocupantes, '0') when '' then '0' else coalesce(f.numero_ocupantes, '0') end) as numero_ocupantes,
                        upper(coalesce(f.motivo_fundo, '')) as motivo_fundo,
                        coalesce(f.contenedor, '') as contenedor,
                        f.entrada_salida_fundo,
                        f.codigo_empresa,
                        f.codigo_usuario,
                        f.estado_entrada_fundo,
                        to_char(f.hora_manual, 'HH24:MI') AS hora_manual,
                        coalesce(f.codigo_ruta, 0) as codigo_ruta,
                        coalesce(f.codigo_entrada, 0) as codigo_entrada,
                        coalesce(f.codigo_caseta, 0) as codigo_caseta,
                        coalesce(f.codigo_off_line, '') as codigo_off,
                        coalesce(f.orientacion, '') as orientacion,
                        coalesce(f.pin, '') as pin,
                        coalesce(f.mano_obra, '') as mano_obra,
                        coalesce(f.codigo_proveedor, 0) as codigo_proveedor,
                        coalesce(f.codigo_ruta_transporte, 0) as codigo_ruta_transporte,
                        coalesce(f.codigo_proveedor_empresa, 0) as codigo_proveedor_empresa,
                        (select count(*) from visita_fundos as v where v.codigo_entrada_fundo = f.codigo_entrada_fundo) as visitas,
                        (select count(*) from fundo_entrada_control_tipo_mano_obra as fectmo where fectmo.codigo_entrada_fundo = f.codigo_entrada_fundo) as num_manos_obra
                    from fundo_entrada_control as f
                    where
                        f.codigo_entrada_fundo >= :codigo_entrada_fundo and f.estado_entrada_fundo='E' and f.codigo_empresa = :codigo_empresa_sesion order by 2 desc;";
                $sentencia = $this->dblink->prepare($sql);
                if ($codigo_maximo != 0 && $codigo_maximo < $nuevoNumeroFundo) {
                    $nuevoNumeroFundo = $codigo_maximo;
                }
                $sentencia->bindParam(":codigo_entrada_fundo", $nuevoNumeroFundo);
                $sentencia->bindParam(":codigo_empresa_sesion", $codigo_empresa_sesion);
                $sentencia->execute();
                $resultado = $sentencia->fetchAll(PDO::FETCH_ASSOC);

                foreach ($resultado as $item) {
                    $id_entrada = $item["codigo_entrada_fundo"];
                    $visitas = $item["visitas"];
                    if ($visitas > 0) {
                        $sql = "SELECT * FROM visita_fundos where codigo_entrada_fundo = :p_codigo_entrada_fundo";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->bindParam(":p_codigo_entrada_fundo", $id_entrada);
                        $sentencia->execute();
                        $item["visitas_lista"] = $sentencia->fetchAll(PDO::FETCH_ASSOC);
                    } else {
                        $item["visitas_lista"] = [];
                    }

                    $num_manos_obra = $item["num_manos_obra"];
                    if($num_manos_obra > 0 ){
                        $sql = "SELECT
                                    codigo_entrada_fundo as codigo_entrada,
                                    codigo_tipo_mano_obra as codigo_mano_obra,
                                    numero_trabajadores,
                                    coalesce(codigo_off_line, '') as off_line,
                                    coalesce(codigo_off_line_fundo, '') as off_line_fundo
                                FROM fundo_entrada_control_tipo_mano_obra where codigo_entrada_fundo = :p_codigo_entrada_fundo";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->bindParam(":p_codigo_entrada_fundo", $id_entrada);
                        $sentencia->execute();
                        $item["manos_obra_lista"] = $sentencia->fetchAll(PDO::FETCH_ASSOC);
                    }else{
                        $item["manos_obra_lista"] = [];
                    }

                    $arrayResultado[] = $item;
                }
            return $arrayResultado;

            } else {
                throw new Exception("No se a configurado el correlativo para FUNDO");
            }
        } catch (Exception $exc) {
            $this->dblink->rollBack();
            throw  $exc;
        }
        return false;
    }

    public function registrarSalida($dni, $placa_entrada, $hora_entrada, $nombre_conductor, $num_ocupantes, $cod_empresa, $cod_usuario, $nro_asientos, $procedencia_fundo, $contenedor_salida, $nombre_empresa, $hora_manual, $fecha_entrada_fundo, $fecha_manual, $codigo_off_line, $pin, $codigo_maximo, $codigo_proveedor, $codigo_ruta_transporte, $codigo_proveedor_empresa){
        $this->dblink->beginTransaction();
        try {
            $holderDni = trim($dni);
            $sql = "SELECT codigo_ruta as id FROM ruta WHERE dni = :p_dni and estado = 'A';";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $holderDni);
            $sentencia->execute();
            $resultado = $sentencia->fetch();
            $codigo_ruta = $resultado['id'];

            $sql = "SELECT codigo_entrada_fundo as cod_ent FROM fundo_entrada_control WHERE dni_conductor = :p_dni and entrada_salida_fundo = 'E' order by codigo_entrada_fundo desc limit 1;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $dni);
            $sentencia->execute();
            $resultado = $sentencia->fetch();
            if(isset($resultado['cod_ent'])){
                $codigo_entrada = $resultado['cod_ent'];
            }

            $sql = "SELECT * FROM f_generar_correlativo('fundo_entrada_control') AS nc";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->execute();
            $resultado = $sentencia->fetch();

            if ($sentencia->rowCount()){
                $nuevoNumeroFundo = $resultado["nc"];

                $sql = "INSERT INTO fundo_entrada_control(
                                        codigo_entrada_fundo,
                                        placa_entrada_fundo,
                                        hora_entrada_fundo,
                                        dni_conductor,
                                        nombre_conductor,
                                        numero_ocupantes,
                                        codigo_empresa,
                                        codigo_usuario,
                                        entrada_salida_fundo,
                                        numero_asientos,
                                        procedencia_fundo,
                                        contenedor,
                                        nombre_empresa,
                                        hora_manual,
                                        fecha_entrada_fundo,
                                        codigo_ruta,
                                        codigo_entrada,
                                        fecha_manual,
                                        codigo_off_line,
                                        pin,
                                        codigo_proveedor,
                                        codigo_ruta_transporte,
                                        codigo_proveedor_empresa
                                    ) VALUES (
                                        :p_codigo_entrada_fundo,
                                        :p_placa_entrada_fundo,
                                        :p_hora_entrada_fundo,
                                        :p_dni_conductor,
                                        :p_nombre_conductor,
                                        :p_numero_ocupantes,
                                        :p_codigo_empresa,
                                        :p_codigo_usuario,
                                        'S',
                                        :p_numero_asientos,
                                        :p_procedencia_fundo,
                                        :p_contenedor,
                                        :p_nombre_empresa,
                                        :p_hora_manual,
                                        :p_fecha_entrada_fundo,
                                        :p_codigo_ruta,
                                        :p_codigo_entrada,
                                        :fecha_manual,
                                        :codigo_off_line,
                                        :pin,
                                        :codigo_proveedor,
                                        :codigo_ruta_transporte,
                                        :codigo_proveedor_empresa
                                    );";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_codigo_entrada_fundo", $nuevoNumeroFundo);
                $sentencia->bindParam(":p_placa_entrada_fundo",$placa_entrada);
                $sentencia->bindParam(":p_hora_entrada_fundo",  $hora_entrada);
                $sentencia->bindParam(":p_dni_conductor", $dni);
                $sentencia->bindParam(":p_nombre_conductor", $nombre_conductor);
                $sentencia->bindParam(":p_numero_ocupantes", $num_ocupantes);
                $sentencia->bindParam(":p_codigo_empresa",  $cod_empresa);
                $sentencia->bindParam(":p_codigo_usuario",  $cod_usuario);
                $sentencia->bindParam(":p_numero_asientos", $nro_asientos);
                $sentencia->bindParam(":p_procedencia_fundo",$procedencia_fundo);
                $sentencia->bindParam(":p_contenedor", $contenedor_salida);
                $sentencia->bindParam(":p_nombre_empresa",  $nombre_empresa);
                $sentencia->bindParam(":p_hora_manual",  $hora_manual);
                $sentencia->bindParam(":p_fecha_entrada_fundo",  $fecha_entrada_fundo);
                $sentencia->bindParam(":p_codigo_ruta",  $codigo_ruta);
                $sentencia->bindParam(":p_codigo_entrada",  $codigo_entrada);
                $sentencia->bindParam(":fecha_manual",  $fecha_manual);
                $sentencia->bindParam(":codigo_off_line",  $codigo_off_line);
                $sentencia->bindParam(":pin",  $pin);
                $sentencia->bindParam(":codigo_proveedor", $codigo_proveedor);
                $sentencia->bindParam(":codigo_ruta_transporte", $codigo_ruta_transporte);
                $sentencia->bindParam(":codigo_proveedor_empresa", $codigo_proveedor_empresa);
                $sentencia->execute();

                $sql = "UPDATE correlativo_control set numero= numero + 1 where tabla = 'fundo_entrada_control'";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->execute();

                if($contenedor_salida === 'C'){
                    $sql = "SELECT
                                ec.telefono,
                                e.nombre_empresa as empresa
                            FROM
                                empresa_contacto AS ec inner join
                                empresa_control as e on(e.codigo_empresa = ec.codigo_empresa)
                            WHERE
                                ec.codigo_empresa = :p_codigo_empresa;";

                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_codigo_empresa", $cod_empresa);
                    $sentencia->execute();
                    $resultado = $sentencia->fetchAll();

                    if($sentencia->rowCount()){

                        foreach($resultado as $item){
                            $telefono = $item["telefono"];
                            $nombreEmpresaSMS = $item["empresa"];
                            $texto = $nombreEmpresaSMS . ': Contenedor con placa ' . $placa_entrada . ", chofer: " . $nombre_conductor . ". Salio el " . $fecha_entrada_fundo . " a las " . $hora_entrada;

                            $url_holder_sms = str_replace(' ', '%20',"http://servicio.smsmasivos.com.ar/enviar_sms.asp?api=1&usuario=PROOLMOS&clave=PROOLMOS352&tos=".$telefono."&texto=".$texto);
                            $ch = curl_init();
                            curl_setopt($ch, CURLOPT_URL, $url_holder_sms);
                            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                            curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
                            $data = curl_exec($ch);
                            curl_close($ch);

                        }

                    }
                }

                $this->dblink->commit();

                $sql = "SELECT
                            f.codigo_entrada_fundo,
                            to_char(f.fecha_entrada_fundo,'DD/MM/YYYY') AS fecha_entrada,
                            coalesce(to_char(f.fecha_manual,'DD/MM/YYYY'),to_char(f.fecha_entrada_fundo,'DD/MM/YYYY')) AS fecha_manual,
                            to_char(f.hora_entrada_fundo, 'HH24:MI') AS hora_entrada,
                            coalesce(f.dni_conductor, 'sin-dni') as dni,
                            coalesce(f.nombre_conductor, 'sin-nombre') as conductor,
                            coalesce(f.codigo_categoria_personal, 0) as codigo_categoria_personal,
                            UPPER(coalesce(f.nombre_empresa, 'sin-nombre')) as nombre_empresa,
                            coalesce(f.codigo_tipo_vehiculo, 0) as codigo_tipo_vehiculo,
                            UPPER(coalesce(f.placa_entrada_fundo, 'sin-placa')) as placa,
                            upper(coalesce(f.procedencia_fundo,'sin-procedencia')) as procedencia_fundo,
                            (case coalesce(f.numero_asientos, '0') when '' then '0' else coalesce(f.numero_asientos, '0') end) as numero_asientos,
                            (case coalesce(f.numero_ocupantes, '0') when '' then '0' else coalesce(f.numero_ocupantes, '0') end) as numero_ocupantes,
                            upper(coalesce(f.motivo_fundo, '')) as motivo_fundo,
                            coalesce(f.contenedor, '') as contenedor,
                            f.entrada_salida_fundo,
                            f.codigo_empresa,
                            f.codigo_usuario,
                            f.estado_entrada_fundo,
                            to_char(f.hora_manual, 'HH24:MI') AS hora_manual,
                            coalesce(f.codigo_ruta, 0) as codigo_ruta,
                            coalesce(f.codigo_entrada, 0) as codigo_entrada,
                            coalesce(f.codigo_caseta, 0) as codigo_caseta,
                            coalesce(f.codigo_off_line, '') as codigo_off,
                            coalesce(f.orientacion, '') as orientacion,
                            coalesce(f.pin, '') as pin,
                            coalesce(f.mano_obra, '') as mano_obra,
                            coalesce(f.codigo_proveedor, 0) as codigo_proveedor,
                            coalesce(f.codigo_ruta_transporte, 0) as codigo_ruta_transporte,
                            coalesce(f.codigo_proveedor_empresa, 0) as codigo_proveedor_empresa,
                            (select count(*) from visita_fundos as v where v.codigo_entrada_fundo = f.codigo_entrada_fundo) as visitas,
                            (select count(*) from fundo_entrada_control_tipo_mano_obra as fectmo where fectmo.codigo_entrada_fundo = f.codigo_entrada_fundo) as num_manos_obra
                        from fundo_entrada_control as f
                        where
                            f.codigo_entrada_fundo >= :codigo_entrada_fundo and f.estado_entrada_fundo='E' and f.codigo_empresa = :codigo_empresa_sesion order by 2 desc;";
                    $sentencia = $this->dblink->prepare($sql);
                    if ($codigo_maximo != 0 && $codigo_maximo < $nuevoNumeroFundo) {
                        $nuevoNumeroFundo = $codigo_maximo;
                    }
                    $sentencia->bindParam(":codigo_entrada_fundo", $nuevoNumeroFundo);
                    $sentencia->bindParam(":codigo_empresa_sesion", $cod_empresa);
                    $sentencia->execute();
                    $resultado = $sentencia->fetchAll(PDO::FETCH_ASSOC);

                    foreach ($resultado as $item) {
                        $id_entrada = $item["codigo_entrada_fundo"];
                        $visitas = $item["visitas"];
                        if ($visitas > 0) {
                            $sql = "SELECT * FROM visita_fundos where codigo_entrada_fundo = :p_codigo_entrada_fundo";
                            $sentencia = $this->dblink->prepare($sql);
                            $sentencia->bindParam(":p_codigo_entrada_fundo", $id_entrada);
                            $sentencia->execute();
                            $item["visitas_lista"] = $sentencia->fetchAll(PDO::FETCH_ASSOC);
                        } else {
                            $item["visitas_lista"] = [];
                        }

                        $num_manos_obra = $item["num_manos_obra"];
                        if($num_manos_obra > 0 ){
                            $sql = "SELECT
                                        codigo_entrada_fundo as codigo_entrada,
                                        codigo_tipo_mano_obra as codigo_mano_obra,
                                        numero_trabajadores,
                                        coalesce(codigo_off_line, '') as off_line,
                                        coalesce(codigo_off_line_fundo, '') as off_line_fundo
                                    FROM fundo_entrada_control_tipo_mano_obra where codigo_entrada_fundo = :p_codigo_entrada_fundo";
                            $sentencia = $this->dblink->prepare($sql);
                            $sentencia->bindParam(":p_codigo_entrada_fundo", $id_entrada);
                            $sentencia->execute();
                            $item["manos_obra_lista"] = $sentencia->fetchAll(PDO::FETCH_ASSOC);
                        }else{
                            $item["manos_obra_lista"] = [];
                        }
                        $arrayResultado[] = $item;
                    }
                return $arrayResultado;
            }  else {
                throw new Exception("No se a configurado el correlativo para FUNDO PRO OLMOS");
            }
        }  catch (Exception $exc){
            $this->dblink->rollBack();
            throw  $exc;
        }
        return false;
    }

    public function sincronizarFundo($detalle, $codigo_max_fundo, $lista_completa, $empresa, $detalleVisita, $detalleManoObra){
        $this->dblink->beginTransaction();
        try{
            foreach ($detalle as $fundo) {
                $fecha_entrada_servidor = date("Y-m-d");
                $hora_entrada_servidor = date("H:i:s");
                $dni_conductor = $fundo["dni_conductor"];
                $nombre_conductor = $fundo["nombre_conductor"];
                $codigo_cat_pers = $fundo["codigo_categoria_personal"];
                $nomb_empresa = $fundo["nombre_empresa"];
                $cod_tip_vehi = $fundo["codigo_tipo_vehiculo"];
                $placa = $fundo["placa_entrada_fundo"];
                $procedencia = $fundo["procedencia_fundo"];
                $numero_asientos = empty($fundo["numero_asientos"]) ? 0 : $fundo["numero_asientos"];
                $numero_ocupantes = empty($fundo["numero_ocupantes"]) ? 0: $fundo["numero_ocupantes"];
                $motivo = $fundo["motivo_fundo"];
                $contenedor = $fundo["contenedor"];
                $entrada_salida_fundo = $fundo["entrada_salida_fundo"];
                $cod_usuario = $fundo["codigo_usuario"];
                $hora_manual = $fundo["hora_manual"];
                $cod_ruta = empty($fundo["codigo_ruta"]) ? 0 : $fundo["codigo_ruta"];
                $codigo_entrada_fundo =  empty($fundo["codigo_entrada"]) ? 0 : $fundo["codigo_entrada"];
                $fecha_manual = date_format(date_create_from_format('d/m/Y', $fundo["fecha_manual"]), 'Y-m-d'); //convierto la fecha de formato dia/mes/año a formato SQL (Año-mes-día)
                $cod_off_line = "";
                if (isset($fundo["codigo_off_line"])) {
                    $cod_off_line = $fundo["codigo_off_line"];
                }

                $cod_caseta = 0;
                if(isset($fundo["codigo_caseta"])){
                    $cod_caseta = $fundo["codigo_caseta"];
                }

                $orientacion = "";
                if (isset($fundo["orientacion"])) {
                    $orientacion = $fundo["orientacion"];
                }
                $pin = "--------";
                if(isset($fundo["pin"])){
                    $pin = $fundo["pin"];
                }

                $mano_obra = "N";
                if(isset($fundo["mano_obra"])){
                    $mano_obra = $fundo["mano_obra"];
                }

                $codigo_proveedor = 0;
                if(isset($fundo["codigo_proveedor"])){
                    $codigo_proveedor = $fundo["codigo_proveedor"];
                }
                $codigo_ruta_transporte = 0;
                if(isset($fundo["codigo_ruta_transporte"])){
                    $codigo_ruta_transporte = $fundo["codigo_ruta_transporte"];
                }
                $codigo_proveedor_empresa = 0;
                if(isset($fundo["codigo_proveedor_empresa"])){
                    $codigo_proveedor_empresa = $fundo["codigo_proveedor_empresa"];
                }
                //primero consultamos la entrada
                $timestamp_manual = $fecha_manual." ".$hora_manual;

                $sql = "SELECT * from fundo_entrada_control where codigo_off_line = ?";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(1,$cod_off_line);
                $sentencia->execute();

                if(!$sentencia->rowCount()){
                    $sql = "SELECT
                                codigo_ruta,
                                (fecha_entrada_salida ||' ' || hora_entrada_salida)::TIMESTAMP as dt
                            from
                                garita_entrada_salida_control
                            where dni_conductor = :dni and tipo = 'E' and
                            (fecha_entrada_salida ||' ' || hora_entrada_salida)::TIMESTAMP <= :timestamp_manual and (fecha_entrada_salida ||' ' || hora_entrada_salida)::TIMESTAMP >= :timestamp_manual::TIMESTAMP - INTERVAL '5 hours'  and codigo_ruta is not null  order by 2 desc limit 1;";

                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":dni",$dni_conductor);
                    $sentencia->bindParam(":timestamp_manual",$timestamp_manual);
                    $sentencia->execute();

                    if($sentencia->rowCount()){ //Si encontró la entrada en la garita, entonces recojo el código de ruta
                        $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
                        $cod_ruta = $resultado["codigo_ruta"];
                    }else{ //sino creo una nueva entrada con el código de ruta creada desde el fundo
                        $lugar_creacion = "F#".$empresa;
                        $sql = "INSERT INTO ruta(dni, lugar_creacion) values(:p_dni, :p_lugar);";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->bindParam(":p_dni", $dni_conductor);
                        $sentencia->bindParam(":p_lugar", $lugar_creacion);
                        $sentencia->execute();
                        $cod_ruta = $this->dblink->lastInsertId();
                        //con lastInsertId obtengo el código de ruta creado en el insert
                    }

                    $sql = "SELECT * FROM f_generar_correlativo('fundo_entrada_control') AS nc";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->execute();
                    $resultado = $sentencia->fetch();

                    if ($sentencia->rowCount()) {
                        $codigo_entrada = $resultado["nc"];

                        if($entrada_salida_fundo == "E"){ //agregamos la entrada
                            $sql = "INSERT INTO fundo_entrada_control(
                                        codigo_entrada_fundo,
                                        placa_entrada_fundo,
                                        hora_entrada_fundo,
                                        hora_manual,
                                        dni_conductor,
                                        nombre_conductor,
                                        numero_ocupantes,
                                        codigo_empresa,
                                        codigo_usuario,
                                        codigo_tipo_vehiculo,
                                        codigo_categoria_personal,
                                        entrada_salida_fundo,
                                        numero_asientos,
                                        procedencia_fundo,
                                        motivo_fundo,
                                        nombre_empresa,
                                        fecha_entrada_fundo,
                                        codigo_ruta,
                                        codigo_caseta,
                                        fecha_manual,
                                        codigo_off_line,
                                        orientacion,
                                        pin,
                                        mano_obra,
                                        codigo_proveedor,
                                        codigo_ruta_transporte,
                                        codigo_proveedor_empresa
                            )VALUES (
                                        :p_codigo_entrada_fundo,
                                        :p_placa_entrada_fundo,
                                        :p_hora_entrada_fundo,
                                        :p_hora_manual,
                                        :p_dni_conductor,
                                        :p_nombre_conductor,
                                        :p_numero_ocupantes,
                                        :p_codigo_empresa,
                                        :p_codigo_usuario,
                                        :p_codigo_tipo_vehiculo,
                                        :p_codigo_categoria_personal,
                                        :p_entrada_salida_fundo,
                                        :p_numero_asientos,
                                        :p_procedencia_fundo,
                                        :p_motivo_fundo,
                                        :p_nombre_empresa,
                                        :p_fecha_entrada_fundo,
                                        :p_codigo_ruta,
                                        :p_codigo_caseta,
                                        :fecha_manual,
                                        :codigo_off_line,
                                        :orientacion,
                                        :pin,
                                        :mano_obra,
                                        :codigo_proveedor,
                                        :codigo_ruta_transporte,
                                        :codigo_proveedor_empresa );";

                            //Preparar la sentencia
                            $sentencia = $this->dblink->prepare($sql);

                            //Asignar un valor a cada parametro
                            $sentencia->bindParam(":p_codigo_entrada_fundo", $codigo_entrada);
                            $sentencia->bindParam(":p_placa_entrada_fundo", $placa);
                            $sentencia->bindParam(":p_hora_entrada_fundo", $hora_entrada_servidor);
                            $sentencia->bindParam(":p_hora_manual", $hora_manual);
                            $sentencia->bindParam(":p_dni_conductor", $dni_conductor);
                            $sentencia->bindParam(":p_nombre_conductor", $nombre_conductor);
                            $sentencia->bindParam(":p_numero_ocupantes", $numero_ocupantes);
                            $sentencia->bindParam(":p_codigo_empresa",  $empresa);
                            $sentencia->bindParam(":p_codigo_usuario",  $cod_usuario);
                            $sentencia->bindParam(":p_codigo_tipo_vehiculo", $cod_tip_vehi);
                            $sentencia->bindParam(":p_codigo_categoria_personal", $codigo_cat_pers);
                            $sentencia->bindParam(":p_entrada_salida_fundo", $entrada_salida_fundo);
                            $sentencia->bindParam(":p_numero_asientos", $numero_asientos);
                            $sentencia->bindParam(":p_procedencia_fundo", $procedencia);
                            $sentencia->bindParam(":p_motivo_fundo", $motivo);
                            $sentencia->bindParam(":p_nombre_empresa", $nomb_empresa);
                            $sentencia->bindParam(":p_fecha_entrada_fundo", $fecha_entrada_servidor);
                            $sentencia->bindParam(":p_codigo_ruta", $cod_ruta);
                            $sentencia->bindParam(":p_codigo_caseta", $cod_caseta);
                            $sentencia->bindParam(":fecha_manual", $fecha_manual);
                            $sentencia->bindParam(":codigo_off_line", $cod_off_line);
                            $sentencia->bindParam(":orientacion", $orientacion);
                            $sentencia->bindParam(":pin", $pin);
                            $sentencia->bindParam(":mano_obra", $mano_obra);
                            $sentencia->bindParam(":codigo_proveedor", $codigo_proveedor);
                            $sentencia->bindParam(":codigo_ruta_transporte", $codigo_ruta_transporte);
                            $sentencia->bindParam(":codigo_proveedor_empresa", $codigo_proveedor_empresa);
                            //Ejecutar la sentencia preparada
                            $sentencia->execute();

                            $sql = "SELECT * from alertas_empresa where dni = :p_dni and codigo_empresa = :p_codigo_empresa;";
                            $sentencia = $this->dblink->prepare($sql);
                            $sentencia->bindParam(":p_dni", $dni_conductor);
                            $sentencia->bindParam(":p_codigo_empresa", $empresa);
                            $sentencia->execute();

                            $resultado = $sentencia->fetchAll();

                            if ($sentencia->rowCount()) {
                                foreach ($resultado as $key => $value) {
                                    $telefono = $value['telefono'];
                                    $mensaje = date('h:i A') . ' ' . $value['mensaje'];

                                    $url_holder_sms = str_replace(' ', '%20', "http://servicio.smsmasivos.com.ar/enviar_sms.asp?api=1&usuario=PROOLMOS&clave=PROOLMOS352&tos=" . $telefono . "&texto=" . $mensaje);
                                    $ch = curl_init();
                                    curl_setopt($ch, CURLOPT_URL, $url_holder_sms);
                                    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                                    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
                                    $data = curl_exec($ch);
                                    curl_close($ch);
                                }
                            }
                        }else{ //agregamos la salida
                            $sql = "SELECT codigo_entrada_fundo as cod_ent FROM fundo_entrada_control WHERE dni_conductor = :p_dni and entrada_salida_fundo = 'E' order by codigo_entrada_fundo desc limit 1;";
                            $sentencia = $this->dblink->prepare($sql);
                            $sentencia->bindParam(":p_dni", $dni_conductor);
                            $sentencia->execute();
                            $resultado = $sentencia->fetch();
                            if (isset($resultado['cod_ent'])) {
                                $codigo_entrada_fundo = $resultado['cod_ent'];
                            }

                            $sql = "INSERT INTO fundo_entrada_control(
                                            codigo_entrada_fundo,
                                            placa_entrada_fundo,
                                            hora_entrada_fundo,
                                            dni_conductor,
                                            nombre_conductor,
                                            numero_ocupantes,
                                            codigo_empresa,
                                            codigo_usuario,
                                            entrada_salida_fundo,
                                            numero_asientos,
                                            procedencia_fundo,
                                            contenedor,
                                            nombre_empresa,
                                            hora_manual,
                                            fecha_entrada_fundo,
                                            codigo_ruta,
                                            codigo_entrada,
                                            fecha_manual,
                                            codigo_off_line,
                                            pin,
                                            codigo_proveedor,
                                            codigo_ruta_transporte,
                                            codigo_proveedor_empresa
                                        ) VALUES (
                                            :p_codigo_entrada_fundo,
                                            :p_placa_entrada_fundo,
                                            :p_hora_entrada_fundo,
                                            :p_dni_conductor,
                                            :p_nombre_conductor,
                                            :p_numero_ocupantes,
                                            :p_codigo_empresa,
                                            :p_codigo_usuario,
                                            'S',
                                            :p_numero_asientos,
                                            :p_procedencia_fundo,
                                            :p_contenedor,
                                            :p_nombre_empresa,
                                            :p_hora_manual,
                                            :p_fecha_entrada_fundo,
                                            :p_codigo_ruta,
                                            :p_codigo_entrada,
                                            :fecha_manual,
                                            :codigo_off_line,
                                            :pin,
                                            :codigo_proveedor,
                                            :codigo_ruta_transporte,
                                            :codigo_proveedor_empresa
                                        );";
                            $sentencia = $this->dblink->prepare($sql);
                            $sentencia->bindParam(":p_codigo_entrada_fundo", $codigo_entrada);
                            $sentencia->bindParam(":p_placa_entrada_fundo", $placa);
                            $sentencia->bindParam(":p_hora_entrada_fundo",  $hora_entrada_servidor);
                            $sentencia->bindParam(":p_dni_conductor", $dni_conductor);
                            $sentencia->bindParam(":p_nombre_conductor", $nombre_conductor);
                            $sentencia->bindParam(":p_numero_ocupantes", $numero_ocupantes);
                            $sentencia->bindParam(":p_codigo_empresa",  $empresa);
                            $sentencia->bindParam(":p_codigo_usuario",  $cod_usuario);
                            $sentencia->bindParam(":p_numero_asientos", $numero_asientos);
                            $sentencia->bindParam(":p_procedencia_fundo", $procedencia);
                            $sentencia->bindParam(":p_contenedor", $contenedor);
                            $sentencia->bindParam(":p_nombre_empresa",  $nomb_empresa);
                            $sentencia->bindParam(":p_hora_manual",  $hora_manual);
                            $sentencia->bindParam(":p_fecha_entrada_fundo",  $fecha_entrada_servidor);
                            $sentencia->bindParam(":p_codigo_ruta",  $cod_ruta);
                            $sentencia->bindParam(":p_codigo_entrada",  $codigo_entrada_fundo);
                            $sentencia->bindParam(":fecha_manual",  $fecha_manual);
                            $sentencia->bindParam(":codigo_off_line",  $cod_off_line);
                            $sentencia->bindParam(":pin",  $pin);
                            $sentencia->bindParam(":codigo_proveedor", $codigo_proveedor);
                            $sentencia->bindParam(":codigo_ruta_transporte", $codigo_ruta_transporte);
                            $sentencia->bindParam(":codigo_proveedor_empresa", $codigo_proveedor_empresa);

                            $sentencia->execute();
                        }

                        $sql = "UPDATE correlativo_control set numero= numero + 1 where tabla = 'fundo_entrada_control'";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->execute();
                    }
                }
            }


            // RECORREMOS EL ARRAY DETALLE DE VISITAS
            $comentarioVisita = "";
            foreach($detalleVisita as $visita){
                $dniVisita = $visita["dni"];
                $cod_off_line_visita = "";
                if (isset($visita["codigo_off_line"])) {
                    $cod_off_line_visita = $visita["codigo_off_line"];
                }

                $sql = "SELECT codigo_entrada_fundo from fundo_entrada_control where codigo_off_line = :codigo_off_line";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":codigo_off_line", $cod_off_line_visita);
                $sentencia->execute();

                if($sentencia->rowCount()){
                    $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
                    $codigo_entrada_fundo_visita = $resultado["codigo_entrada_fundo"];
                    //INSERTAMOS LAS VISITAS EN EL FUNDO CORRESPONDIENTE
                    $sql = "SELECT * FROM f_generar_correlativo('visita_fundos') AS nc";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->execute();
                    $resultado = $sentencia->fetch();
                    $codigo_visita = $resultado["nc"];

                    $sql = "SELECT
                                (nombre_conductor||','||apellidos_maternos_conductor||' '||apellidos_paternos_conductor) as nombre_completo
                            FROM
                                conductor_control
                            WHERE
                                lower(dni_conductor) = :p_dni;";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_dni", $dniVisita);
                    $sentencia->execute();
                    $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
                    $nombre_visita = "";
                    if(isset($resultado["nombre_completo"])){
                        $nombre_visita = $resultado["nombre_completo"];
                    }else{
                        if(strlen($dniVisita) == 8){
                            $json = '{CODDNI: "' . $dniVisita . '"}';
                            $ch = curl_init();
                            curl_setopt($ch, CURLOPT_URL, CONSULTA_JNE);
                            curl_setopt($ch, CURLOPT_POST, 1);
                            curl_setopt($ch, CURLOPT_POSTFIELDS, $json);
                            curl_setopt($ch, CURLOPT_HEADER, TRUE);
                            curl_setopt($ch, CURLOPT_HTTPHEADER, array("Content-Type: application/json", "RequestVerificationToken: " . TOKEN_JNE));
                            curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);

                            $head = curl_exec($ch);
                            $codigoHttpd = curl_getinfo($ch, CURLINFO_HTTP_CODE);
                            curl_close($ch);

                            $porciones = explode("GMT", $head);
                            $ultimo_elemento = count($porciones) - 1;
                            $respuesta = json_decode($porciones[$ultimo_elemento], TRUE);

                            if ($codigoHttpd == 200) {
                                $respuesta = explode("|", $respuesta["data"]);
                                if ($respuesta[0] == "" && $respuesta[1] == "" && $respuesta[2] == "") {
                                    $nombre_visita = "";
                                }else{
                                    $nombre_visita = $respuesta[0] . " " . $respuesta[1] . ", " . $respuesta[2];
                                }
                            } else {
                                $nombre_visita = "";
                            }
                        }else{
                            $nombre_visita = "";
                        }
                    }

                    $sql = "INSERT INTO
                                    visita_fundos(codigo_visita_fundo, codigo_entrada_fundo, dni, nombre, comentario, codigo_off_line, codigo_off_line_fundo)
                                    values(:p_cod_visita, :p_cod_entrada, :p_dni, :p_nombre, :p_comentario, :codigo_off_line, :codigo_off_line_fundo);";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_cod_visita", $codigo_visita);
                    $sentencia->bindParam(":p_cod_entrada", $codigo_entrada_fundo_visita);
                    $sentencia->bindParam(":p_dni", $dniVisita);
                    $sentencia->bindParam(":p_nombre", $nombre_visita);
                    $sentencia->bindParam(":p_comentario", $comentarioVisita);
                    $sentencia->bindParam(":codigo_off_line", $cod_off_line_visita);
                    $sentencia->bindParam(":codigo_off_line_fundo", $cod_off_line_visita);
                    $sentencia->execute();

                    $sql = "UPDATE correlativo_control SET numero = numero + 1 WHERE tabla = 'visita_fundos'";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->execute();
                    //INSERTAMOS LAS VISITAS EN EL FUNDO CORRESPONDIENTE
                }
            }
            // RECORREMOS EL ARRAY DETALLE DE VISITAS

            // RECORREMOS EL ARRAY DETALLE DE MANO DE OBRA
            foreach($detalleManoObra as $manoObra){
                $cod_off_line_mano_obra = "";
                if (isset($manoObra["codigo_off_line"])) {
                    $cod_off_line_mano_obra = $manoObra["codigo_off_line"];
                }


                $cod_off_line_fundo_mano_obra = "";
                if (isset($manoObra["codigo_off_line_fundo"])) {
                    $cod_off_line_fundo_mano_obra = $manoObra["codigo_off_line_fundo"];
                }

                $numero_trabajadores = $manoObra["numero_trabajadores"];
                $codigo_tipo_mano_obra = $manoObra["codigo_tipo_mano_obra"];

                $sql = "SELECT codigo_entrada_fundo from fundo_entrada_control where codigo_off_line = :codigo_off_line";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":codigo_off_line", $cod_off_line_fundo_mano_obra);
                $sentencia->execute();

                if($sentencia->rowCount()){
                    $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
                    $codigo_entrada_fundo_mano_obra = $resultado["codigo_entrada_fundo"];

                    //VALIDAMOS QUE NO ESTÉ EN EL SERVIDOR PARA INSERTARLA
                    $sql = "SELECT
                                *
                            from
                                fundo_entrada_control_tipo_mano_obra
                            where
                                codigo_entrada_fundo = :codigo_entrada_fundo and codigo_tipo_mano_obra = :codigo_tipo_mano_obra";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":codigo_entrada_fundo", $codigo_entrada_fundo_mano_obra);
                    $sentencia->bindParam(":codigo_tipo_mano_obra", $codigo_tipo_mano_obra);
                    $sentencia->execute();
                    //VALIDAMOS QUE NO ESTÉ EN EL SERVIDOR PARA INSERTARLA
                    if(!$sentencia->rowCount()){
                        //INSERTAMOS LA MANO DE OBRA EN EL FUNDO CORRESPONDIENTE
                        $sql = "INSERT INTO
                                        fundo_entrada_control_tipo_mano_obra
                                        (codigo_entrada_fundo, codigo_tipo_mano_obra, numero_trabajadores, codigo_off_line, codigo_off_line_fundo)
                                values
                                    (:codigo_entrada_fundo, :codigo_tipo_mano_obra, :numero_trabajadores, :codigo_off_line, :codigo_off_line_fundo);";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->bindParam(":codigo_entrada_fundo", $codigo_entrada_fundo_mano_obra);
                        $sentencia->bindParam(":codigo_tipo_mano_obra", $codigo_tipo_mano_obra);
                        $sentencia->bindParam(":numero_trabajadores", $numero_trabajadores);
                        $sentencia->bindParam(":codigo_off_line", $cod_off_line_mano_obra);
                        $sentencia->bindParam(":codigo_off_line_fundo", $cod_off_line_fundo_mano_obra);
                        $sentencia->execute();
                        //INSERTAMOS LA MANO DE OBRA EN EL FUNDO CORRESPONDIENTE
                    }
                }
            }
            // RECORREMOS EL ARRAY DETALLE DE MANO DE OBRA

            $sql = "SELECT
                        f.codigo_entrada_fundo,
                        to_char(f.fecha_entrada_fundo,'DD/MM/YYYY') AS fecha_entrada,
                        coalesce(to_char(f.fecha_manual,'DD/MM/YYYY'),to_char(f.fecha_entrada_fundo,'DD/MM/YYYY')) AS fecha_manual,
                        to_char(f.hora_entrada_fundo, 'HH24:MI') AS hora_entrada,
                        coalesce(f.dni_conductor, 'sin-dni') as dni,
                        coalesce(f.nombre_conductor, 'sin-nombre') as conductor,
                        coalesce(f.codigo_categoria_personal, 0) as codigo_categoria_personal,
                        UPPER(coalesce(f.nombre_empresa, 'sin-nombre')) as nombre_empresa,
                        coalesce(f.codigo_tipo_vehiculo, 0) as codigo_tipo_vehiculo,
                        UPPER(coalesce(f.placa_entrada_fundo, 'sin-placa')) as placa,
                        upper(
                            case coalesce(f.procedencia_fundo,'') when '' then 'sin-procedencia' else f.procedencia_fundo end
                            ) as procedencia_fundo,
                        (
                            case coalesce(f.numero_asientos,'') when '' then '0' else f.numero_asientos end
                        ) as numero_asientos,
                        (
                            case coalesce(f.numero_ocupantes,'') when '' then '0' else f.numero_ocupantes end
                        ) as numero_ocupantes,
                        upper(coalesce(f.motivo_fundo, '')) as motivo_fundo,
                        coalesce(f.contenedor, '') as contenedor,
                        f.entrada_salida_fundo,
                        f.codigo_empresa,
                        f.codigo_usuario,
                        f.estado_entrada_fundo,
                        to_char(f.hora_manual, 'HH24:MI') AS hora_manual,
                        coalesce(f.codigo_ruta, 0) as codigo_ruta,
                        coalesce(f.codigo_entrada, 0) as codigo_entrada,
                        coalesce(f.codigo_caseta, 0) as codigo_caseta,
                        coalesce(f.codigo_off_line, '') as codigo_off,
                        coalesce(f.orientacion, '') as orientacion,
                        coalesce(f.pin, '') as pin,
                        coalesce(f.mano_obra, '') as mano_obra,
                        coalesce(f.codigo_proveedor, 0) as codigo_proveedor,
                        coalesce(f.codigo_ruta_transporte, 0) as codigo_ruta_transporte,
                        coalesce(f.codigo_proveedor_empresa, 0) as codigo_proveedor_empresa,
                        (select count(*) from visita_fundos as v where v.codigo_entrada_fundo = f.codigo_entrada_fundo) as visitas,
                        (select count(*) from fundo_entrada_control_tipo_mano_obra as fectmo where fectmo.codigo_entrada_fundo = f.codigo_entrada_fundo) as num_manos_obra
                    from fundo_entrada_control as f where f.estado_entrada_fundo='E' and f.codigo_empresa = :codigo_empresa ";
            if($lista_completa == "N"){
                $sql .= " and f.codigo_entrada_fundo > :maximo ";
            }else{
                $sql .= " and f.fecha_entrada_fundo >= CURRENT_DATE - INTERVAL '30 DAYS' ";
            }

            $sql .= " order by 2 desc;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":codigo_empresa", $empresa);
            if($lista_completa == "N"){
                $sentencia->bindParam(":maximo", $codigo_max_fundo);
            }
            $sentencia->execute();

            if (!$sentencia->rowCount()) {
                return [];
            }

            $resultado = $sentencia->fetchAll(PDO::FETCH_ASSOC);

            foreach ($resultado as $item) {
                $id_entrada = $item["codigo_entrada_fundo"];
                $visitas = $item["visitas"];
                if ($visitas > 0) {
                    $sql = "SELECT * FROM visita_fundos where codigo_entrada_fundo = :p_codigo_entrada_fundo";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_codigo_entrada_fundo", $id_entrada);
                    $sentencia->execute();
                    $item["visitas_lista"] = $sentencia->fetchAll(PDO::FETCH_ASSOC);
                } else {
                    $item["visitas_lista"] = [];
                }

                $num_manos_obra = $item["num_manos_obra"];
                if($num_manos_obra > 0 ){
                    $sql = "SELECT
                                codigo_entrada_fundo as codigo_entrada,
                                codigo_tipo_mano_obra as codigo_mano_obra,
                                numero_trabajadores,
                                coalesce(codigo_off_line, '') as off_line,
                                coalesce(codigo_off_line_fundo, '') as off_line_fundo
                            FROM fundo_entrada_control_tipo_mano_obra where codigo_entrada_fundo = :p_codigo_entrada_fundo";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_codigo_entrada_fundo", $id_entrada);
                    $sentencia->execute();
                    $item["manos_obra_lista"] = $sentencia->fetchAll(PDO::FETCH_ASSOC);
                }else{
                    $item["manos_obra_lista"] = [];
                }

                $arrayResultado[] = $item;
            }
            $this->dblink->commit();
            return $arrayResultado;
        }catch(Exception $e){
            $this->dblink->rollBack();
            throw $e;
        }
    }

    public function sincronizarSoloFundo($detalle, $empresa, $detalleVisita, $detalleManoObra){
        $this->dblink->beginTransaction();
        try{
            $sincronizados = 0; $noSincronizados = 0; $arrayNoSincronizado = array();
            foreach ($detalle as $fundo) {
                $fecha_entrada_servidor = date("Y-m-d");
                $hora_entrada_servidor = date("H:i:s");
                $dni_conductor = $fundo["dni_conductor"];
                $nombre_conductor = $fundo["nombre_conductor"];
                $codigo_cat_pers = $fundo["codigo_categoria_personal"];
                $nomb_empresa = $fundo["nombre_empresa"];
                $cod_tip_vehi = $fundo["codigo_tipo_vehiculo"];
                $placa = $fundo["placa_entrada_fundo"];
                $procedencia = $fundo["procedencia_fundo"];
                $numero_asientos = empty($fundo["numero_asientos"]) ? 0 : $fundo["numero_asientos"];
                $numero_ocupantes = empty($fundo["numero_ocupantes"]) ? 0: $fundo["numero_ocupantes"];
                $motivo = $fundo["motivo_fundo"];
                $contenedor = $fundo["contenedor"];
                $entrada_salida_fundo = $fundo["entrada_salida_fundo"];
                $cod_usuario = $fundo["codigo_usuario"];
                $hora_manual = $fundo["hora_manual"];
                $cod_ruta = empty($fundo["codigo_ruta"]) ? 0 : $fundo["codigo_ruta"];
                $codigo_entrada_fundo =  empty($fundo["codigo_entrada"]) ? 0 : $fundo["codigo_entrada"];
                $fecha_manual = date_format(date_create_from_format('d/m/Y', $fundo["fecha_manual"]), 'Y-m-d'); //convierto la fecha de formato dia/mes/año a formato SQL (Año-mes-día)
                $cod_off_line = "";
                if (isset($fundo["codigo_off_line"])) {
                    $cod_off_line = $fundo["codigo_off_line"];
                }

                $cod_caseta = 0;
                if(isset($fundo["codigo_caseta"])){
                    $cod_caseta = $fundo["codigo_caseta"];
                }

                $orientacion = "";
                if (isset($fundo["orientacion"])) {
                    $orientacion = $fundo["orientacion"];
                }
                $pin = "--------";
                if(isset($fundo["pin"])){
                    $pin = $fundo["pin"];
                }

                $mano_obra = "N";
                if(isset($fundo["mano_obra"])){
                    $mano_obra = $fundo["mano_obra"];
                }

                $codigo_proveedor = 0;
                if(isset($fundo["codigo_proveedor"])){
                    $codigo_proveedor = $fundo["codigo_proveedor"];
                }
                $codigo_ruta_transporte = 0;
                if(isset($fundo["codigo_ruta_transporte"])){
                    $codigo_ruta_transporte = $fundo["codigo_ruta_transporte"];
                }
                $codigo_proveedor_empresa = 0;
                if(isset($fundo["codigo_proveedor_empresa"])){
                    $codigo_proveedor_empresa = $fundo["codigo_proveedor_empresa"];
                }
                //primero consultamos la entrada
                $timestamp_manual = $fecha_manual." ".$hora_manual;

                $sql = "SELECT * from fundo_entrada_control where codigo_off_line = ?";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(1,$cod_off_line);
                $sentencia->execute();


                if($sentencia->rowCount()){
                    $noSincronizados++;
                    $arrayNoSincronizado[] = $cod_off_line;
                }else{
                    $sincronizados++;
                    $sql = "SELECT
                                codigo_ruta,
                                (fecha_entrada_salida ||' ' || hora_entrada_salida)::TIMESTAMP as dt
                            from
                                garita_entrada_salida_control
                            where dni_conductor = :dni and tipo = 'E' and
                            (fecha_entrada_salida ||' ' || hora_entrada_salida)::TIMESTAMP <= :timestamp_manual and (fecha_entrada_salida ||' ' || hora_entrada_salida)::TIMESTAMP >= :timestamp_manual::TIMESTAMP - INTERVAL '5 hours'  and codigo_ruta is not null  order by 2 desc limit 1;";

                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":dni",$dni_conductor);
                    $sentencia->bindParam(":timestamp_manual",$timestamp_manual);
                    $sentencia->execute();

                    if($sentencia->rowCount()){ //Si encontró la entrada en la garita, entonces recojo el código de ruta
                        $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
                        $cod_ruta = $resultado["codigo_ruta"];
                    }else{ //sino creo una nueva entrada con el código de ruta creada desde el fundo
                        $lugar_creacion = "F#".$empresa;
                        $sql = "INSERT INTO ruta(dni, lugar_creacion) values(:p_dni, :p_lugar);";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->bindParam(":p_dni", $dni_conductor);
                        $sentencia->bindParam(":p_lugar", $lugar_creacion);
                        $sentencia->execute();
                        $cod_ruta = $this->dblink->lastInsertId();
                        //con lastInsertId obtengo el código de ruta creado en el insert
                    }

                    $sql = "SELECT * FROM f_generar_correlativo('fundo_entrada_control') AS nc";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->execute();
                    $resultado = $sentencia->fetch();

                    if ($sentencia->rowCount()) {
                        $codigo_entrada = $resultado["nc"];

                        if($entrada_salida_fundo == "E"){ //agregamos la entrada
                            $sql = "INSERT INTO fundo_entrada_control(
                                        codigo_entrada_fundo,
                                        placa_entrada_fundo,
                                        hora_entrada_fundo,
                                        hora_manual,
                                        dni_conductor,
                                        nombre_conductor,
                                        numero_ocupantes,
                                        codigo_empresa,
                                        codigo_usuario,
                                        codigo_tipo_vehiculo,
                                        codigo_categoria_personal,
                                        entrada_salida_fundo,
                                        numero_asientos,
                                        procedencia_fundo,
                                        motivo_fundo,
                                        nombre_empresa,
                                        fecha_entrada_fundo,
                                        codigo_ruta,
                                        codigo_caseta,
                                        fecha_manual,
                                        codigo_off_line,
                                        orientacion,
                                        pin,
                                        mano_obra,
                                        codigo_proveedor,
                                        codigo_ruta_transporte,
                                        codigo_proveedor_empresa
                            )VALUES (
                                        :p_codigo_entrada_fundo,
                                        :p_placa_entrada_fundo,
                                        :p_hora_entrada_fundo,
                                        :p_hora_manual,
                                        :p_dni_conductor,
                                        :p_nombre_conductor,
                                        :p_numero_ocupantes,
                                        :p_codigo_empresa,
                                        :p_codigo_usuario,
                                        :p_codigo_tipo_vehiculo,
                                        :p_codigo_categoria_personal,
                                        :p_entrada_salida_fundo,
                                        :p_numero_asientos,
                                        :p_procedencia_fundo,
                                        :p_motivo_fundo,
                                        :p_nombre_empresa,
                                        :p_fecha_entrada_fundo,
                                        :p_codigo_ruta,
                                        :p_codigo_caseta,
                                        :fecha_manual,
                                        :codigo_off_line,
                                        :orientacion,
                                        :pin,
                                        :mano_obra,
                                        :codigo_proveedor,
                                        :codigo_ruta_transporte,
                                        :codigo_proveedor_empresa );";

                            //Preparar la sentencia
                            $sentencia = $this->dblink->prepare($sql);

                            //Asignar un valor a cada parametro
                            $sentencia->bindParam(":p_codigo_entrada_fundo", $codigo_entrada);
                            $sentencia->bindParam(":p_placa_entrada_fundo", $placa);
                            $sentencia->bindParam(":p_hora_entrada_fundo", $hora_entrada_servidor);
                            $sentencia->bindParam(":p_hora_manual", $hora_manual);
                            $sentencia->bindParam(":p_dni_conductor", $dni_conductor);
                            $sentencia->bindParam(":p_nombre_conductor", $nombre_conductor);
                            $sentencia->bindParam(":p_numero_ocupantes", $numero_ocupantes);
                            $sentencia->bindParam(":p_codigo_empresa",  $empresa);
                            $sentencia->bindParam(":p_codigo_usuario",  $cod_usuario);
                            $sentencia->bindParam(":p_codigo_tipo_vehiculo", $cod_tip_vehi);
                            $sentencia->bindParam(":p_codigo_categoria_personal", $codigo_cat_pers);
                            $sentencia->bindParam(":p_entrada_salida_fundo", $entrada_salida_fundo);
                            $sentencia->bindParam(":p_numero_asientos", $numero_asientos);
                            $sentencia->bindParam(":p_procedencia_fundo", $procedencia);
                            $sentencia->bindParam(":p_motivo_fundo", $motivo);
                            $sentencia->bindParam(":p_nombre_empresa", $nomb_empresa);
                            $sentencia->bindParam(":p_fecha_entrada_fundo", $fecha_entrada_servidor);
                            $sentencia->bindParam(":p_codigo_ruta", $cod_ruta);
                            $sentencia->bindParam(":p_codigo_caseta", $cod_caseta);
                            $sentencia->bindParam(":fecha_manual", $fecha_manual);
                            $sentencia->bindParam(":codigo_off_line", $cod_off_line);
                            $sentencia->bindParam(":orientacion", $orientacion);
                            $sentencia->bindParam(":pin", $pin);
                            $sentencia->bindParam(":mano_obra", $mano_obra);
                            $sentencia->bindParam(":codigo_proveedor", $codigo_proveedor);
                            $sentencia->bindParam(":codigo_ruta_transporte", $codigo_ruta_transporte);
                            $sentencia->bindParam(":codigo_proveedor_empresa", $codigo_proveedor_empresa);
                            //Ejecutar la sentencia preparada
                            $sentencia->execute();

                            $sql = "SELECT * from alertas_empresa where dni = :p_dni and codigo_empresa = :p_codigo_empresa;";
                            $sentencia = $this->dblink->prepare($sql);
                            $sentencia->bindParam(":p_dni", $dni_conductor);
                            $sentencia->bindParam(":p_codigo_empresa", $empresa);
                            $sentencia->execute();

                            $resultado = $sentencia->fetchAll();

                            if ($sentencia->rowCount()) {
                                foreach ($resultado as $key => $value) {
                                    $telefono = $value['telefono'];
                                    $mensaje = date('h:i A') . ' ' . $value['mensaje'];

                                    $url_holder_sms = str_replace(' ', '%20', "http://servicio.smsmasivos.com.ar/enviar_sms.asp?api=1&usuario=PROOLMOS&clave=PROOLMOS352&tos=" . $telefono . "&texto=" . $mensaje);
                                    $ch = curl_init();
                                    curl_setopt($ch, CURLOPT_URL, $url_holder_sms);
                                    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                                    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
                                    $data = curl_exec($ch);
                                    curl_close($ch);
                                }
                            }
                        }else{ //agregamos la salida
                            $sql = "SELECT codigo_entrada_fundo as cod_ent FROM fundo_entrada_control WHERE dni_conductor = :p_dni and entrada_salida_fundo = 'E' order by codigo_entrada_fundo desc limit 1;";
                            $sentencia = $this->dblink->prepare($sql);
                            $sentencia->bindParam(":p_dni", $dni_conductor);
                            $sentencia->execute();
                            $resultado = $sentencia->fetch();
                            if (isset($resultado['cod_ent'])) {
                                $codigo_entrada_fundo = $resultado['cod_ent'];
                            }

                            $sql = "INSERT INTO fundo_entrada_control(
                                            codigo_entrada_fundo,
                                            placa_entrada_fundo,
                                            hora_entrada_fundo,
                                            dni_conductor,
                                            nombre_conductor,
                                            numero_ocupantes,
                                            codigo_empresa,
                                            codigo_usuario,
                                            entrada_salida_fundo,
                                            numero_asientos,
                                            procedencia_fundo,
                                            contenedor,
                                            nombre_empresa,
                                            hora_manual,
                                            fecha_entrada_fundo,
                                            codigo_ruta,
                                            codigo_entrada,
                                            fecha_manual,
                                            codigo_off_line,
                                            pin,
                                            codigo_proveedor,
                                            codigo_ruta_transporte,
                                            codigo_proveedor_empresa
                                        ) VALUES (
                                            :p_codigo_entrada_fundo,
                                            :p_placa_entrada_fundo,
                                            :p_hora_entrada_fundo,
                                            :p_dni_conductor,
                                            :p_nombre_conductor,
                                            :p_numero_ocupantes,
                                            :p_codigo_empresa,
                                            :p_codigo_usuario,
                                            'S',
                                            :p_numero_asientos,
                                            :p_procedencia_fundo,
                                            :p_contenedor,
                                            :p_nombre_empresa,
                                            :p_hora_manual,
                                            :p_fecha_entrada_fundo,
                                            :p_codigo_ruta,
                                            :p_codigo_entrada,
                                            :fecha_manual,
                                            :codigo_off_line,
                                            :pin,
                                            :codigo_proveedor,
                                            :codigo_ruta_transporte,
                                            :codigo_proveedor_empresa
                                        );";
                            $sentencia = $this->dblink->prepare($sql);
                            $sentencia->bindParam(":p_codigo_entrada_fundo", $codigo_entrada);
                            $sentencia->bindParam(":p_placa_entrada_fundo", $placa);
                            $sentencia->bindParam(":p_hora_entrada_fundo",  $hora_entrada_servidor);
                            $sentencia->bindParam(":p_dni_conductor", $dni_conductor);
                            $sentencia->bindParam(":p_nombre_conductor", $nombre_conductor);
                            $sentencia->bindParam(":p_numero_ocupantes", $numero_ocupantes);
                            $sentencia->bindParam(":p_codigo_empresa",  $empresa);
                            $sentencia->bindParam(":p_codigo_usuario",  $cod_usuario);
                            $sentencia->bindParam(":p_numero_asientos", $numero_asientos);
                            $sentencia->bindParam(":p_procedencia_fundo", $procedencia);
                            $sentencia->bindParam(":p_contenedor", $contenedor);
                            $sentencia->bindParam(":p_nombre_empresa",  $nomb_empresa);
                            $sentencia->bindParam(":p_hora_manual",  $hora_manual);
                            $sentencia->bindParam(":p_fecha_entrada_fundo",  $fecha_entrada_servidor);
                            $sentencia->bindParam(":p_codigo_ruta",  $cod_ruta);
                            $sentencia->bindParam(":p_codigo_entrada",  $codigo_entrada_fundo);
                            $sentencia->bindParam(":fecha_manual",  $fecha_manual);
                            $sentencia->bindParam(":codigo_off_line",  $cod_off_line);
                            $sentencia->bindParam(":pin",  $pin);
                            $sentencia->bindParam(":codigo_proveedor", $codigo_proveedor);
                            $sentencia->bindParam(":codigo_ruta_transporte", $codigo_ruta_transporte);
                            $sentencia->bindParam(":codigo_proveedor_empresa", $codigo_proveedor_empresa);

                            $sentencia->execute();
                        }

                        $sql = "UPDATE correlativo_control set numero= numero + 1 where tabla = 'fundo_entrada_control'";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->execute();
                    }
                }
            }

            $this->dblink->commit();
            $arrayResultado["sincronizados"] = $sincronizados;
            $arrayResultado["noSincronizados"] = $noSincronizados;
            $arrayResultado["codigosNoSincronizados"] = $arrayNoSincronizado;
            return $arrayResultado;
        }catch(Exception $e){
            $this->dblink->rollBack();
            throw $e;
        }
    }


    public function sincAgro($detalle, $empresa){
        $this->dblink->beginTransaction();
        try{
            foreach ($detalle as $fundo) {
                $fecha_entrada_servidor = date("Y-m-d");
                $hora_entrada_servidor = date("H:i:s");
                $dni_conductor = $fundo["dni_conductor"];
                $nombre_conductor = $fundo["nombre_conductor"];
                $codigo_cat_pers = $fundo["codigo_categoria_personal"];
                $nomb_empresa = $fundo["nombre_empresa"];
                $cod_tip_vehi = $fundo["codigo_tipo_vehiculo"];
                $placa = $fundo["placa_entrada_fundo"];
                $procedencia = $fundo["procedencia_fundo"];
                $numero_asientos = $fundo["numero_asientos"];
                $numero_ocupantes = $fundo["numero_ocupantes"];
                $motivo = $fundo["motivo_fundo"];
                $contenedor = $fundo["contenedor"];
                $entrada_salida_fundo = $fundo["entrada_salida_fundo"];
                $cod_usuario = $fundo["codigo_usuario"];
                $hora_manual = $fundo["hora_manual"];
                $cod_ruta = $fundo["codigo_ruta"];
                $codigo_entrada_fundo =  $fundo["codigo_entrada"];
                $fecha_manual = date_format(date_create_from_format('d/m/Y', $fundo["fecha_manual"]), 'Y-m-d'); //convierto la fecha de formato dia/mes/año a formato SQL (Año-mes-día)
                // print_r($fundo);
                // exit;
                $cod_off_line = "";
                if (isset($fundo["codigo_off_line"])) {
                    $cod_off_line = $fundo["codigo_off_line"];
                }

                $cod_caseta = 0;
                if(isset($fundo["codigo_caseta"])){
                    $cod_caseta = $fundo["codigo_caseta"];
                }

                $orientacion = "";
                if (isset($fundo["orientacion"])) {
                    $orientacion = $fundo["orientacion"];
                }
                $pin = "--------";
                if(isset($fundo["pin"])){
                    $pin = $fundo["pin"];
                }

                $mano_obra = "N";
                if(isset($fundo["mano_obra"])){
                    $mano_obra = $fundo["mano_obra"];
                }

                $codigo_proveedor = null;
                if(isset($fundo["codigo_proveedor"])){
                    $codigo_proveedor = $fundo["codigo_proveedor"];
                }
                $codigo_ruta_transporte = null;
                if(isset($fundo["codigo_ruta_transporte"])){
                    $codigo_ruta_transporte = $fundo["codigo_ruta_transporte"];
                }
                $codigo_proveedor_empresa = null;
                if(isset($fundo["codigo_proveedor_empresa"])){
                    $codigo_proveedor_empresa = $fundo["codigo_proveedor_empresa"];
                }

                //primero consultamos la entrada
                $timestamp_manual = $fecha_manual." ".$hora_manual;

                $sql = "SELECT
                            codigo_ruta,
                            (fecha_entrada_salida ||' ' || hora_entrada_salida)::TIMESTAMP as dt
                        from
                            garita_entrada_salida_control
                        where dni_conductor = :dni and tipo = 'E' and
                        (fecha_entrada_salida ||' ' || hora_entrada_salida)::TIMESTAMP <= :timestamp_manual and (fecha_entrada_salida ||' ' || hora_entrada_salida)::TIMESTAMP >= :timestamp_manual::TIMESTAMP - INTERVAL '5 hours'  and codigo_ruta is not null  order by 2 desc limit 1;";

                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":dni",$dni_conductor);
                $sentencia->bindParam(":timestamp_manual",$timestamp_manual);
                $sentencia->execute();

                if($sentencia->rowCount()){ //Si encontró la entrada en la garita, entonces recojo el código de ruta
                    $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
                    $cod_ruta = $resultado["codigo_ruta"];
                }else{ //sino creo una nueva entrada con el código de ruta creada desde el fundo
                    $lugar_creacion = "F#".$empresa;
                    $sql = "INSERT INTO ruta(dni, lugar_creacion) values(:p_dni, :p_lugar);";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_dni", $dni_conductor);
                    $sentencia->bindParam(":p_lugar", $lugar_creacion);
                    $sentencia->execute();
                    $cod_ruta = $this->dblink->lastInsertId();
                    //con lastInsertId obtengo el código de ruta creado en el insert
                }

                $sql = "SELECT * from fundo_entrada_control where codigo_off_line = ?";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(1, $cod_off_line);
                $sentencia->execute();

                if(!$sentencia->rowCount()){ //busco el código off line, si no lo encuentra, lo agrego
                    $sql = "SELECT * FROM f_generar_correlativo('fundo_entrada_control') AS nc";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->execute();
                    $resultado = $sentencia->fetch();

                    if ($sentencia->rowCount()) {
                        $codigo_entrada = $resultado["nc"];

                        if($entrada_salida_fundo == "E"){ //agregamos la entrada
                            $sql = "INSERT INTO fundo_entrada_control(
                                        codigo_entrada_fundo,
                                        placa_entrada_fundo,
                                        hora_entrada_fundo,
                                        hora_manual,
                                        dni_conductor,
                                        nombre_conductor,
                                        numero_ocupantes,
                                        codigo_empresa,
                                        codigo_usuario,
                                        codigo_tipo_vehiculo,
                                        codigo_categoria_personal,
                                        entrada_salida_fundo,
                                        numero_asientos,
                                        procedencia_fundo,
                                        motivo_fundo,
                                        nombre_empresa,
                                        fecha_entrada_fundo,
                                        codigo_ruta,
                                        codigo_caseta,
                                        fecha_manual,
                                        codigo_off_line,
                                        orientacion,
                                        pin,
                                        mano_obra,
                                        codigo_proveedor,
                                        codigo_ruta_transporte,
                                        codigo_proveedor_empresa
                            )VALUES (
                                        :p_codigo_entrada_fundo,
                                        :p_placa_entrada_fundo,
                                        :p_hora_entrada_fundo,
                                        :p_hora_manual,
                                        :p_dni_conductor,
                                        :p_nombre_conductor,
                                        :p_numero_ocupantes,
                                        :p_codigo_empresa,
                                        :p_codigo_usuario,
                                        :p_codigo_tipo_vehiculo,
                                        :p_codigo_categoria_personal,
                                        :p_entrada_salida_fundo,
                                        :p_numero_asientos,
                                        :p_procedencia_fundo,
                                        :p_motivo_fundo,
                                        :p_nombre_empresa,
                                        :p_fecha_entrada_fundo,
                                        :p_codigo_ruta,
                                        :p_codigo_caseta,
                                        :fecha_manual,
                                        :codigo_off_line,
                                        :orientacion,
                                        :pin,
                                        :mano_obra,
                                        :codigo_proveedor,
                                        :codigo_ruta_transporte,
                                        :codigo_proveedor_empresa );";

                            //Preparar la sentencia
                            $sentencia = $this->dblink->prepare($sql);

                            //Asignar un valor a cada parametro
                            $sentencia->bindParam(":p_codigo_entrada_fundo", $codigo_entrada);
                            $sentencia->bindParam(":p_placa_entrada_fundo", $placa);
                            $sentencia->bindParam(":p_hora_entrada_fundo", $hora_entrada_servidor);
                            $sentencia->bindParam(":p_hora_manual", $hora_manual);
                            $sentencia->bindParam(":p_dni_conductor", $dni_conductor);
                            $sentencia->bindParam(":p_nombre_conductor", $nombre_conductor);
                            $sentencia->bindParam(":p_numero_ocupantes", $numero_ocupantes);
                            $sentencia->bindParam(":p_codigo_empresa",  $empresa);
                            $sentencia->bindParam(":p_codigo_usuario",  $cod_usuario);
                            $sentencia->bindParam(":p_codigo_tipo_vehiculo", $cod_tip_vehi);
                            $sentencia->bindParam(":p_codigo_categoria_personal", $codigo_cat_pers);
                            $sentencia->bindParam(":p_entrada_salida_fundo", $entrada_salida_fundo);
                            $sentencia->bindParam(":p_numero_asientos", $numero_asientos);
                            $sentencia->bindParam(":p_procedencia_fundo", $procedencia);
                            $sentencia->bindParam(":p_motivo_fundo", $motivo);
                            $sentencia->bindParam(":p_nombre_empresa", $nomb_empresa);
                            $sentencia->bindParam(":p_fecha_entrada_fundo", $fecha_entrada_servidor);
                            $sentencia->bindParam(":p_codigo_ruta", $cod_ruta);
                            $sentencia->bindParam(":p_codigo_caseta", $cod_caseta);
                            $sentencia->bindParam(":fecha_manual", $fecha_manual);
                            $sentencia->bindParam(":codigo_off_line", $cod_off_line);
                            $sentencia->bindParam(":orientacion", $orientacion);
                            $sentencia->bindParam(":pin", $pin);
                            $sentencia->bindParam(":mano_obra", $mano_obra);
                            $sentencia->bindParam(":codigo_proveedor", $codigo_proveedor);
                            $sentencia->bindParam(":codigo_ruta_transporte", $codigo_ruta_transporte);
                            $sentencia->bindParam(":codigo_proveedor_empresa", $codigo_proveedor_empresa);

                            //Ejecutar la sentencia preparada
                            $sentencia->execute();

                        }else{ //agregamos la salida
                            $sql = "SELECT codigo_entrada_fundo as cod_ent FROM fundo_entrada_control WHERE dni_conductor = :p_dni and entrada_salida_fundo = 'E' order by codigo_entrada_fundo desc limit 1;";
                            $sentencia = $this->dblink->prepare($sql);
                            $sentencia->bindParam(":p_dni", $dni_conductor);
                            $sentencia->execute();
                            $resultado = $sentencia->fetch();
                            if (isset($resultado['cod_ent'])) {
                                $codigo_entrada_fundo = $resultado['cod_ent'];
                            }

                            $sql = "INSERT INTO fundo_entrada_control(
                                            codigo_entrada_fundo,
                                            placa_entrada_fundo,
                                            hora_entrada_fundo,
                                            dni_conductor,
                                            nombre_conductor,
                                            numero_ocupantes,
                                            codigo_empresa,
                                            codigo_usuario,
                                            entrada_salida_fundo,
                                            numero_asientos,
                                            procedencia_fundo,
                                            contenedor,
                                            nombre_empresa,
                                            hora_manual,
                                            fecha_entrada_fundo,
                                            codigo_ruta,
                                            codigo_entrada,
                                            fecha_manual,
                                            codigo_off_line,
                                            pin,
                                            codigo_proveedor,
                                            codigo_ruta_transporte,
                                            codigo_proveedor_empresa
                                        ) VALUES (
                                            :p_codigo_entrada_fundo,
                                            :p_placa_entrada_fundo,
                                            :p_hora_entrada_fundo,
                                            :p_dni_conductor,
                                            :p_nombre_conductor,
                                            :p_numero_ocupantes,
                                            :p_codigo_empresa,
                                            :p_codigo_usuario,
                                            'S',
                                            :p_numero_asientos,
                                            :p_procedencia_fundo,
                                            :p_contenedor,
                                            :p_nombre_empresa,
                                            :p_hora_manual,
                                            :p_fecha_entrada_fundo,
                                            :p_codigo_ruta,
                                            :p_codigo_entrada,
                                            :fecha_manual,
                                            :codigo_off_line,
                                            :pin,
                                            :codigo_proveedor,
                                            :codigo_ruta_transporte,
                                            :codigo_proveedor_empresa
                                        );";
                            $sentencia = $this->dblink->prepare($sql);
                            $sentencia->bindParam(":p_codigo_entrada_fundo", $codigo_entrada);
                            $sentencia->bindParam(":p_placa_entrada_fundo", $placa);
                            $sentencia->bindParam(":p_hora_entrada_fundo",  $hora_entrada_servidor);
                            $sentencia->bindParam(":p_dni_conductor", $dni_conductor);
                            $sentencia->bindParam(":p_nombre_conductor", $nombre_conductor);
                            $sentencia->bindParam(":p_numero_ocupantes", $numero_ocupantes);
                            $sentencia->bindParam(":p_codigo_empresa",  $empresa);
                            $sentencia->bindParam(":p_codigo_usuario",  $cod_usuario);
                            $sentencia->bindParam(":p_numero_asientos", $numero_asientos);
                            $sentencia->bindParam(":p_procedencia_fundo", $procedencia);
                            $sentencia->bindParam(":p_contenedor", $contenedor);
                            $sentencia->bindParam(":p_nombre_empresa",  $nomb_empresa);
                            $sentencia->bindParam(":p_hora_manual",  $hora_manual);
                            $sentencia->bindParam(":p_fecha_entrada_fundo",  $fecha_entrada_servidor);
                            $sentencia->bindParam(":p_codigo_ruta",  $cod_ruta);
                            $sentencia->bindParam(":p_codigo_entrada",  $codigo_entrada_fundo);
                            $sentencia->bindParam(":fecha_manual",  $fecha_manual);
                            $sentencia->bindParam(":codigo_off_line",  $cod_off_line);
                            $sentencia->bindParam(":pin",  $pin);
                            $sentencia->bindParam(":codigo_proveedor", $codigo_proveedor);
                            $sentencia->bindParam(":codigo_ruta_transporte", $codigo_ruta_transporte);
                            $sentencia->bindParam(":codigo_proveedor_empresa", $codigo_proveedor_empresa);

                            $sentencia->execute();
                        }

                        $sql = "UPDATE correlativo_control set numero= numero + 1 where tabla = 'fundo_entrada_control'";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->execute();
                    }
                }
            }

            $this->dblink->commit();
            return true;

            //return $arrayRespuesta;
        }catch(Exception $e){
            $this->dblink->rollBack();
            throw $e;
        }
    }

    public function listarManosObraFundo($codigo_fundo){
        try {
            $sql = "SELECT
                        tmo.nombre as man_obra, etm.codigo_tipo_mano_obra, etm.estado
                    from
                        empresa_tipo_mano_obra as etm inner join tipo_mano_obra as tmo on(etm.codigo_tipo_mano_obra = tmo.codigo_tipo_mano_obra)
                    where
                        etm.codigo_empresa = :codigo_empresa";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":codigo_empresa", $codigo_fundo);
            $sentencia->execute();

            if (!$sentencia->rowCount()) {
                return [];
            }

            return $sentencia->fetchAll(PDO::FETCH_ASSOC);
        } catch (Exception $e) {
            throw $e;
        }
    }

    public function listarFundosFechaSinc($codigo_empresa, $fecha_seleccionada, $detalleFundos, $detalleManoObra, $detalleVisitas){
        $this->dblink->beginTransaction();
        try {
            //SUBIDA DE DATOS
            // print_r($detalleFundos);
            foreach ($detalleFundos as $fundo) {
                $fecha_entrada_servidor = date("Y-m-d");
                $hora_entrada_servidor = date("H:i:s");
                $dni_conductor = $fundo["dni_conductor"];
                $nombre_conductor = $fundo["nombre_conductor"];
                $codigo_cat_pers = $fundo["codigo_categoria_personal"];
                $nomb_empresa = $fundo["nombre_empresa"];
                $cod_tip_vehi = $fundo["codigo_tipo_vehiculo"];
                $placa = $fundo["placa_entrada_fundo"];
                $procedencia = $fundo["procedencia_fundo"];
                $numero_asientos = $fundo["numero_asientos"];
                $numero_ocupantes = $fundo["numero_ocupantes"];
                $motivo = $fundo["motivo_fundo"];
                $contenedor = $fundo["contenedor"];
                $entrada_salida_fundo = $fundo["entrada_salida_fundo"];
                $cod_usuario = $fundo["codigo_usuario"];
                $hora_manual = $fundo["hora_manual"];
                $cod_ruta = $fundo["codigo_ruta"];
                $codigo_entrada_fundo =  $fundo["codigo_entrada"];

                $dia = substr($fundo["fecha_manual"], 0, 2);
                $mes = substr($fundo["fecha_manual"], 3, 2);
                $anio = substr($fundo["fecha_manual"], 6);
                // $fecha_manual = date_format(date_create_from_format('d/m/Y', $fundo["fecha_manual"]), 'Y-m-d'); //convierto la fecha de formato dia/mes/año a formato SQL (Año-mes-día)

                $fecha_manual = $anio.'-'.$mes.'-'.$dia; //convierto la fecha de formato dia/mes/año a formato SQL (Año-mes-día)

                // echo "<pre>";
                // echo $fecha_manual;
                // echo "</pre>";
                $cod_off_line = "";
                if (isset($fundo["codigo_off_line"])) {
                    $cod_off_line = $fundo["codigo_off_line"];
                }

                $cod_caseta = 0;
                if(isset($fundo["codigo_caseta"])){
                    $cod_caseta = $fundo["codigo_caseta"];
                }

                $orientacion = "";
                if (isset($fundo["orientacion"])) {
                    $orientacion = $fundo["orientacion"];
                }
                $pin = "--------";
                if(isset($fundo["pin"])){
                    $pin = $fundo["pin"];
                }

                $mano_obra = "N";
                if(isset($fundo["mano_obra"])){
                    $mano_obra = $fundo["mano_obra"];
                }

                $codigo_proveedor = null;
                if(isset($fundo["codigo_proveedor"])){
                    $codigo_proveedor = $fundo["codigo_proveedor"];
                }
                $codigo_ruta_transporte = null;
                if(isset($fundo["codigo_ruta_transporte"])){
                    $codigo_ruta_transporte = $fundo["codigo_ruta_transporte"];
                }
                $codigo_proveedor_empresa = null;
                if(isset($fundo["codigo_proveedor_empresa"])){
                    $codigo_proveedor_empresa = $fundo["codigo_proveedor_empresa"];
                }
                //primero consultamos la entrada
                $timestamp_manual = $fecha_manual." ".$hora_manual;

                $sql = "SELECT
                            codigo_ruta,
                            (fecha_entrada_salida ||' ' || hora_entrada_salida)::TIMESTAMP as dt
                        from
                            garita_entrada_salida_control
                        where dni_conductor = :dni and tipo = 'E' and
                        (fecha_entrada_salida ||' ' || hora_entrada_salida)::TIMESTAMP <= :timestamp_manual and (fecha_entrada_salida ||' ' || hora_entrada_salida)::TIMESTAMP >= :timestamp_manual::TIMESTAMP - INTERVAL '5 hours'  and codigo_ruta is not null  order by 2 desc limit 1;";

                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":dni",$dni_conductor);
                $sentencia->bindParam(":timestamp_manual",$timestamp_manual);
                $sentencia->execute();

                if($sentencia->rowCount()){ //Si encontró la entrada en la garita, entonces recojo el código de ruta
                    $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
                    $cod_ruta = $resultado["codigo_ruta"];
                }else{ //sino creo una nueva entrada con el código de ruta creada desde el fundo
                    $lugar_creacion = "F#".$codigo_empresa;
                    $sql = "INSERT INTO ruta(dni, lugar_creacion) values(:p_dni, :p_lugar);";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_dni", $dni_conductor);
                    $sentencia->bindParam(":p_lugar", $lugar_creacion);
                    $sentencia->execute();
                    $cod_ruta = $this->dblink->lastInsertId();
                    //con lastInsertId obtengo el código de ruta creado en el insert
                }

                $sql = "SELECT * FROM f_generar_correlativo('fundo_entrada_control') AS nc";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->execute();
                $resultado = $sentencia->fetch();

                if ($sentencia->rowCount()) {
                    $codigo_entrada = $resultado["nc"];

                    if($entrada_salida_fundo == "E"){ //agregamos la entrada
                        $sql = "INSERT INTO fundo_entrada_control(
                                    codigo_entrada_fundo,
                                    placa_entrada_fundo,
                                    hora_entrada_fundo,
                                    hora_manual,
                                    dni_conductor,
                                    nombre_conductor,
                                    numero_ocupantes,
                                    codigo_empresa,
                                    codigo_usuario,
                                    codigo_tipo_vehiculo,
                                    codigo_categoria_personal,
                                    entrada_salida_fundo,
                                    numero_asientos,
                                    procedencia_fundo,
                                    motivo_fundo,
                                    nombre_empresa,
                                    fecha_entrada_fundo,
                                    codigo_ruta,
                                    codigo_caseta,
                                    fecha_manual,
                                    codigo_off_line,
                                    orientacion,
                                    pin,
                                    mano_obra,
                                    codigo_proveedor,
                                    codigo_ruta_transporte,
                                    codigo_proveedor_empresa
                        )VALUES (
                                    :p_codigo_entrada_fundo,
                                    :p_placa_entrada_fundo,
                                    :p_hora_entrada_fundo,
                                    :p_hora_manual,
                                    :p_dni_conductor,
                                    :p_nombre_conductor,
                                    :p_numero_ocupantes,
                                    :p_codigo_empresa,
                                    :p_codigo_usuario,
                                    :p_codigo_tipo_vehiculo,
                                    :p_codigo_categoria_personal,
                                    :p_entrada_salida_fundo,
                                    :p_numero_asientos,
                                    :p_procedencia_fundo,
                                    :p_motivo_fundo,
                                    :p_nombre_empresa,
                                    :p_fecha_entrada_fundo,
                                    :p_codigo_ruta,
                                    :p_codigo_caseta,
                                    :fecha_manual,
                                    :codigo_off_line,
                                    :orientacion,
                                    :pin,
                                    :mano_obra,
                                    :codigo_proveedor,
                                    :codigo_ruta_transporte,
                                    :codigo_proveedor_empresa );";

                        //Preparar la sentencia
                        $sentencia = $this->dblink->prepare($sql);

                        //Asignar un valor a cada parametro
                        $sentencia->bindParam(":p_codigo_entrada_fundo", $codigo_entrada);
                        $sentencia->bindParam(":p_placa_entrada_fundo", $placa);
                        $sentencia->bindParam(":p_hora_entrada_fundo", $hora_entrada_servidor);
                        $sentencia->bindParam(":p_hora_manual", $hora_manual);
                        $sentencia->bindParam(":p_dni_conductor", $dni_conductor);
                        $sentencia->bindParam(":p_nombre_conductor", $nombre_conductor);
                        $sentencia->bindParam(":p_numero_ocupantes", $numero_ocupantes);
                        $sentencia->bindParam(":p_codigo_empresa",  $codigo_empresa);
                        $sentencia->bindParam(":p_codigo_usuario",  $cod_usuario);
                        $sentencia->bindParam(":p_codigo_tipo_vehiculo", $cod_tip_vehi);
                        $sentencia->bindParam(":p_codigo_categoria_personal", $codigo_cat_pers);
                        $sentencia->bindParam(":p_entrada_salida_fundo", $entrada_salida_fundo);
                        $sentencia->bindParam(":p_numero_asientos", $numero_asientos);
                        $sentencia->bindParam(":p_procedencia_fundo", $procedencia);
                        $sentencia->bindParam(":p_motivo_fundo", $motivo);
                        $sentencia->bindParam(":p_nombre_empresa", $nomb_empresa);
                        $sentencia->bindParam(":p_fecha_entrada_fundo", $fecha_entrada_servidor);
                        $sentencia->bindParam(":p_codigo_ruta", $cod_ruta);
                        $sentencia->bindParam(":p_codigo_caseta", $cod_caseta);
                        $sentencia->bindParam(":fecha_manual", $fecha_manual);
                        $sentencia->bindParam(":codigo_off_line", $cod_off_line);
                        $sentencia->bindParam(":orientacion", $orientacion);
                        $sentencia->bindParam(":pin", $pin);
                        $sentencia->bindParam(":mano_obra", $mano_obra);
                        $sentencia->bindParam(":codigo_proveedor", $codigo_proveedor);
                        $sentencia->bindParam(":codigo_ruta_transporte", $codigo_ruta_transporte);
                        $sentencia->bindParam(":codigo_proveedor_empresa", $codigo_proveedor_empresa);
                        //Ejecutar la sentencia preparada
                        $sentencia->execute();

                        $sql = "SELECT * from alertas_empresa where dni = :p_dni and codigo_empresa = :p_codigo_empresa;";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->bindParam(":p_dni", $dni_conductor);
                        $sentencia->bindParam(":p_codigo_empresa", $codigo_empresa);
                        $sentencia->execute();

                        $resultado = $sentencia->fetchAll();

                        if ($sentencia->rowCount()) {
                            foreach ($resultado as $key => $value) {
                                $telefono = $value['telefono'];
                                $mensaje = date('h:i A') . ' ' . $value['mensaje'];

                                $url_holder_sms = str_replace(' ', '%20', "http://servicio.smsmasivos.com.ar/enviar_sms.asp?api=1&usuario=PROOLMOS&clave=PROOLMOS352&tos=" . $telefono . "&texto=" . $mensaje);
                                $ch = curl_init();
                                curl_setopt($ch, CURLOPT_URL, $url_holder_sms);
                                curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                                curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
                                $data = curl_exec($ch);
                                curl_close($ch);
                            }
                        }
                    }else{ //agregamos la salida
                        $sql = "SELECT codigo_entrada_fundo as cod_ent FROM fundo_entrada_control WHERE dni_conductor = :p_dni and entrada_salida_fundo = 'E' order by codigo_entrada_fundo desc limit 1;";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->bindParam(":p_dni", $dni_conductor);
                        $sentencia->execute();
                        $resultado = $sentencia->fetch();
                        if (isset($resultado['cod_ent'])) {
                            $codigo_entrada_fundo = $resultado['cod_ent'];
                        }

                        $sql = "INSERT INTO fundo_entrada_control(
                                        codigo_entrada_fundo,
                                        placa_entrada_fundo,
                                        hora_entrada_fundo,
                                        dni_conductor,
                                        nombre_conductor,
                                        numero_ocupantes,
                                        codigo_empresa,
                                        codigo_usuario,
                                        entrada_salida_fundo,
                                        numero_asientos,
                                        procedencia_fundo,
                                        contenedor,
                                        nombre_empresa,
                                        hora_manual,
                                        fecha_entrada_fundo,
                                        codigo_ruta,
                                        codigo_entrada,
                                        fecha_manual,
                                        codigo_off_line,
                                        pin,
                                        codigo_proveedor,
                                        codigo_ruta_transporte,
                                        codigo_proveedor_empresa
                                    ) VALUES (
                                        :p_codigo_entrada_fundo,
                                        :p_placa_entrada_fundo,
                                        :p_hora_entrada_fundo,
                                        :p_dni_conductor,
                                        :p_nombre_conductor,
                                        :p_numero_ocupantes,
                                        :p_codigo_empresa,
                                        :p_codigo_usuario,
                                        'S',
                                        :p_numero_asientos,
                                        :p_procedencia_fundo,
                                        :p_contenedor,
                                        :p_nombre_empresa,
                                        :p_hora_manual,
                                        :p_fecha_entrada_fundo,
                                        :p_codigo_ruta,
                                        :p_codigo_entrada,
                                        :fecha_manual,
                                        :codigo_off_line,
                                        :pin,
                                        :codigo_proveedor,
                                        :codigo_ruta_transporte,
                                        :codigo_proveedor_empresa
                                    );";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->bindParam(":p_codigo_entrada_fundo", $codigo_entrada);
                        $sentencia->bindParam(":p_placa_entrada_fundo", $placa);
                        $sentencia->bindParam(":p_hora_entrada_fundo",  $hora_entrada_servidor);
                        $sentencia->bindParam(":p_dni_conductor", $dni_conductor);
                        $sentencia->bindParam(":p_nombre_conductor", $nombre_conductor);
                        $sentencia->bindParam(":p_numero_ocupantes", $numero_ocupantes);
                        $sentencia->bindParam(":p_codigo_empresa",  $codigo_empresa);
                        $sentencia->bindParam(":p_codigo_usuario",  $cod_usuario);
                        $sentencia->bindParam(":p_numero_asientos", $numero_asientos);
                        $sentencia->bindParam(":p_procedencia_fundo", $procedencia);
                        $sentencia->bindParam(":p_contenedor", $contenedor);
                        $sentencia->bindParam(":p_nombre_empresa",  $nomb_empresa);
                        $sentencia->bindParam(":p_hora_manual",  $hora_manual);
                        $sentencia->bindParam(":p_fecha_entrada_fundo",  $fecha_entrada_servidor);
                        $sentencia->bindParam(":p_codigo_ruta",  $cod_ruta);
                        $sentencia->bindParam(":p_codigo_entrada",  $codigo_entrada_fundo);
                        $sentencia->bindParam(":fecha_manual",  $fecha_manual);
                        $sentencia->bindParam(":codigo_off_line",  $cod_off_line);
                        $sentencia->bindParam(":pin",  $pin);
                        $sentencia->bindParam(":codigo_proveedor", $codigo_proveedor);
                        $sentencia->bindParam(":codigo_ruta_transporte", $codigo_ruta_transporte);
                        $sentencia->bindParam(":codigo_proveedor_empresa", $codigo_proveedor_empresa);
                        $sentencia->execute();
                    }

                    $sql = "UPDATE correlativo_control set numero= numero + 1 where tabla = 'fundo_entrada_control'";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->execute();
                }
            }


            // RECORREMOS EL ARRAY DETALLE DE VISITAS
            $comentarioVisita = "";
            foreach($detalleVisitas as $visita){
                $dniVisita = $visita["dni"];
                $cod_off_line_visita = "";
                if (isset($visita["codigo_off_line"])) {
                    $cod_off_line_visita = $visita["codigo_off_line"];
                }

                $sql = "SELECT codigo_entrada_fundo from fundo_entrada_control where codigo_off_line = :codigo_off_line";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":codigo_off_line", $cod_off_line_visita);
                $sentencia->execute();

                if($sentencia->rowCount()){
                    $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
                    $codigo_entrada_fundo_visita = $resultado["codigo_entrada_fundo"];
                    //INSERTAMOS LAS VISITAS EN EL FUNDO CORRESPONDIENTE
                    $sql = "SELECT * FROM f_generar_correlativo('visita_fundos') AS nc";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->execute();
                    $resultado = $sentencia->fetch();
                    $codigo_visita = $resultado["nc"];

                    $sql = "SELECT
                                (nombre_conductor||','||apellidos_maternos_conductor||' '||apellidos_paternos_conductor) as nombre_completo
                            FROM
                                conductor_control
                            WHERE
                                lower(dni_conductor) = :p_dni;";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_dni", $dniVisita);
                    $sentencia->execute();
                    $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
                    $nombre_visita = "";
                    if(isset($resultado["nombre_completo"])){
                        $nombre_visita = $resultado["nombre_completo"];
                    }else{
                        if(strlen($dniVisita) == 8){
                            $json = '{CODDNI: "' . $dniVisita . '"}';
                            $ch = curl_init();
                            curl_setopt($ch, CURLOPT_URL, CONSULTA_JNE);
                            curl_setopt($ch, CURLOPT_POST, 1);
                            curl_setopt($ch, CURLOPT_POSTFIELDS, $json);
                            curl_setopt($ch, CURLOPT_HEADER, TRUE);
                            curl_setopt($ch, CURLOPT_HTTPHEADER, array("Content-Type: application/json", "RequestVerificationToken: " . TOKEN_JNE));
                            curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);

                            $head = curl_exec($ch);
                            $codigoHttpd = curl_getinfo($ch, CURLINFO_HTTP_CODE);
                            curl_close($ch);

                            $porciones = explode("GMT", $head);
                            $ultimo_elemento = count($porciones) - 1;
                            $respuesta = json_decode($porciones[$ultimo_elemento], TRUE);

                            if ($codigoHttpd == 200) {
                                $respuesta = explode("|", $respuesta["data"]);
                                if ($respuesta[0] == "" && $respuesta[1] == "" && $respuesta[2] == "") {
                                    $nombre_visita = "";
                                }else{
                                    $nombre_visita = $respuesta[0] . " " . $respuesta[1] . ", " . $respuesta[2];
                                }
                            } else {
                                $nombre_visita = "";
                            }
                        }else{
                            $nombre_visita = "";
                        }
                    }

                    $sql = "INSERT INTO
                                    visita_fundos(codigo_visita_fundo, codigo_entrada_fundo, dni, nombre, comentario, codigo_off_line, codigo_off_line_fundo)
                                    values(:p_cod_visita, :p_cod_entrada, :p_dni, :p_nombre, :p_comentario, :codigo_off_line, :codigo_off_line_fundo);";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_cod_visita", $codigo_visita);
                    $sentencia->bindParam(":p_cod_entrada", $codigo_entrada_fundo_visita);
                    $sentencia->bindParam(":p_dni", $dniVisita);
                    $sentencia->bindParam(":p_nombre", $nombre_visita);
                    $sentencia->bindParam(":p_comentario", $comentarioVisita);
                    $sentencia->bindParam(":codigo_off_line", $cod_off_line_visita);
                    $sentencia->bindParam(":codigo_off_line_fundo", $cod_off_line_visita);
                    $sentencia->execute();

                    $sql = "UPDATE correlativo_control SET numero = numero + 1 WHERE tabla = 'visita_fundos'";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->execute();
                    //INSERTAMOS LAS VISITAS EN EL FUNDO CORRESPONDIENTE
                }
            }
            // RECORREMOS EL ARRAY DETALLE DE VISITAS

            // RECORREMOS EL ARRAY DETALLE DE MANO DE OBRA
            foreach($detalleManoObra as $manoObra){
                $cod_off_line_mano_obra = "";
                if (isset($manoObra["codigo_off_line"])) {
                    $cod_off_line_mano_obra = $manoObra["codigo_off_line"];
                }


                $cod_off_line_fundo_mano_obra = "";
                if (isset($manoObra["codigo_off_line_fundo"])) {
                    $cod_off_line_fundo_mano_obra = $manoObra["codigo_off_line_fundo"];
                }

                $numero_trabajadores = $manoObra["numero_trabajadores"];
                $codigo_tipo_mano_obra = $manoObra["codigo_tipo_mano_obra"];

                $sql = "SELECT codigo_entrada_fundo from fundo_entrada_control where codigo_off_line = :codigo_off_line";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":codigo_off_line", $cod_off_line_fundo_mano_obra);
                $sentencia->execute();

                if($sentencia->rowCount()){
                    $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
                    $codigo_entrada_fundo_mano_obra = $resultado["codigo_entrada_fundo"];

                    //VALIDAMOS QUE NO ESTÉ EN EL SERVIDOR PARA INSERTARLA
                    $sql = "SELECT
                                *
                            from
                                fundo_entrada_control_tipo_mano_obra
                            where
                                codigo_entrada_fundo = :codigo_entrada_fundo and codigo_tipo_mano_obra = :codigo_tipo_mano_obra";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":codigo_entrada_fundo", $codigo_entrada_fundo_mano_obra);
                    $sentencia->bindParam(":codigo_tipo_mano_obra", $codigo_tipo_mano_obra);
                    $sentencia->execute();
                    //VALIDAMOS QUE NO ESTÉ EN EL SERVIDOR PARA INSERTARLA
                    if(!$sentencia->rowCount()){
                        //INSERTAMOS LA MANO DE OBRA EN EL FUNDO CORRESPONDIENTE
                        $sql = "INSERT INTO
                                        fundo_entrada_control_tipo_mano_obra
                                        (codigo_entrada_fundo, codigo_tipo_mano_obra, numero_trabajadores, codigo_off_line, codigo_off_line_fundo)
                                values
                                    (:codigo_entrada_fundo, :codigo_tipo_mano_obra, :numero_trabajadores, :codigo_off_line, :codigo_off_line_fundo);";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->bindParam(":codigo_entrada_fundo", $codigo_entrada_fundo_mano_obra);
                        $sentencia->bindParam(":codigo_tipo_mano_obra", $codigo_tipo_mano_obra);
                        $sentencia->bindParam(":numero_trabajadores", $numero_trabajadores);
                        $sentencia->bindParam(":codigo_off_line", $cod_off_line_mano_obra);
                        $sentencia->bindParam(":codigo_off_line_fundo", $cod_off_line_fundo_mano_obra);
                        $sentencia->execute();
                        //INSERTAMOS LA MANO DE OBRA EN EL FUNDO CORRESPONDIENTE
                    }
                }
            }
            // RECORREMOS EL ARRAY DETALLE DE MANO DE OBRA

            $this->dblink->commit();
            //SUBIDA DE DATOS
            $sql = "SELECT
                        f.codigo_entrada_fundo,
                        to_char(f.fecha_entrada_fundo,'DD/MM/YYYY') AS fecha_entrada,
                        coalesce(to_char(f.fecha_manual,'DD/MM/YYYY'),to_char(f.fecha_entrada_fundo,'DD/MM/YYYY')) AS fecha_manual,
                        to_char(f.hora_entrada_fundo, 'HH24:MI') AS hora_entrada,
                        coalesce(f.dni_conductor, 'sin-dni') as dni,
                        coalesce(f.nombre_conductor, 'sin-nombre') as conductor,
                        coalesce(f.codigo_categoria_personal, 0) as codigo_categoria_personal,
                        UPPER(coalesce(f.nombre_empresa, 'sin-nombre')) as nombre_empresa,
                        coalesce(f.codigo_tipo_vehiculo, 0) as codigo_tipo_vehiculo,
                        UPPER(coalesce(f.placa_entrada_fundo, 'sin-placa')) as placa,
                        upper(coalesce(f.procedencia_fundo,'sin-procedencia')) as procedencia_fundo,
                        (case coalesce(f.numero_asientos, '0') when '' then '0' else coalesce(f.numero_asientos, '0') end) as numero_asientos,
                        (case coalesce(f.numero_ocupantes, '0') when '' then '0' else coalesce(f.numero_ocupantes, '0') end) as numero_ocupantes,
                        upper(coalesce(f.motivo_fundo, '')) as motivo_fundo,
                        coalesce(f.contenedor, '') as contenedor,
                        f.entrada_salida_fundo,
                        f.codigo_empresa,
                        f.codigo_usuario,
                        f.estado_entrada_fundo,
                        to_char(f.hora_manual, 'HH24:MI') AS hora_manual,
                        coalesce(f.codigo_ruta, 0) as codigo_ruta,
                        coalesce(f.codigo_entrada, 0) as codigo_entrada,
                        coalesce(f.codigo_caseta, 0) as codigo_caseta,
                        coalesce(f.codigo_off_line, '') as codigo_off,
                        coalesce(f.orientacion, '') as orientacion,
                        coalesce(f.pin, '') as pin,
                        coalesce(f.mano_obra, '') as mano_obra,
                        coalesce(f.codigo_proveedor, 0) as codigo_proveedor,
                        coalesce(f.codigo_ruta_transporte, 0) as codigo_ruta_transporte,
                        coalesce(f.codigo_proveedor_empresa, 0) as codigo_proveedor_empresa,
                        (select count(*) from visita_fundos as v where v.codigo_entrada_fundo = f.codigo_entrada_fundo) as visitas,
                        (select count(*) from fundo_entrada_control_tipo_mano_obra as fectmo where fectmo.codigo_entrada_fundo = f.codigo_entrada_fundo) as num_manos_obra
                    from
                        fundo_entrada_control as f
                    where
                        f.codigo_empresa = :codigo_empresa and f.estado_entrada_fundo='E' and coalesce(f.fecha_manual,f.fecha_entrada_fundo) = :fecha
                    order by 2 desc;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":codigo_empresa", $codigo_empresa);
            $sentencia->bindParam(":fecha", $fecha_seleccionada);
            $sentencia->execute();

            if (!$sentencia->rowCount()) {
                return [];
            }else{

                $resultado = $sentencia->fetchAll(PDO::FETCH_ASSOC);

                // print_r($result);
                foreach ($resultado as $item){
                    $id_entrada = $item["codigo_entrada_fundo"];
                    $visitas = $item["visitas"];
                    if($visitas > 0 ){
                        $sql = "SELECT
                                    codigo_visita_fundo,
                                    codigo_entrada_fundo,
                                    dni,
                                    coalesce(nombre, '') as nombre,
                                    coalesce(comentario, '') as comentario,
                                    coalesce(codigo_off_line, '') as codigo_off_line,
                                    coalesce(codigo_off_line_fundo, '') as codigo_off_line_fundo
                                FROM visita_fundos where codigo_entrada_fundo = :p_codigo_entrada_fundo";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->bindParam(":p_codigo_entrada_fundo", $id_entrada);
                        $sentencia->execute();
                        $item["visitas_lista"] = $sentencia->fetchAll(PDO::FETCH_ASSOC);
                    }else{
                        $item["visitas_lista"] = [];
                    }

                    $num_manos_obra = $item["num_manos_obra"];
                    if($num_manos_obra > 0 ){
                        $sql = "SELECT
                                    codigo_entrada_fundo as codigo_entrada,
                                    codigo_tipo_mano_obra as codigo_mano_obra,
                                    numero_trabajadores,
                                    coalesce(codigo_off_line, '') as off_line,
                                    coalesce(codigo_off_line_fundo, '') as off_line_fundo
                                FROM fundo_entrada_control_tipo_mano_obra where codigo_entrada_fundo = :p_codigo_entrada_fundo";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->bindParam(":p_codigo_entrada_fundo", $id_entrada);
                        $sentencia->execute();
                        $item["manos_obra_lista"] = $sentencia->fetchAll(PDO::FETCH_ASSOC);
                    }else{
                        $item["manos_obra_lista"] = [];
                    }

                    $arrayResultado[] = $item;
                }

                return $arrayResultado;
            }

        } catch (Exception $e) {
            $this->dblink->rollBack();
            throw $e;
        }
    }
}
