<?php
require __DIR__.'/../datos/Conexion.clase.php';
require __DIR__ . '/../util/definiciones.php';
date_default_timezone_set("America/Lima");

class Garita extends Conexion {

    public function obtenerEstadoConductor($codigo_condcutor, $codigo_proveedor, $codigo_empresa, $codigo_vehiculo){
        $sql = "SELECT estado from proveedor_empresa where
                    codigo_conductor = ? and
                    codigo_proveedor = ? and
                    codigo_empresa = ? and
                    codigo_vehiculo = ?";
        $sentencia = $this->dblink->prepare($sql);
        $sentencia->bindParam(1, $codigo_condcutor);
        $sentencia->bindParam(2, $codigo_proveedor);
        $sentencia->bindParam(3, $codigo_empresa);
        $sentencia->bindParam(4, $codigo_vehiculo);
        $sentencia->execute();
        if(!$sentencia->rowCount()){
            throw new Exception("Este código QR no es válido o ha sido revocado");
        }
        $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
        return $resultado["estado"];
    }
    public function listadoGarita($fecha1, $fecha2, $tipo, $codigo_caseta, $tipo_entrada) {
        try {
            if($tipo_entrada == "E"){
                $sql = "SELECT distinct
                        g.codigo_entrada_salida,
                        g.dni_conductor as dni,
                        (case g.nombre_conductor
                            when '' then 'SIN REGISTRO'
                            when 'null' then 'SIN REGISTRO'
                            when 'CONDUCTOR NO, ENCONTRADO ' then 'CONDUCTOR NO ENCONTRADO'
                            else coalesce(g.nombre_conductor, 'SIN REGISTRO') end) as nomb,
                        coalesce(c.estado_conductor, 'A') as est,
                        to_char(g.hora_entrada_salida,'HH12:MI AM')AS hora,
                        to_char(g.fecha_entrada_salida,'DD/MM/YYYY')AS fech,
                        (case g.exceso when 1 then 'EXCESO DE VELOCIDAD' else 'VELOCIDAD PERMITIDA' end) AS EXCESO,
                        coalesce(e.nombre_empresa, '') as perte,
                        coalesce(e2.nombre_empresa, '') as diri,
                        upper(g.placa_entrada_salida) as placa,
                        coalesce(v.descripcion_tipo_vehiculo, '') as tipo_vehi
                    FROM garita_entrada_salida_control as g Left join conductor_control as c on(g.dni_conductor = c.dni_conductor)
                            INNER JOIN empresa_control as e on (g.codigo_empresa = e.codigo_empresa)
                            inner join empresa_control as e2 on (g.codigo_empresa2 = e2.codigo_empresa)
                            inner join tipo_vehiculo_control as v on (g.codigo_tipo_vehiculo = v.codigo_tipo_vehiculo)
                    WHERE
                        g.tipo= 'E' and g.codigo_caseta = :p_codigo_caseta and g.estado_entrada = 'E'
                        and (
                            case :p_tipo
                            when 'H' then g.fecha_entrada_salida = current_date
                            else g.fecha_entrada_salida between :p_fecha1 and :p_fecha2
                            end
                            )
                    order by
                        g.codigo_entrada_salida DESC;";
            }else{
                $sql = "SELECT
                        distinct
                        g.codigo_entrada_salida,
                        g.dni_conductor as dni,
                        (case g.nombre_conductor
                            when '' then 'SIN REGISTRO'
                            when 'null' then 'SIN REGISTRO'
                            when 'CONDUCTOR NO, ENCONTRADO ' then 'CONDUCTOR NO ENCONTRADO'
                            else coalesce(g.nombre_conductor, 'SIN REGISTRO') end) as nomb,
                        coalesce(c.estado_conductor, 'A') as est,
                        to_char(g.hora_entrada_salida,'HH12:MI AM')AS hora,
                        to_char(g.fecha_entrada_salida,'DD/MM/YYYY') AS fech ,
						(case g.exceso when 1 then 'EXCESO DE VELOCIDAD' else 'VELOCIDAD PERMITIDA' end) AS EXCESO,
                        upper(g.placa_entrada_salida) as placa,
                        upper(g.placa_alterna_salida) as placa_alterna,
                        (case coalesce(g.contenedor, 'N') when '' then 'N' else g.contenedor end) as contenedor
                    FROM garita_entrada_salida_control as g Left join conductor_control as c on(g.dni_conductor = c.dni_conductor)
                    WHERE
                        g.tipo= 'S' and g.codigo_caseta = :p_codigo_caseta and g.estado_entrada = 'E'
                        and (
                            case :p_tipo
                            when 'H' then g.fecha_entrada_salida = current_date
                            else g.fecha_entrada_salida between :p_fecha1 and :p_fecha2
                            end
                            )
                    order by
                        g.codigo_entrada_salida desc;";
            }


            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_fecha1", $fecha1);
            $sentencia->bindParam(":p_fecha2", $fecha2);
            $sentencia->bindParam(":p_tipo", $tipo);
            $sentencia->bindParam(":p_codigo_caseta", $codigo_caseta);
            $sentencia->execute();

            $resultado = $sentencia->fetchAll(PDO::FETCH_ASSOC);

            return $resultado;

        } catch (Exception $exc) {
            throw $exc;
        }
    }


    public function datosSalidaGarita($dni){
        try{
            $sql = "SELECT
                        (case g.nombre_conductor
                                when ('null')
                                    then 'CONDUCTOR NO ENCONTRADO'
                                when ''
                                    then 'CONDUCTOR NO ENCONTRADO'
                                else g.nombre_conductor end) as nomb,
                        upper(g.placa_entrada_salida) as placa,
                        to_char(g.hora_entrada_salida,'HH12:MI AM')AS hora,
                        to_char(g.fecha_entrada_salida,'DD/MM/YYYY')AS fecha,
                        c.descripcion_caseta as ingreso,
                        g.codigo_tipo_vehiculo, 
                        codigo_empresa2 as cod_emp_pertenece,
                        coalesce(co.estado_conductor, 'A') as estado
                    from
                        garita_entrada_salida_control as g inner join caseta_control as c on (g.codigo_caseta = c.codigo_caseta)
                        Left join conductor_control as co on(g.dni_conductor = co.dni_conductor)
                    where
                        lower(trim(g.dni_conductor)) = lower(trim(:p_dni)) and g.tipo='E' and fecha_entrada_salida >= '2023-01-01'
                        order by g.codigo_entrada_salida desc
                        limit 1;";
            //order by g.fecha_entrada_salida desc
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $dni);
            $sentencia->execute();
            if($sentencia->rowCount()){
                $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
            }else{
                $sql = "SELECT
                    upper(apellidos_paternos_conductor||' '||apellidos_maternos_conductor||', '||nombre_conductor) as nomb,
                    estado_conductor as estado
                FROM
                    conductor_control
                WHERE
                    lower(trim(dni_conductor)) = lower(trim(:p_dni));";

                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_dni", $dni);
                $sentencia->execute();
                $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
                
               

                if (!$sentencia->rowCount()) { //si no encuentra los datos del conductor en nuestra base de datos, buscamos en la reniec
                   
                    try{
                        //eliminar esto cuando la api se levante nuevamente, si se cae la api levantar nuevamente
                          // $resultado["nomb"] = "CONDUCTOR NO ENCONTRADO";
                          // $resultado["estado"] = "A";
                        //eliminar esto cuando la api se levante nuevamente y descomentar lo de abajo $response
    
                            $url = "https://dniruc.apisperu.com/api/v1/dni/".trim($dni)."?token=eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJlbWFpbCI6Imtpa2UuamVwZkBnbWFpbC5jb20ifQ.NRMVtJiFKRXYBXuJPLwNsKeRK5hPWIALRxbqNSWloXU";
                
                            $options = [
                                'http' => [
                                    'timeout' => 7, // Establece el tiempo límite en segundos
                                ],
                            ];
                            
                            $context = stream_context_create($options);
                            $response = @file_get_contents($url, false, $context);
                            
                            if ($response === false) {
                                // La petición ha excedido el tiempo límite
                                $resultado["nomb"] = "CONDUCTOR NO ENCONTRADO";
                                $resultado["estado"] = "A";
                            } else {
                                $persona = json_decode($response);
                            
                                if ($persona->{"nombres"}) {
                                    // Datos encontrados
                                    $paterno = $persona->{'apellidoPaterno'};
                                    $materno = $persona->{'apellidoMaterno'};
                                    $nombres = $persona->{'nombres'};
                            
                                    $res = $paterno .' '. $materno .' '. $nombres;
                                    $resultado["nomb"] = $res;
                                    $resultado["estado"] = "A";
                            
                                    $registrar = $this->registarConductorSegundoPlano(trim($dni), $paterno, $materno, $nombres);
                                } else if ($persona->{"success"}) {
                                    // No se encontró el conductor
                                    $resultado["nomb"] = "CONDUCTOR NO ENCONTRADO";
                                    $resultado["estado"] = "A";
                                } else {
                                    // Otro caso de error
                                    $resultado["nomb"] = "CONDUCTOR NO ENCONTRADO";
                                    $resultado["estado"] = "A";
                                }
                            }
                                           
                           
                    }catch(Exception $e){
                        $e;
                    }
                       
                   
                   
                }
                
                $resultado["placa"] = "";
                $resultado["hora"] = date("H:i:s");
                $resultado["fecha"] = date('d/m/Y');
                $resultado["ingreso"] = "SIN CASETA DE INGRESO";
                
                
            }
            
         
           $resultado["fecha_actual"] = $res["fecha"];
           $resultado["hora_actual"] = $res["hora"];
            
            $resultado["dni"] = trim($dni);
            return $resultado;
        }catch(Exception $e){
            throw $e;
        }
    }

    public function obtenerQRConductorPorDni($dni, $tipoLista = null){
        try{
            $sql = "SELECT
                        upper(v.placa) as placa,
                        dt.codigo_empresa as cod_emp_pertenece,
                        dt.codigo_empresa as cod_emp_dirige,
                        v.codigo_tipo_vehiculo,
                        upper(c.apellidos_paternos_conductor||' '||c.apellidos_maternos_conductor||', '||c.nombre_conductor) as nombre_conductor,
                        upper(trim(c.dni_conductor)) as dni,
                        dt.estado as estado,
                        dt.codigo_ruta_transporte as codigo_ruta,
                        rt.nombre_ruta as nombre_ruta,
                        dt.codigo_proveedor
                    FROM
                        proveedor_empresa as dt inner join conductor_control as c on(dt.codigo_conductor = c.codigo_conductor)
                        inner join vehiculo as v on (dt.codigo_vehiculo = v.codigo_vehiculo)
                        inner join proveedor as e on(dt.codigo_proveedor = e.codigo_proveedor)
                        inner join ruta_transporte as rt on(dt.codigo_ruta_transporte = rt.codigo_ruta_transporte)
                    WHERE
                        trim(c.dni_conductor) = trim(?) and dt.estado = 'A'
                    ORDER BY
                        dt.codigo_proveedor_empresa desc";
            if(empty($tipoLista)){
                $sql.="LIMIT
                        1;";
            }
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(1, $dni);
            $sentencia->execute();
            if (!$sentencia->rowCount()) {
                return [];
            }

            if(!empty($tipoLista)){
                $resultado = $sentencia->fetchAll(PDO::FETCH_ASSOC);
            }else{
                $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
                if($resultado["estado"] == "I"){
                    $resultado["alerta"] = "Este código está inactivo, por favor verifique todos los datos";
                }
                $resultado["tipo_empresa"] = "P";
            }

            return $resultado;
        } catch (Exception $exc) {
            throw $exc;
        }
    }

    public function obtenerQRConductorPorDniSalida($dni){
        try{
            $sql = "SELECT
                        upper(c.apellidos_paternos_conductor||' '||c.apellidos_maternos_conductor||', '||c.nombre_conductor) as nomb,
                        upper(v.placa) as placa,
                        upper(trim(c.dni_conductor)) as dni,
                        dt.estado as estado
                    FROM
                        proveedor_empresa as dt inner join conductor_control as c on(dt.codigo_conductor = c.codigo_conductor)
                        inner join vehiculo as v on (dt.codigo_vehiculo = v.codigo_vehiculo)
                        inner join proveedor as e on(dt.codigo_proveedor = e.codigo_proveedor)
                        inner join ruta_transporte as rt on(dt.codigo_ruta_transporte = rt.codigo_ruta_transporte)
                    WHERE
                        trim(c.dni_conductor) = trim(?) and dt.estado = 'A'
                    ORDER BY
                        dt.codigo_proveedor_empresa desc
                    LIMIT
                        1;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(1, $dni);
            $sentencia->execute();
            if (!$sentencia->rowCount()) {
                return [];
            }
            $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
            if($resultado["estado"] == "I"){
                $resultado["alerta"] = "Este código está inactivo, por favor verifique todos los datos";
            }
            $resultado["tipo_empresa"] = "P";

            $sql = "SELECT
                        to_char(g.hora_entrada_salida,'HH12:MI AM')AS hora,
                        to_char(g.fecha_entrada_salida,'DD/MM/YYYY')AS fecha,
                        c.descripcion_caseta as ingreso
                    from
                        garita_entrada_salida_control as g inner join caseta_control as c on (g.codigo_caseta = c.codigo_caseta)
                        Left join conductor_control as co on(g.dni_conductor = co.dni_conductor)
                    where
                        lower(trim(g.dni_conductor)) = lower(trim(:p_dni)) and g.tipo='E'
                        order by g.fecha_entrada_salida desc
                        limit 1;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $dni);
            $sentencia->execute();
            if($sentencia->rowCount()){
                $resultadoSalida = $sentencia->fetch(PDO::FETCH_ASSOC);
                $resultado["hora"] = $resultadoSalida["hora"];
                $resultado["fecha"] = $resultadoSalida["fecha"];
                $resultado["ingreso"] = $resultadoSalida["ingreso"];
            }else{
                $resultado["hora"] = date("H:i:s");
                $resultado["fecha"] = date('d/m/Y');
                $resultado["ingreso"] = "SIN CASETA DE INGRESO";
            }

            return $resultado;
        } catch (Exception $exc) {
            throw $exc;
        }
    }
    

    public function datosEntradaGarita($dni){
        try {
            $sql = "SELECT
                        upper(placa_entrada_salida) as placa,
                        codigo_empresa as cod_emp_pertenece,
                        codigo_empresa2 as cod_emp_dirige,
                        codigo_tipo_vehiculo,
                        fecha_entrada_salida
                    from
                        garita_entrada_salida_control
                    where
                        lower(trim(dni_conductor)) = lower(trim(:p_dni)) and tipo='E'  and fecha_entrada_salida >= '2023-01-01'
                        order by fecha_entrada_salida desc, hora_entrada_salida desc
                        limit 1;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $dni);
            $sentencia->execute();
            $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);

            if ($sentencia->rowCount()) {
                $datosRespuesta["placa"] =  $resultado["placa"];
                $datosRespuesta["cod_emp_pertenece"] = $resultado["cod_emp_pertenece"];
                $datosRespuesta["cod_emp_dirige"] = $resultado["cod_emp_dirige"];
                $datosRespuesta["codigo_tipo_vehiculo"] = $resultado["codigo_tipo_vehiculo"];
            }else{
                $datosRespuesta["placa"] = "";
                $datosRespuesta["cod_emp_pertenece"] = -1;
                $datosRespuesta["cod_emp_dirige"] = -1;
                $datosRespuesta["codigo_tipo_vehiculo"] = -1;
            }

            $sql = "SELECT
                    upper(apellidos_paternos_conductor||' '||apellidos_maternos_conductor||', '||nombre_conductor) as nombre_completo,
                    estado_conductor
                FROM
                    conductor_control
                WHERE
                    lower(trim(dni_conductor)) = lower(trim(:p_dni));";

            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $dni);
            $sentencia->execute();
            $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);

            if ($sentencia->rowCount() && strpos($resultado["nombre_completo"], "CONDUCTOR NO") === false) {
                $datosRespuesta["nombre_conductor"] = $resultado["nombre_completo"];
                $datosRespuesta["estado"] = $resultado["estado_conductor"];
            }else{ //si no encuentra los datos del conductor en nuestra base de datos, buscamos en la reniec
                try{
                     //eliminar esto cuando la api se levante nuevamente y descomentar lo de abajo $response
                       // $datosRespuesta["nombre_conductor"] = "CONDUCTOR NO ENCONTRADO";
                       // $datosRespuesta["estado"] = "A";
                     //eliminar esto cuando la api se levante nuevamente y descomentar lo de abajo $response
                   
                   
                    $url = "https://dniruc.apisperu.com/api/v1/dni/".trim($dni)."?token=eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJlbWFpbCI6Imtpa2UuamVwZkBnbWFpbC5jb20ifQ.NRMVtJiFKRXYBXuJPLwNsKeRK5hPWIALRxbqNSWloXU";

                    $options = [
                        'http' => [
                            'timeout' => 7, // Establece el tiempo límite en segundos
                        ],
                    ];
                    
                    $context = stream_context_create($options);
                    $response = @file_get_contents($url, false, $context);
                    
                    if ($response === false) {
                        // La petición ha excedido el tiempo límite
                            $datosRespuesta["nombre_conductor"] = "CONDUCTOR NO ENCONTRADO";
                            $datosRespuesta["estado"] = "A";
                    } else {
                        $persona = json_decode($response);
                    
                       if($persona->{"nombres"}){
                            // Si encuentra en RENIEC 
                            $paterno = $persona->{'apellidoPaterno'};
                            $materno = $persona->{'apellidoMaterno'};
                            $nombres = $persona->{'nombres'};
                           
                            $resultado = $paterno .' '. $materno .' '. $nombres;
                            $datosRespuesta["nombre_conductor"] = $resultado;
                            $datosRespuesta["estado"] = "A";
                            
                            $registrar = $this->registarConductorSegundoPlano(trim($dni), $paterno, $materno, $nombres);
                            
                           
                            // Si no fue encontrado en RENIEC se regresa un array vacío
                       }else if ($persona->{'success'}){
                            $datosRespuesta["nombre_conductor"] = "CONDUCTOR NO ENCONTRADO";
                            $datosRespuesta["estado"] = "A";
                        }
                        else{
                            $datosRespuesta["nombre_conductor"] = "CONDUCTOR NO ENCONTRADO";
                            $datosRespuesta["estado"] = 'A';
                        }
                    
                    }
                
                    
                }catch(Exception $e){
                        throw $e;
                }
                 
            }
            
           
            
            $datosRespuesta["dni"] = trim($dni);
            return $datosRespuesta;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /*AGREGAR DEL REGISTRO DE SALIDA DE GARITA*/

    public function agregarSalidaMovil($dni_conductor, $placa_alterna_salida, $placa_entrada_salida, $nombre_conductor, $hora_actual, $fecha_entrada_salida, $codigo_caseta, $codigo_usuario, 
    $tipo, $contenedor, $codigo_proveedor, $codigo_ruta_transporte, $codigo_tipo_vehiculo, $exceso, $version, $dirige, $fecha_subida, $hora_subida, $procedencia, $id_qr)
    {
        $this->dblink->beginTransaction();
        try {
            /** VALIDAR CASETA DEL USUARIO **/
            $codigo_caseta = $this->obtenerEmpresaDeUsuario($codigo_usuario);
            /** VALIDAR CASETA DEL USUARIO **/
            
            /** SI NO ESTÁ REGISTRADO EL CONDUCTOR, LO REGISTRAMOS */
            
            $sql = "SELECT * FROM conductor_control WHERE lower(trim(dni_conductor)) = lower(trim(:p_dni));";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $dni_conductor);
            $sentencia->execute();
            $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);

           /* if (!$sentencia->rowCount()) {
               
                $porciones = explode(" ",$nombre_conductor);
                $paterno = $porciones[0];
                $materno = $porciones[1];
                $nombres = $porciones[2].' '.$porciones[3];


                        $sql = "SELECT * FROM f_generar_correlativo('conductor_control') as nc";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->execute();
                        $resultado = $sentencia->fetch();
                        $nuevoCodigoConductor = $resultado["nc"];
                        $dni = trim($dni_conductor);
                        $sql = "INSERT INTO conductor_control(
                                            codigo_conductor,
                                            dni_conductor,
                                            nombre_conductor,
                                            apellidos_maternos_conductor,
                                            apellidos_paternos_conductor,
                                            estado_conductor
                                        )VALUES(
                                            :p_codigo_conductor,
                                            :p_dni_conductor,
                                            :p_nombre_conductor,
                                            :p_apellidos_maternos_conductor,
                                            :p_apellidos_paternos_conductor,
                                            'A');";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->bindParam(":p_codigo_conductor", $nuevoCodigoConductor);
                        $sentencia->bindParam(":p_dni_conductor", $dni);
                        $sentencia->bindParam(":p_nombre_conductor", $nombres);
                        $sentencia->bindParam(":p_apellidos_paternos_conductor", $paterno);
                        $sentencia->bindParam(":p_apellidos_maternos_conductor", $materno);
                        $sentencia->execute();
                        $sql = "UPDATE correlativo_control SET numero = numero + 1 where tabla = 'conductor_control'";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->execute();
                        
            }*/
            /** SI NO ESTÁ REGISTRADO EL CONDUCTOR, LO REGISTRAMOS */

            $sql = "SELECT codigo_ruta as id FROM ruta WHERE dni = lower(trim(:p_dni)) and estado = 'A' order by timestamp_registro desc;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $dni_conductor);
            $sentencia->execute();
            $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);

            $codigo_ruta = "";

            if ($sentencia->rowCount()) {
                $codigo_ruta = $resultado['id'];
                $sql = "UPDATE ruta SET estado = 'C' WHERE dni = lower(trim(:p_dni));";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_dni", $dni_conductor);
                $sentencia->execute();
            }

            $sql = "SELECT * FROM f_generar_correlativo('garita_entrada_salida_control') AS nc";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->execute();
            $resultado = $sentencia->fetch();

            if ($sentencia->rowCount()) {
                $nuevoNumerosalida = $resultado["nc"];

                if ($codigo_ruta == "") {
                    $sql = "INSERT INTO garita_entrada_salida_control(codigo_entrada_salida, dni_conductor, placa_entrada_salida, placa_alterna_salida, nombre_conductor, codigo_usuario, hora_entrada_salida, fecha_entrada_salida, codigo_caseta, tipo, contenedor,
                    codigo_proveedor, codigo_ruta_transporte, codigo_tipo_vehiculo, exceso, version_app, codigo_empresa, codigo_empresa2, fecha_subida, hora_subida, procedencia, id_qr)
                    VALUES (:p_codigo_entrada_salida, :p_dni_conductor, :p_placa_entrada_salida, :p_placa_alterna_salida, :p_nombre_conductor, :p_codigo_usuario, :p_hora_entrada_salida, :p_fecha_entrada_salida, :p_codigo_caseta, :p_tipo, :p_contenedor, :codigo_proveedor,
                    :codigo_ruta_transporte,  :p_codigo_tipo_vehiculo, :p_exceso, :p_version, :p_codigo_empresa, :p_codigo_empresa2, :p_fecha_subida, :p_hora_subida, :p_procedencia, :p_id_qr);";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_codigo_entrada_salida", $nuevoNumerosalida);
                    $sentencia->bindParam(":p_dni_conductor", $dni_conductor);
                    $sentencia->bindParam(":p_placa_entrada_salida", $placa_entrada_salida);
                    $sentencia->bindParam(":p_placa_alterna_salida", $placa_alterna_salida);
                    $sentencia->bindParam(":p_nombre_conductor", $nombre_conductor);
                    $sentencia->bindParam(":p_codigo_usuario",  $codigo_usuario);
                    $sentencia->bindParam(":p_hora_entrada_salida",  $hora_actual);
                    $sentencia->bindParam(":p_fecha_entrada_salida", $fecha_entrada_salida);
                    $sentencia->bindParam(":p_codigo_caseta", $codigo_caseta);
                    $sentencia->bindParam(":p_tipo", $tipo);
                    $sentencia->bindParam(":p_contenedor", $contenedor);
                    $sentencia->bindParam(":codigo_proveedor", $codigo_proveedor);
                    $sentencia->bindParam(":codigo_ruta_transporte", $codigo_ruta_transporte);
                    $sentencia->bindParam(":p_codigo_tipo_vehiculo", $codigo_tipo_vehiculo);
                    $sentencia->bindParam(":p_exceso", $exceso);
                    $sentencia->bindParam(":p_version", $version);
                    $sentencia->bindParam(":p_codigo_empresa", $dirige);
                    $sentencia->bindParam(":p_codigo_empresa2", $dirige);
                    $sentencia->bindParam(":p_fecha_subida", $fecha_subida);
                    $sentencia->bindParam(":p_hora_subida", $hora_subida);
                    $sentencia->bindParam(":p_procedencia", $procedencia);
                    $sentencia->bindParam(":p_id_qr", $id_qr);


                } else {
                    $sql = "INSERT INTO garita_entrada_salida_control(codigo_entrada_salida, dni_conductor, placa_entrada_salida, placa_alterna_salida, nombre_conductor, codigo_usuario, hora_entrada_salida, fecha_entrada_salida, codigo_caseta, tipo, contenedor, codigo_ruta,
                    codigo_proveedor, codigo_ruta_transporte, codigo_tipo_vehiculo, exceso, version_app, codigo_empresa, codigo_empresa2, fecha_subida, hora_subida, procedencia, id_qr)
                    VALUES (:p_codigo_entrada_salida, :p_dni_conductor, :p_placa_entrada_salida, :p_placa_alterna_salida, :p_nombre_conductor, :p_codigo_usuario, :p_hora_entrada_salida, :p_fecha_entrada_salida, :p_codigo_caseta, :p_tipo, :p_contenedor,:p_codigo_ruta, :codigo_proveedor,
                    :codigo_ruta_transporte, :p_codigo_tipo_vehiculo, :p_exceso, :p_version, :p_codigo_empresa, :p_codigo_empresa2, :p_fecha_subida, :p_hora_subida, :p_procedencia, :p_id_qr);";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_codigo_entrada_salida", $nuevoNumerosalida);
                    $sentencia->bindParam(":p_dni_conductor", $dni_conductor);
                    $sentencia->bindParam(":p_placa_entrada_salida", $placa_entrada_salida);
                    $sentencia->bindParam(":p_placa_alterna_salida", $placa_alterna_salida);
                    $sentencia->bindParam(":p_nombre_conductor", $nombre_conductor);
                    $sentencia->bindParam(":p_codigo_usuario",  $codigo_usuario);
                    $sentencia->bindParam(":p_hora_entrada_salida",  $hora_actual);
                    $sentencia->bindParam(":p_fecha_entrada_salida", $fecha_entrada_salida);
                    $sentencia->bindParam(":p_codigo_caseta", $codigo_caseta);
                    $sentencia->bindParam(":p_tipo", $tipo);
                    $sentencia->bindParam(":p_contenedor", $contenedor);
                    $sentencia->bindParam(":p_codigo_ruta", $codigo_ruta);
                    $sentencia->bindParam(":codigo_proveedor", $codigo_proveedor);
                    $sentencia->bindParam(":codigo_ruta_transporte", $codigo_ruta_transporte);
                    $sentencia->bindParam(":p_codigo_tipo_vehiculo", $codigo_tipo_vehiculo);
                    $sentencia->bindParam(":p_exceso", $exceso);
                    $sentencia->bindParam(":p_version", $version);
                    $sentencia->bindParam(":p_codigo_empresa", $dirige);
                    $sentencia->bindParam(":p_codigo_empresa2", $dirige);
                    $sentencia->bindParam(":p_fecha_subida", $fecha_subida);
                    $sentencia->bindParam(":p_hora_subida", $hora_subida);
                    $sentencia->bindParam(":p_procedencia", $procedencia);
                    $sentencia->bindParam(":p_id_qr", $id_qr);
                }
                //Ejecutar la sentencia preparada
                $sentencia->execute();

                $sql = "UPDATE correlativo_control SET numero = numero + 1 WHERE tabla = 'garita_entrada_salida_control'";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->execute();

                $sql = "INSERT into garita_movil values(:p_codigo_entrada, :p_codigo_caseta);";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_codigo_entrada", $nuevoNumerosalida);
                $sentencia->bindParam(":p_codigo_caseta", $codigo_caseta);
                $sentencia->execute();

                $this->dblink->commit();

                return true;
            } else {
                throw new Exception("No se a configurado el correlativo para Garitas Principales");
            }
        } catch (Exception $exc) {
            $this->dblink->rollBack();
            throw  $exc;
        }
        return false;
    }

    public function agregarEntradaMovil($fecha_entrada_salida, $placa_entrada_salida, $hora_entrada_salida, $codigo_empresa, $dni_conductor, $codigo_usuario, $nombre_conductor, $codigo_tipo_vehiculo, 
    $codigo_empresa2,$codigo_caseta, $tipo, $punto_control, $codigo_ruta, $codigo_proveedor, $codigo_ruta_transporte, $exceso, $version, $fecha_subida, $hora_subida, $procedencia, $id_qr){
        $placaAlterna = "";
        $this->dblink->beginTransaction();
        try {
            /** VALIDAR CASETA DEL USUARIO **/
            $codigo_caseta = $this->obtenerEmpresaDeUsuario($codigo_usuario);
            /** VALIDAR CASETA DEL USUARIO **/
            
            /** SI NO ESTÁ REGISTRADO EL CONDUCTOR, LO REGISTRAMOS */
           
            /** SI NO ESTÁ REGISTRADO EL CONDUCTOR, LO REGISTRAMOS */

            /* OBTENER EL CÓDIGO DE RUTA */
            if ($punto_control == "N" || $punto_control == "-") {
                $holderDni = trim($dni_conductor);
                // Check out if there is a route open.... if its open then close it.
                $sql = "SELECT codigo_ruta as id, timestamp_registro as hora_registro FROM ruta WHERE dni = :p_dni and estado = 'A';";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_dni", $dni_conductor);
                $sentencia->execute();
                $resultado = $sentencia->fetch();

                if ($sentencia->rowCount()) {
                    $hora_registro_ruta = new DateTime($resultado['hora_registro']);
                    $hora_actual = new DateTime();
                    $d = $hora_actual->diff($hora_registro_ruta);

                    if ($d->y == 0 && $d->m == 0 && $d->d == 0 && $d->h > 5) {
                        $sql = "UPDATE ruta SET estado = 'C' WHERE dni = :p_dni and estado = 'A';";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->bindParam(":p_dni", $dni_conductor);
                        $sentencia->execute();

                        // registrar la ruta
                        $lugar_creacion = "G#" . $codigo_caseta;
                        $sql = "INSERT INTO ruta(dni, lugar_creacion) values(:p_dni, :p_creacion);";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->bindParam(":p_dni", $dni_conductor);
                        $sentencia->bindParam(":p_creacion", $lugar_creacion);
                        $sentencia->execute();
                    }
                } else {
                    // registrar la ruta
                    $lugar_creacion = "G#" . $codigo_caseta;
                    $sql = "INSERT INTO ruta(dni, lugar_creacion) values(:p_dni, :p_creacion);";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_dni", $dni_conductor);
                    $sentencia->bindParam(":p_creacion", $lugar_creacion);
                    $sentencia->execute();
                }

                $sql = "SELECT codigo_ruta as id FROM ruta WHERE dni = :p_dni and estado = 'A' order by codigo_ruta desc limit 1;";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_dni", $dni_conductor);
                $sentencia->execute();
                $resultado = $sentencia->fetch();

                $codigo_ruta = $resultado['id'];
            }
            /* OBTENER EL CÓDIGO DE RUTA */

            /* VERIFICAR SI SE DEBE RECORRER TODOS LOS QR O SOLO LOS DATOS ENVIADOS */
            $resultado = ""; // $this->obtenerQRConductorPorDni($dni_conductor, 'T'); // Parámetro 'T' -> Obtener todas las tarjetas QR y recorrerlas
            // TODO: REVISAR PROCESO SI SE APLICA COMO ENTRADA TODOS LOS QR
            if (!empty($resultado)){
                foreach($resultado as $item){
                    if($item["estado"] == "A"){
                        $sql = "SELECT * FROM f_generar_correlativo('garita_entrada_salida_control') as nc";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->execute();
                        $resultado = $sentencia->fetch();

                        $nuevoNumeroEntradaSalida = $resultado["nc"];

                        $sql = "INSERT INTO garita_entrada_salida_control(
                                        codigo_entrada_salida,
                                        placa_entrada_salida,
                                        fecha_entrada_salida,
                                        hora_entrada_salida,
                                        codigo_empresa,
                                        dni_conductor,
                                        codigo_usuario,
                                        nombre_conductor,
                                        codigo_tipo_vehiculo,
                                        codigo_empresa2,
                                        placa_alterna_salida,
                                        tipo,
                                        codigo_caseta,
                                        codigo_ruta,
                                        punto_control,
                                        codigo_proveedor,
                                        codigo_ruta_transporte,
                                        exceso,
                                        version_app,
                                        fecha_subida,
                                        hora_subida,
                                        procedencia,
                                        id_qr
                                    )VALUES (
                                        :p_codigo_entrada_salida,
                                        :p_placa_entrada_salida,
                                        :p_fecha_entrada_salida,
                                        :p_hora_entrada_salida,
                                        :p_codigo_empresa,
                                        :p_dni_conductor,
                                        :p_codigo_usuario,
                                        :p_nombre_conductor,
                                        :p_codigo_tipo_vehiculo,
                                        :p_codigo_empresa2,
                                        :p_placa_alterna_salida,
                                        :p_tipo,
                                        :p_codigo_caseta,
                                        :p_codigo_ruta,
                                        :p_punto_control,
                                        :codigo_proveedor,
                                        :codigo_ruta_transporte,
                                        :p_exceso,
                                        :p_version
                                        :p_fecha_subida,
                                        :p_hora_subida,
                                        :p_procedencia,
                                        :p_id_qr);";

                        $sentencia = $this->dblink->prepare($sql);

                        $sentencia->bindParam(":p_codigo_entrada_salida", $nuevoNumeroEntradaSalida);
                        $sentencia->bindParam(":p_placa_entrada_salida", $item["placa"]);
                        $sentencia->bindParam(":p_fecha_entrada_salida", $fecha_entrada_salida);
                        $sentencia->bindParam(":p_hora_entrada_salida", $hora_entrada_salida);
                        $sentencia->bindParam(":p_codigo_empresa", $item["cod_emp_pertenece"]);
                        $sentencia->bindParam(":p_dni_conductor", $item["dni"]);
                        $sentencia->bindParam(":p_codigo_usuario", $codigo_usuario);
                        $sentencia->bindParam(":p_nombre_conductor", $item["nombre_conductor"]);
                        $sentencia->bindParam(":p_codigo_tipo_vehiculo", $item["codigo_tipo_vehiculo"]);
                        $sentencia->bindParam(":p_codigo_empresa2", $item["cod_emp_pertenece"]);
                        $sentencia->bindParam(":p_placa_alterna_salida", $placaAlterna);
                        $sentencia->bindParam(":p_tipo", $tipo);
                        $sentencia->bindParam(":p_codigo_caseta", $codigo_caseta);
                        $sentencia->bindParam(":p_codigo_ruta", $codigo_ruta);
                        $sentencia->bindParam(":p_punto_control", $punto_control);
                        $sentencia->bindParam(":codigo_proveedor", $item["codigo_proveedor"]);
                        $sentencia->bindParam(":codigo_ruta_transporte", $item["codigo_ruta"]);
                        $sentencia->bindParam(":p_exceso", $exceso);
                        $sentencia->bindParam(":p_version", $version);
                        $sentencia->bindParam(":p_fecha_subida", $fecha_subida);
                        $sentencia->bindParam(":p_hora_subida", $hora_subida);
                        $sentencia->bindParam(":p_procedencia", $procedencia);
                        $sentencia->bindParam(":p_id_qr", $id_qr);

                        $sentencia->execute();

                        $sql = "UPDATE correlativo_control SET numero = numero + 1 WHERE tabla = 'garita_entrada_salida_control'";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->execute();

                        $sql = "INSERT into garita_movil values(:p_codigo_entrada, :p_codigo_caseta);";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->bindParam(":p_codigo_entrada", $nuevoNumeroEntradaSalida);
                        $sentencia->bindParam(":p_codigo_caseta", $codigo_caseta);
                        $sentencia->execute();
                    }
                }
            }else{
                $sql = "SELECT * FROM f_generar_correlativo('garita_entrada_salida_control') as nc";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->execute();
                $resultado = $sentencia->fetch();

                $nuevoNumeroEntradaSalida = $resultado["nc"];

                $sql = "INSERT INTO garita_entrada_salida_control(
                                codigo_entrada_salida,
                                placa_entrada_salida,
                                fecha_entrada_salida,
                                hora_entrada_salida,
                                codigo_empresa,
                                dni_conductor,
                                codigo_usuario,
                                nombre_conductor,
                                codigo_tipo_vehiculo,
                                codigo_empresa2,
                                placa_alterna_salida,
                                tipo,
                                codigo_caseta,
                                codigo_ruta,
                                punto_control,
                                codigo_proveedor,
                                codigo_ruta_transporte,
                                exceso,
                                version_app,
                                fecha_subida,
                                hora_subida,
                                procedencia,
                                id_qr
                            )VALUES (
                                :p_codigo_entrada_salida,
                                :p_placa_entrada_salida,
                                :p_fecha_entrada_salida,
                                :p_hora_entrada_salida,
                                :p_codigo_empresa,
                                :p_dni_conductor,
                                :p_codigo_usuario,
                                :p_nombre_conductor,
                                :p_codigo_tipo_vehiculo,
                                :p_codigo_empresa2,
                                :p_placa_alterna_salida,
                                :p_tipo,
                                :p_codigo_caseta,
                                :p_codigo_ruta,
                                :p_punto_control,
                                :codigo_proveedor,
                                :codigo_ruta_transporte,
                                :p_exceso,
                                :p_version,
                                :p_fecha_subida,
                                :p_hora_subida,
                                :p_procedencia,
                                :p_id_qr);";

                $sentencia = $this->dblink->prepare($sql);

                $sentencia->bindParam(":p_codigo_entrada_salida", $nuevoNumeroEntradaSalida);
                $sentencia->bindParam(":p_placa_entrada_salida", $placa_entrada_salida);
                $sentencia->bindParam(":p_fecha_entrada_salida", $fecha_entrada_salida);
                $sentencia->bindParam(":p_hora_entrada_salida", $hora_entrada_salida);
                $sentencia->bindParam(":p_codigo_empresa", $codigo_empresa);
                $sentencia->bindParam(":p_dni_conductor", $dni_conductor);
                $sentencia->bindParam(":p_codigo_usuario", $codigo_usuario);
                $sentencia->bindParam(":p_nombre_conductor", $nombre_conductor);
                $sentencia->bindParam(":p_codigo_tipo_vehiculo", $codigo_tipo_vehiculo);
                $sentencia->bindParam(":p_codigo_empresa2", $codigo_empresa2);
                $sentencia->bindParam(":p_placa_alterna_salida", $placaAlterna);
                $sentencia->bindParam(":p_tipo", $tipo);
                $sentencia->bindParam(":p_codigo_caseta", $codigo_caseta);
                $sentencia->bindParam(":p_codigo_ruta", $codigo_ruta);
                $sentencia->bindParam(":p_punto_control", $punto_control);
                $sentencia->bindParam(":codigo_proveedor", $codigo_proveedor);
                $sentencia->bindParam(":codigo_ruta_transporte", $codigo_ruta_transporte);
                $sentencia->bindParam(":p_exceso", $exceso);
                $sentencia->bindParam(":p_version", $version);
                $sentencia->bindParam(":p_fecha_subida", $fecha_subida);
                $sentencia->bindParam(":p_hora_subida", $hora_subida);
                $sentencia->bindParam(":p_procedencia", $procedencia);
                $sentencia->bindParam(":p_id_qr", $id_qr);
                
                $sentencia->execute();

                $sql = "UPDATE correlativo_control SET numero = numero + 1 WHERE tabla = 'garita_entrada_salida_control'";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->execute();

                $sql = "INSERT into garita_movil values(:p_codigo_entrada, :p_codigo_caseta);";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_codigo_entrada", $nuevoNumeroEntradaSalida);
                $sentencia->bindParam(":p_codigo_caseta", $codigo_caseta);
                $sentencia->execute();
            }
            /* VERIFICAR SI SE DEBE RECORRER TODOS LOS QR O SOLO LOS DATOS ENVIADOS */

            $sql = "SELECT * from alertas_empresa where dni = :p_dni;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $holderDni);
            $sentencia->execute();

            $resultado = $sentencia->fetchAll();

            if ($codigo_caseta == 1) {
                        $holderGaritaMensaje = ' Garita km 855 ';
                } else {
                       if ($codigo_caseta == 2) {
                        $holderGaritaMensaje = ' Garita Palo Verde ';
                } else {
                        if ($codigo_caseta == 3) {
                        $holderGaritaMensaje = ' Garita Morrope ';
                        } else {
                        $holderGaritaMensaje = ' Garita Km 32 ';
                        }
                    }
                }
            if ($sentencia->rowCount()) {
                foreach ($resultado as $item) {
                    $telefono = $item['telefono'];
                    $mensaje = date('h:i A') . ' ' . $holderGaritaMensaje . ' ' . $item['mensaje'];

                    $url_holder_sms = str_replace(' ', '%20', "http://servicio.smsmasivos.com.ar/enviar_sms.asp?api=1&usuario=PROOLMOS&clave=PROOLMOS352&tos=" . $telefono . "&texto=" . $mensaje);
                    $ch = curl_init();
                    curl_setopt($ch, CURLOPT_URL, $url_holder_sms);
                    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
                    $data = curl_exec($ch);
                    curl_close($ch);
                }
            }
            $this->dblink->commit();
            
           
            return true;
            
            
            
        } catch (Exception $exc) {
            $this->dblink->rollBack();
            throw  $exc;
        }
        
        return false;
    }

    public function obtenerDatosQrEntradaConductor($codigo_conductor, $codigo_empresa, $codigo_garita)
    {
        try {
            $sql = "SELECT
                                upper(v.placa) as placa,
                                $codigo_empresa as cod_emp_pertenece,
                                $codigo_empresa as cod_emp_dirige,
                                v.codigo_tipo_vehiculo,
                                upper(c.apellidos_paternos_conductor||' '||c.apellidos_maternos_conductor||', '||c.nombre_conductor) as nombre_conductor,
                                dt.estado as estado,
                                upper(trim(c.dni_conductor)) as dni
                            FROM
                                detalle_transporte as dt inner join conductor_control as c on(dt.codigo_conductor = c.codigo_conductor)
                                inner join vehiculo as v on (dt.codigo_vehiculo = v.codigo_vehiculo)
                            WHERE
                                dt.codigo_conductor = :p_codigo_conductor
                            ORDER BY
                                dt.fecha_registro desc;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_codigo_conductor", $codigo_conductor);
            $sentencia->execute();
            $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);

            if (!$sentencia->rowCount()) {
                throw new Exception("Este código QR no es válido o ha sido revocado");
            }
            return $resultado;
        } catch (Exception $e) {
            throw $e;
        }
    }

    public function obtenerDatosQrEntradaVehiculo($codigo_vehiculo, $codigo_empresa, $codigo_garita)
    {
        try {
            $sql = "SELECT
                                upper(v.placa) as placa,
                                $codigo_empresa as cod_emp_pertenece,
                                $codigo_empresa as cod_emp_dirige,
                                v.codigo_tipo_vehiculo,
                                upper(c.apellidos_paternos_conductor||' '||c.apellidos_maternos_conductor||', '||c.nombre_conductor) as nombre_conductor,
                                dt.estado as estado,
                                upper(trim(c.dni_conductor)) as dni
                            FROM
                                detalle_transporte as dt inner join conductor_control as c on(dt.codigo_conductor = c.codigo_conductor)
                                inner join vehiculo as v on (dt.codigo_vehiculo = v.codigo_vehiculo)
                            WHERE
                                dt.codigo_vehiculo = :p_codigo_vehiculo
                            ORDER BY
                                dt.fecha_registro desc;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_codigo_vehiculo", $codigo_vehiculo);
            $sentencia->execute();
            $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);

            if (!$sentencia->rowCount()) {
                throw new Exception("Este código QR no es válido o ha sido revocado");
            }

            return $resultado;
        } catch (Exception $e) {
            throw $e;
        }
    }

    public function obtenerDatosQrSalidaProveedorEmpresa($codigo_conductor, $codigo_proveedor, $codigo_empresa, $codigo_vehiculo){
        try{
            $sql = "SELECT
                        upper(c.apellidos_paternos_conductor||' '||c.apellidos_maternos_conductor||', '||c.nombre_conductor) as nomb,
                        upper(v.placa) as placa,
                        dt.estado as estado,
                        upper(trim(c.dni_conductor)) as dni
                    FROM
                        proveedor_empresa as dt inner join conductor_control as c on(dt.codigo_conductor = c.codigo_conductor)
                        inner join vehiculo as v on (dt.codigo_vehiculo = v.codigo_vehiculo)
                    WHERE
                        dt.codigo_conductor = ? and
                        dt.codigo_proveedor = ? and
                        dt.codigo_empresa = ? and
                        dt.codigo_vehiculo = ?
                    ORDER by
                        dt.fecha_registro desc;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(1, $codigo_conductor);
            $sentencia->bindParam(2, $codigo_proveedor);
            $sentencia->bindParam(3, $codigo_empresa);
            $sentencia->bindParam(4, $codigo_vehiculo);
            $sentencia->execute();
            if(!$sentencia->rowCount()){
                throw new Exception("Este código QR no es válido o ha sido revocado");
            }
            $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
            $resultadoFinal["nomb"] = $resultado["nomb"];
            $resultadoFinal["placa"] = $resultado["placa"];
            $resultadoFinal["estado"] = $resultado["estado"];
            $resultadoFinal["dni"] = $resultado["dni"];

            $sql = "SELECT
                        to_char(g.hora_entrada_salida,'HH12:MI AM')AS hora,
                        to_char(g.fecha_entrada_salida,'DD/MM/YYYY')AS fecha,
                        c.descripcion_caseta as ingreso
                    from
                        garita_entrada_salida_control as g inner join caseta_control as c on (g.codigo_caseta = c.codigo_caseta)
                    where
                        lower(trim(g.dni_conductor)) = lower(trim(:p_dni)) and g.tipo='E'
                        order by g.fecha_entrada_salida desc
                        limit 1;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $resultadoFinal["dni"]);
            $sentencia->execute();
            if ($sentencia->rowCount()) {
                $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
                $resultadoFinal["hora"] = $resultado["hora"];
                $resultadoFinal["fecha"] = $resultado["fecha"];
                $resultadoFinal["ingreso"] = $resultado["ingreso"];
            }else{
                $resultadoFinal["hora"] = date("H:i:s");
                $resultadoFinal["fecha"] = date('d/m/Y');
                $resultadoFinal["ingreso"] = "SIN CASETA DE INGRESO";
            }

            $resultadoFinal["tipo_empresa"] ="P";

            return $resultadoFinal;
        }catch(Exception $e){
            throw $e;
        }
    }

    public function obtenerDatosQrSalidaConductor($codigo_conductor, $codigo_garita){
        try{
            $sql = "SELECT
                        upper(c.apellidos_paternos_conductor||' '||c.apellidos_maternos_conductor||', '||c.nombre_conductor) as nomb,
                        upper(v.placa) as placa,
                        dt.estado as estado,
                        upper(trim(c.dni_conductor)) as dni
                    FROM
                        detalle_transporte as dt inner join conductor_control as c on(dt.codigo_conductor = c.codigo_conductor)
                        inner join vehiculo as v on (dt.codigo_vehiculo = v.codigo_vehiculo)
                    WHERE
                        dt.codigo_conductor = :codigo_conductor
                    ORDER by
                        dt.fecha_registro desc;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":codigo_conductor", $codigo_conductor);
            $sentencia->execute();
            if(!$sentencia->rowCount()){
                throw new Exception("Este código QR no es válido o ha sido revocado");
            }
            $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
            $resultadoFinal["nomb"] = $resultado["nomb"];
            $resultadoFinal["placa"] = $resultado["placa"];
            $resultadoFinal["estado"] = $resultado["estado"];
            $resultadoFinal["dni"] = $resultado["dni"];

            $sql = "SELECT
                        to_char(g.hora_entrada_salida,'HH12:MI AM')AS hora,
                        to_char(g.fecha_entrada_salida,'DD/MM/YYYY')AS fecha,
                        c.descripcion_caseta as ingreso
                    from
                        garita_entrada_salida_control as g inner join caseta_control as c on (g.codigo_caseta = c.codigo_caseta)
                    where
                        lower(trim(g.dni_conductor)) = lower(trim(:p_dni)) and g.tipo='E'
                        order by g.fecha_entrada_salida desc
                        limit 1;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $resultadoFinal["dni"]);
            $sentencia->execute();
            if ($sentencia->rowCount()) {
                $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
                $resultadoFinal["hora"] = $resultado["hora"];
                $resultadoFinal["fecha"] = $resultado["fecha"];
                $resultadoFinal["ingreso"] = $resultado["ingreso"];
            }else{
                $resultadoFinal["hora"] = date("H:i:s");
                $resultadoFinal["fecha"] = date('d/m/Y');
                $resultadoFinal["ingreso"] = "SIN CASETA DE INGRESO";
            }

            return $resultadoFinal;
        }catch(Exception $e){
            throw $e;
        }
    }

    public function obtenerDatosQrSalidaVehiculo($codigo_vehiculo, $codigo_garita){
        try{
            $sql = "SELECT
                        upper(c.apellidos_paternos_conductor||' '||c.apellidos_maternos_conductor||', '||c.nombre_conductor) as nomb,
                        upper(v.placa) as placa,
                        dt.estado as estado,
                        upper(trim(c.dni_conductor)) as dni
                    FROM
                        detalle_transporte as dt inner join conductor_control as c on(dt.codigo_conductor = c.codigo_conductor)
                        inner join vehiculo as v on (dt.codigo_vehiculo = v.codigo_vehiculo)
                    WHERE
                        dt.codigo_vehiculo = :codigo_vehiculo
                    ORDER by
                        dt.fecha_registro desc;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":codigo_vehiculo", $codigo_vehiculo);
            $sentencia->execute();
            if(!$sentencia->rowCount()){
                throw new Exception("Este código QR no es válido o ha sido revocado");
            }
            $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
            $resultadoFinal["nomb"] = $resultado["nomb"];
            $resultadoFinal["placa"] = $resultado["placa"];
            $resultadoFinal["estado"] = $resultado["estado"];
            $resultadoFinal["dni"] = $resultado["dni"];

            $sql = "SELECT
                        to_char(g.hora_entrada_salida,'HH12:MI AM')AS hora,
                        to_char(g.fecha_entrada_salida,'DD/MM/YYYY')AS fecha,
                        c.descripcion_caseta as ingreso
                    from
                        garita_entrada_salida_control as g inner join caseta_control as c on (g.codigo_caseta = c.codigo_caseta)
                    where
                        lower(trim(g.dni_conductor)) = lower(trim(:p_dni)) and g.tipo='E'
                        order by g.fecha_entrada_salida desc
                        limit 1;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $resultadoFinal["dni"]);
            $sentencia->execute();
            if ($sentencia->rowCount()) {
                $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
                $resultadoFinal["hora"] = $resultado["hora"];
                $resultadoFinal["fecha"] = $resultado["fecha"];
                $resultadoFinal["ingreso"] = $resultado["ingreso"];
            }else{
                $resultadoFinal["hora"] = date("H:i:s");
                $resultadoFinal["fecha"] = date('d/m/Y');
                $resultadoFinal["ingreso"] = "SIN CASETA DE INGRESO";
            }
            $resultadoFinal["tipo_empresa"] = "P";
            return $resultadoFinal;
        }catch(Exception $e){
            throw $e;
        }
    }

    public function entradaRapidaConductor($codigo_conductor, $codigo_empresa, $nombre_conductor, $dni_conductor, $codigo_usuario, $codigo_garita, $codigo_proveedor = 0, $codigo_ruta_transporte = 0){
        $this->dblink->beginTransaction();
        try{
            /** VALIDAR CASETA DEL USUARIO **/
            $codigo_garita = $this->obtenerEmpresaDeUsuario($codigo_usuario);
            /** VALIDAR CASETA DEL USUARIO **/
            $insertarEnRevision = "N";
            $sql = "SELECT
                                upper(v.placa) as placa,
                                v.codigo_tipo_vehiculo
                            FROM
                                detalle_transporte as dt inner join conductor_control as c on(dt.codigo_conductor = c.codigo_conductor)
                                inner join vehiculo as v on (dt.codigo_vehiculo = v.codigo_vehiculo)
                            WHERE
                                dt.codigo_conductor = :p_codigo_conductor
                            ORDER BY
                                dt.fecha_registro desc;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_codigo_conductor", $codigo_conductor);
            $sentencia->execute();

            if (!$sentencia->rowCount()) {
                $insertarEnRevision = "S";
                $placa = "";
                $codigo_tipo_vehiculo = 13;
            }else{
                $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
                $placa = $resultado["placa"];
                $codigo_tipo_vehiculo = $resultado["codigo_tipo_vehiculo"];
            }

            $hora_entrada_salida = date("H:i:s");
            $fecha_entrada_salida = date('Y-m-d');
            $holderDni = trim($dni_conductor);
            // Check out if there is a route open.... if its open then close it.
            $sql = "SELECT codigo_ruta as id, timestamp_registro as hora_registro FROM ruta WHERE trim(dni) = trim(:p_dni) and estado = 'A';";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $dni_conductor);
            $sentencia->execute();
            $resultado = $sentencia->fetch();

            if ($sentencia->rowCount()) {
                $hora_registro_ruta = new DateTime($resultado['hora_registro']);
                $hora_actual = new DateTime();
                $d = $hora_actual->diff($hora_registro_ruta);

                if ($d->y == 0 && $d->m == 0 && $d->d == 0 && $d->h > 5) {
                    $sql = "UPDATE ruta SET estado = 'C' WHERE dni = :p_dni and estado = 'A';";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_dni", $dni_conductor);
                    $sentencia->execute();

                    // registrar la ruta
                    $lugar_creacion = "G#" . $codigo_garita;
                    $sql = "INSERT INTO ruta(dni, lugar_creacion) values(:p_dni, :p_creacion);";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_dni", $dni_conductor);
                    $sentencia->bindParam(":p_creacion", $lugar_creacion);
                    $sentencia->execute();
                }
            } else {
                // registrar la ruta
                $lugar_creacion = "G#" . $codigo_garita;
                $sql = "INSERT INTO ruta(dni, lugar_creacion) values(:p_dni, :p_creacion);";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_dni", $dni_conductor);
                $sentencia->bindParam(":p_creacion", $lugar_creacion);
                $sentencia->execute();
            }

            $sql = "SELECT codigo_ruta as id FROM ruta WHERE dni = :p_dni and estado = 'A' order by codigo_ruta desc limit 1;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $dni_conductor);
            $sentencia->execute();
            $resultado = $sentencia->fetch();

            $codigo_ruta = $resultado['id'];

            $sql = "SELECT * FROM f_generar_correlativo('garita_entrada_salida_control') as nc";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->execute();
            $resultado = $sentencia->fetch();

            if ($sentencia->rowCount()) {
                $nuevoNumeroEntradaSalida = $resultado["nc"];
                $sql = "INSERT INTO garita_entrada_salida_control(
                                codigo_entrada_salida,
                                placa_entrada_salida,
                                fecha_entrada_salida,
                                hora_entrada_salida,
                                codigo_empresa,
                                dni_conductor,
                                codigo_usuario,
                                nombre_conductor,
                                codigo_tipo_vehiculo,
                                codigo_empresa2,
                                tipo,
                                codigo_caseta,
                                codigo_ruta,
                                codigo_proveedor,
                                codigo_ruta_transporte
                            )VALUES (
                                :p_codigo_entrada_salida,
                                :p_placa_entrada_salida,
                                :p_fecha_entrada_salida,
                                :p_hora_entrada_salida,
                                :p_codigo_empresa,
                                :p_dni_conductor,
                                :p_codigo_usuario,
                                :p_nombre_conductor,
                                :p_codigo_tipo_vehiculo,
                                :p_codigo_empresa2,
                                'E',
                                :p_codigo_caseta,
                                :p_codigo_ruta,
                                :codigo_proveedor,
                                :codigo_ruta_transporte
                                );";

                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_codigo_entrada_salida", $nuevoNumeroEntradaSalida);
                $sentencia->bindParam(":p_placa_entrada_salida", $placa);
                $sentencia->bindParam(":p_fecha_entrada_salida", $fecha_entrada_salida);
                $sentencia->bindParam(":p_hora_entrada_salida", $hora_entrada_salida);
                $sentencia->bindParam(":p_codigo_empresa", $codigo_empresa);
                $sentencia->bindParam(":p_dni_conductor", $dni_conductor);
                $sentencia->bindParam(":p_codigo_usuario", $codigo_usuario);
                $sentencia->bindParam(":p_nombre_conductor", $nombre_conductor);
                $sentencia->bindParam(":p_codigo_tipo_vehiculo", $codigo_tipo_vehiculo);
                $sentencia->bindParam(":p_codigo_empresa2", $codigo_empresa);
                $sentencia->bindParam(":p_codigo_caseta", $codigo_garita);
                $sentencia->bindParam(":p_codigo_ruta", $codigo_ruta);
                $sentencia->bindParam(":codigo_proveedor", $codigo_proveedor);
                $sentencia->bindParam(":codigo_ruta_transporte", $codigo_ruta_transporte);

                $sentencia->execute();

                $sql = "UPDATE correlativo_control SET numero = numero + 1 WHERE tabla = 'garita_entrada_salida_control'";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->execute();

                $sql = "INSERT into garita_movil values(:p_codigo_entrada, :p_codigo_caseta);";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_codigo_entrada", $nuevoNumeroEntradaSalida);
                $sentencia->bindParam(":p_codigo_caseta", $codigo_garita);
                $sentencia->execute();

                $sql = "SELECT * from alertas_empresa where dni = :p_dni;";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_dni", $holderDni);
                $sentencia->execute();

                $resultado = $sentencia->fetchAll();

                if ($codigo_garita == 1) {
                    $holderGaritaMensaje = ' Garita km 855 ';
                } else {
                    $holderGaritaMensaje = ' Garita palo verde ';
                }
                if ($sentencia->rowCount()) {
                    foreach ($resultado as $item) {
                        $telefono = $item['telefono'];
                        $mensaje = date('h:i A') . ' ' . $holderGaritaMensaje . ' ' . $item['mensaje'];

                        $url_holder_sms = str_replace(' ', '%20', "http://servicio.smsmasivos.com.ar/enviar_sms.asp?api=1&usuario=PROOLMOS&clave=PROOLMOS352&tos=" . $telefono . "&texto=" . $mensaje);
                        $ch = curl_init();
                        curl_setopt($ch, CURLOPT_URL, $url_holder_sms);
                        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
                        $data = curl_exec($ch);
                        curl_close($ch);
                    }
                }

                $sql = "INSERT into monitor_rapido (codigo_empresa, tipo, tipo_empresa, codigo_entrada_salida, revision) values (:codigo_empresa, 'E', 'P', :codigo, :revision);";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":codigo_empresa", $codigo_garita);
                $sentencia->bindParam(":codigo", $nuevoNumeroEntradaSalida);
                $sentencia->bindParam(":revision", $insertarEnRevision);
                $sentencia->execute();

                $sql = "SELECT nombre_empresa FROM empresa_control where codigo_empresa = :codigo_empresa;";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":codigo_empresa", $codigo_empresa);
                $sentencia->execute();
                $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);

                $this->dblink->commit();
                $datos["conductor"] = $nombre_conductor . "\n". $resultado["nombre_empresa"];
                return $datos;
            } else {
                throw new Exception("No se a configurado el correlativo para Garitas Principales");
            }
        }catch(Exception $e){
            $this->dblink->rollBack();
            throw $e;
        }
    }

    public function entradaRapidaVehiculo($codigo_vehiculo, $placa, $tipo_vehiculo, $codigo_empresa, $codigo_usuario, $codigo_garita){
        $this->dblink->beginTransaction();
        try{
            /** VALIDAR CASETA DEL USUARIO **/
            $codigo_garita = $this->obtenerEmpresaDeUsuario($codigo_usuario);
            /** VALIDAR CASETA DEL USUARIO **/
            $insertarEnRevision = "N";
            $sql = "SELECT
                                upper(c.apellidos_paternos_conductor||' '||c.apellidos_maternos_conductor||', '||c.nombre_conductor) as nombre_conductor,
                                upper(trim(c.dni_conductor)) as dni
                            FROM
                                detalle_transporte as dt inner join conductor_control as c on(dt.codigo_conductor = c.codigo_conductor)
                                inner join vehiculo as v on (dt.codigo_vehiculo = v.codigo_vehiculo)
                            WHERE
                                dt.codigo_vehiculo = :p_codigo_vehiculo
                            ORDER BY
                                dt.fecha_registro desc;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_codigo_vehiculo", $codigo_vehiculo);
            $sentencia->execute();

            if (!$sentencia->rowCount()) {
                $insertarEnRevision = "S";
                $nombre_conductor = "";
                $dni_conductor = "";
            }else{
                $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
                $nombre_conductor = $resultado["nombre_conductor"];
                $dni_conductor = $resultado["dni"];
            }

            $hora_entrada_salida = date("H:i:s");
            $fecha_entrada_salida = date('Y-m-d');
            $holderDni = trim($dni_conductor);
            // Check out if there is a route open.... if its open then close it.
            $sql = "SELECT codigo_ruta as id, timestamp_registro as hora_registro FROM ruta WHERE trim(dni) = trim(:p_dni) and estado = 'A';";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $dni_conductor);
            $sentencia->execute();
            $resultado = $sentencia->fetch();

            if ($sentencia->rowCount()) {
                $hora_registro_ruta = new DateTime($resultado['hora_registro']);
                $hora_actual = new DateTime();
                $d = $hora_actual->diff($hora_registro_ruta);

                if ($d->y == 0 && $d->m == 0 && $d->d == 0 && $d->h > 5) {
                    $sql = "UPDATE ruta SET estado = 'C' WHERE dni = :p_dni and estado = 'A';";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_dni", $dni_conductor);
                    $sentencia->execute();

                    // registrar la ruta
                    $lugar_creacion = "G#" . $codigo_garita;
                    $sql = "INSERT INTO ruta(dni, lugar_creacion) values(:p_dni, :p_creacion);";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_dni", $dni_conductor);
                    $sentencia->bindParam(":p_creacion", $lugar_creacion);
                    $sentencia->execute();
                }
            } else {
                // registrar la ruta
                $lugar_creacion = "G#" . $codigo_garita;
                $sql = "INSERT INTO ruta(dni, lugar_creacion) values(:p_dni, :p_creacion);";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_dni", $dni_conductor);
                $sentencia->bindParam(":p_creacion", $lugar_creacion);
                $sentencia->execute();
            }

            $sql = "SELECT codigo_ruta as id FROM ruta WHERE dni = :p_dni and estado = 'A' order by codigo_ruta desc limit 1;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $dni_conductor);
            $sentencia->execute();
            $resultado = $sentencia->fetch();

            $codigo_ruta = $resultado['id'];

            $sql = "SELECT * FROM f_generar_correlativo('garita_entrada_salida_control') as nc";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->execute();
            $resultado = $sentencia->fetch();

            if ($sentencia->rowCount()) {
                $nuevoNumeroEntradaSalida = $resultado["nc"];
                $sql = "INSERT INTO garita_entrada_salida_control(
                                codigo_entrada_salida,
                                placa_entrada_salida,
                                fecha_entrada_salida,
                                hora_entrada_salida,
                                codigo_empresa,
                                dni_conductor,
                                codigo_usuario,
                                nombre_conductor,
                                codigo_tipo_vehiculo,
                                codigo_empresa2,
                                tipo,
                                codigo_caseta,
                                codigo_ruta
                            )VALUES (
                                :p_codigo_entrada_salida,
                                :p_placa_entrada_salida,
                                :p_fecha_entrada_salida,
                                :p_hora_entrada_salida,
                                :p_codigo_empresa,
                                :p_dni_conductor,
                                :p_codigo_usuario,
                                :p_nombre_conductor,
                                :p_codigo_tipo_vehiculo,
                                :p_codigo_empresa2,
                                'E',
                                :p_codigo_caseta,
                                :p_codigo_ruta
                                );";

                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_codigo_entrada_salida", $nuevoNumeroEntradaSalida);
                $sentencia->bindParam(":p_placa_entrada_salida", $placa);
                $sentencia->bindParam(":p_fecha_entrada_salida", $fecha_entrada_salida);
                $sentencia->bindParam(":p_hora_entrada_salida", $hora_entrada_salida);
                $sentencia->bindParam(":p_codigo_empresa", $codigo_empresa);
                $sentencia->bindParam(":p_dni_conductor", $dni_conductor);
                $sentencia->bindParam(":p_codigo_usuario", $codigo_usuario);
                $sentencia->bindParam(":p_nombre_conductor", $nombre_conductor);
                $sentencia->bindParam(":p_codigo_tipo_vehiculo", $tipo_vehiculo);
                $sentencia->bindParam(":p_codigo_empresa2", $codigo_empresa);
                $sentencia->bindParam(":p_codigo_caseta", $codigo_garita);
                $sentencia->bindParam(":p_codigo_ruta", $codigo_ruta);

                $sentencia->execute();

                $sql = "UPDATE correlativo_control SET numero = numero + 1 WHERE tabla = 'garita_entrada_salida_control'";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->execute();

                $sql = "INSERT into garita_movil values(:p_codigo_entrada, :p_codigo_caseta);";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_codigo_entrada", $nuevoNumeroEntradaSalida);
                $sentencia->bindParam(":p_codigo_caseta", $codigo_garita);
                $sentencia->execute();

                $sql = "SELECT * from alertas_empresa where dni = :p_dni;";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_dni", $holderDni);
                $sentencia->execute();

                $resultado = $sentencia->fetchAll();

                if ($codigo_garita == 1) {
                    $holderGaritaMensaje = ' Garita km 855 ';
                } else {
                    $holderGaritaMensaje = ' Garita palo verde ';
                }
                if ($sentencia->rowCount()) {
                    foreach ($resultado as $item) {
                        $telefono = $item['telefono'];
                        $mensaje = date('h:i A') . ' ' . $holderGaritaMensaje . ' ' . $item['mensaje'];

                        $url_holder_sms = str_replace(' ', '%20', "http://servicio.smsmasivos.com.ar/enviar_sms.asp?api=1&usuario=PROOLMOS&clave=PROOLMOS352&tos=" . $telefono . "&texto=" . $mensaje);
                        $ch = curl_init();
                        curl_setopt($ch, CURLOPT_URL, $url_holder_sms);
                        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
                        $data = curl_exec($ch);
                        curl_close($ch);
                    }
                }

                $sql = "INSERT into monitor_rapido (codigo_empresa, tipo, tipo_empresa, codigo_entrada_salida, revision) values (:codigo_empresa, 'E', 'P', :codigo, :revision);";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":codigo_empresa", $codigo_garita);
                $sentencia->bindParam(":codigo", $nuevoNumeroEntradaSalida);
                $sentencia->bindParam(":revision", $insertarEnRevision);
                $sentencia->execute();

                $sql = "SELECT nombre_empresa FROM empresa_control where codigo_empresa = :codigo_empresa;";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":codigo_empresa", $codigo_empresa);
                $sentencia->execute();
                $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);

                $this->dblink->commit();
                $datos["conductor"] = $nombre_conductor . "\n". $resultado["nombre_empresa"];
                return $datos;
            } else {
                throw new Exception("No se a configurado el correlativo para Garitas Principales");
            }
        }catch(Exception $e){
            $this->dblink->rollBack();
            throw $e;
        }
    }

    public function entradaRapidaDni($dni_conductor, $codigo_garita, $codigo_usuario){
        $this->dblink->beginTransaction();
        try{
            /** VALIDAR CASETA DEL USUARIO **/
            $codigo_garita = $this->obtenerEmpresaDeUsuario($codigo_usuario);
            /** VALIDAR CASETA DEL USUARIO **/
            $insertarEnRevision = "N";
            $sql = "SELECT
                        upper(trim(placa_entrada_salida)) as placa,
                        codigo_empresa as cod_emp_pertenece,
                        codigo_empresa2 as cod_emp_dirige,
                        codigo_tipo_vehiculo,
                        fecha_entrada_salida,
                        (
                            coalesce((SELECT upper(c.apellidos_paternos_conductor||' '||c.apellidos_maternos_conductor||', '||c.nombre_conductor) FROM conductor_control as c WHERE lower(trim(c.dni_conductor)) = lower(trim(:p_dni))), 'CONDUCTOR NO ENCONTRADO')
                        ) as nombre_completo
                    from
                        garita_entrada_salida_control
                    where
                        lower(trim(dni_conductor)) = lower(trim(:p_dni)) and tipo='E'
                        order by fecha_entrada_salida desc, hora_entrada_salida desc
                        limit 1;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $dni_conductor);
            $sentencia->execute();
            $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);

            if ($sentencia->rowCount()) {
                $placa =  $resultado["placa"];
                $empresa_pert = $resultado["cod_emp_pertenece"];
                $empresa_dir = $resultado["cod_emp_dirige"];
                $tipo_vehiculo = $resultado["codigo_tipo_vehiculo"];
                $conductor = $resultado["nombre_completo"];
                $insertar_conductor = $conductor;
            } else {
                $placa = "";
                $empresa_pert = 20;
                $empresa_dir = 20;
                $tipo_vehiculo = 13;

                  $token = 'apis-token-1.aTSI1U7KEuT-6bbbCguH-4Y8TI6KS73N';

                   // Iniciar llamada a API
                   $curl = curl_init();
                
                   // Buscar dni
                   curl_setopt_array($curl, array(
                   CURLOPT_URL => 'https://api.apis.net.pe/v1/dni?numero=' . $dni_conductor,
                   CURLOPT_RETURNTRANSFER => true,
                   CURLOPT_ENCODING => '',
                   CURLOPT_MAXREDIRS => 2,
                   CURLOPT_TIMEOUT => 0,
                   CURLOPT_FOLLOWLOCATION => true,
                   CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                   CURLOPT_CUSTOMREQUEST => 'GET',
                   CURLOPT_HTTPHEADER => array(
                   'Referer: https://apis.net.pe/consulta-dni-api',
                   'Authorization: Bearer ' . $token
                   ),
                   ));
                
                   $respuesta = curl_exec($curl);
                
                   $obj = json_decode($respuesta);

                   if($respuesta === "Not Found"){
                        /* Si no fue encontrado en RENIEC se regresa un array vacío */
                        $conductor = "CONDUCTOR NO ENCONTRADO";
                        $insertar_conductor = "";
                        /* Si no fue encontrado en RENIEC se regresa un array vacío */
                    }else if($obj->{'error'} === "Nro DNI debe contener 8 digitos"){
                        /* Si no fue encontrado en RENIEC se regresa un array vacío */
                        $conductor = "CONDUCTOR NO ENCONTRADO";
                        $insertar_conductor = "";
                        /* Si no fue encontrado en RENIEC se regresa un array vacío */
                    }
                    else {
                        $conductor = $obj->{'nombre'};
                        $insertar_conductor = $obj->{'nombre'};
                    }
            
            }

            if($placa == ""){
                $insertarEnRevision = "S";
            }

            $hora_entrada_salida = date("H:i:s");
            $fecha_entrada_salida = date('Y-m-d');
            $holderDni = trim($dni_conductor);
            // Check out if there is a route open.... if its open then close it.
            $sql = "SELECT codigo_ruta as id, timestamp_registro as hora_registro FROM ruta WHERE trim(dni) = trim(:p_dni) and estado = 'A';";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $dni_conductor);
            $sentencia->execute();
            $resultado = $sentencia->fetch();

            if ($sentencia->rowCount()) {
                $hora_registro_ruta = new DateTime($resultado['hora_registro']);
                $hora_actual = new DateTime();
                $d = $hora_actual->diff($hora_registro_ruta);

                if ($d->y == 0 && $d->m == 0 && $d->d == 0 && $d->h > 5) {
                    $sql = "UPDATE ruta SET estado = 'C' WHERE dni = :p_dni and estado = 'A';";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_dni", $dni_conductor);
                    $sentencia->execute();

                    // registrar la ruta
                    $lugar_creacion = "G#" . $codigo_garita;
                    $sql = "INSERT INTO ruta(dni, lugar_creacion) values(:p_dni, :p_creacion);";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_dni", $dni_conductor);
                    $sentencia->bindParam(":p_creacion", $lugar_creacion);
                    $sentencia->execute();
                }
            } else {
                // registrar la ruta
                $lugar_creacion = "G#" . $codigo_garita;
                $sql = "INSERT INTO ruta(dni, lugar_creacion) values(:p_dni, :p_creacion);";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_dni", $dni_conductor);
                $sentencia->bindParam(":p_creacion", $lugar_creacion);
                $sentencia->execute();
            }

            $sql = "SELECT codigo_ruta as id FROM ruta WHERE dni = :p_dni and estado = 'A' order by codigo_ruta desc limit 1;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $dni_conductor);
            $sentencia->execute();
            $resultado = $sentencia->fetch();

            $codigo_ruta = $resultado['id'];

            $sql = "SELECT * FROM f_generar_correlativo('garita_entrada_salida_control') as nc";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->execute();
            $resultado = $sentencia->fetch();

            if ($sentencia->rowCount()) {
                $nuevoNumeroEntradaSalida = $resultado["nc"];
                $sql = "INSERT INTO garita_entrada_salida_control(
                                codigo_entrada_salida,
                                placa_entrada_salida,
                                fecha_entrada_salida,
                                hora_entrada_salida,
                                codigo_empresa,
                                dni_conductor,
                                codigo_usuario,
                                nombre_conductor,
                                codigo_tipo_vehiculo,
                                codigo_empresa2,
                                tipo,
                                codigo_caseta,
                                codigo_ruta
                            )VALUES (
                                :p_codigo_entrada_salida,
                                :p_placa_entrada_salida,
                                :p_fecha_entrada_salida,
                                :p_hora_entrada_salida,
                                :p_codigo_empresa,
                                :p_dni_conductor,
                                :p_codigo_usuario,
                                :p_nombre_conductor,
                                :p_codigo_tipo_vehiculo,
                                :p_codigo_empresa2,
                                'E',
                                :p_codigo_caseta,
                                :p_codigo_ruta
                                );";

                $sentencia = $this->dblink->prepare($sql);

                $sentencia->bindParam(":p_codigo_entrada_salida", $nuevoNumeroEntradaSalida);
                $sentencia->bindParam(":p_placa_entrada_salida", $placa);
                $sentencia->bindParam(":p_fecha_entrada_salida", $fecha_entrada_salida);
                $sentencia->bindParam(":p_hora_entrada_salida", $hora_entrada_salida);
                $sentencia->bindParam(":p_codigo_empresa", $empresa_pert);
                $sentencia->bindParam(":p_dni_conductor", $dni_conductor);
                $sentencia->bindParam(":p_codigo_usuario", $codigo_usuario);
                $sentencia->bindParam(":p_nombre_conductor", $insertar_conductor);
                $sentencia->bindParam(":p_codigo_tipo_vehiculo", $tipo_vehiculo);
                $sentencia->bindParam(":p_codigo_empresa2", $empresa_dir);
                $sentencia->bindParam(":p_codigo_caseta", $codigo_garita);
                $sentencia->bindParam(":p_codigo_ruta", $codigo_ruta);

                $sentencia->execute();

                $sql = "UPDATE correlativo_control SET numero = numero + 1 WHERE tabla = 'garita_entrada_salida_control'";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->execute();

                $sql = "INSERT into garita_movil values(:p_codigo_entrada, :p_codigo_caseta);";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_codigo_entrada", $nuevoNumeroEntradaSalida);
                $sentencia->bindParam(":p_codigo_caseta", $codigo_garita);
                $sentencia->execute();

                $sql = "SELECT * from alertas_empresa where dni = :p_dni;";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_dni", $holderDni);
                $sentencia->execute();

                $resultado = $sentencia->fetchAll();

                if ($codigo_garita == 1) {
                    $holderGaritaMensaje = ' Garita km 855 ';
                } else {
                    $holderGaritaMensaje = ' Garita palo verde ';
                }
                if ($sentencia->rowCount()) {
                    foreach ($resultado as $item) {
                        $telefono = $item['telefono'];
                        $mensaje = date('h:i A') . ' ' . $holderGaritaMensaje . ' ' . $item['mensaje'];

                        $url_holder_sms = str_replace(' ', '%20', "http://servicio.smsmasivos.com.ar/enviar_sms.asp?api=1&usuario=PROOLMOS&clave=PROOLMOS352&tos=" . $telefono . "&texto=" . $mensaje);
                        $ch = curl_init();
                        curl_setopt($ch, CURLOPT_URL, $url_holder_sms);
                        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
                        $data = curl_exec($ch);
                        curl_close($ch);
                    }
                }

                $sql = "INSERT into monitor_rapido (codigo_empresa, tipo, tipo_empresa, codigo_entrada_salida, revision) values (:codigo_empresa, 'E', 'P', :codigo, :revision);";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":codigo_empresa", $codigo_garita);
                $sentencia->bindParam(":codigo", $nuevoNumeroEntradaSalida);
                $sentencia->bindParam(":revision", $insertarEnRevision);
                $sentencia->execute();

                $sql = "SELECT nombre_empresa FROM empresa_control where codigo_empresa = :codigo_empresa;";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":codigo_empresa", $empresa_pert);
                $sentencia->execute();
                $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);

                $this->dblink->commit();
                $datos["conductor"] = $conductor . "\n". $resultado["nombre_empresa"];
                return $datos;
            } else {
                throw new Exception("No se a configurado el correlativo para Garitas Principales");
            }
        }catch(Exception $e){
            $this->dblink->rollBack();
            throw $e;
        }
    }

    public function salidaRapidaConductor($codigo_conductor, $codigo_empresa, $nombre_conductor, $dni_conductor, $codigo_usuario, $codigo_garita, $codigo_proveedor=0, $codigo_ruta_transporte=0){
        $this->dblink->beginTransaction();
        try{
            /** VALIDAR CASETA DEL USUARIO **/
            $codigo_garita = $this->obtenerEmpresaDeUsuario($codigo_usuario);
            /** VALIDAR CASETA DEL USUARIO **/
            $insertarEnRevision = "N";
            $sql = "SELECT
                                upper(v.placa) as placa
                            FROM
                                detalle_transporte as dt inner join conductor_control as c on(dt.codigo_conductor = c.codigo_conductor)
                                inner join vehiculo as v on (dt.codigo_vehiculo = v.codigo_vehiculo)
                            WHERE
                                dt.codigo_conductor = :p_codigo_conductor
                            ORDER BY
                                dt.fecha_registro desc;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_codigo_conductor", $codigo_conductor);
            $sentencia->execute();

            if (!$sentencia->rowCount()) {
                $insertarEnRevision = "S";
                $placa = "";
            }else{
                $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
                $placa = $resultado["placa"];
            }

            $hora_entrada_salida = date("H:i:s");
            $fecha_entrada_salida = date('Y-m-d');

            /**SALIDA */
            $sql = "SELECT codigo_ruta as id FROM ruta WHERE dni = lower(trim(:p_dni)) and estado = 'A' order by timestamp_registro desc;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $dni_conductor);
            $sentencia->execute();
            $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);

            $codigo_ruta = "";

            if ($sentencia->rowCount()) {
                $codigo_ruta = $resultado['id'];
                $sql = "UPDATE ruta SET estado = 'C' WHERE dni = lower(trim(:p_dni));";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_dni", $dni_conductor);
                $sentencia->execute();
            }

            $sql = "SELECT * FROM f_generar_correlativo('garita_entrada_salida_control') AS nc";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->execute();
            $resultado = $sentencia->fetch();

            if ($sentencia->rowCount()) {
                $nuevoNumerosalida = $resultado["nc"];
                $tipo = "S";
                if ($codigo_ruta == "") {
                    $sql = "INSERT INTO garita_entrada_salida_control(codigo_entrada_salida, dni_conductor, placa_entrada_salida, nombre_conductor, codigo_usuario, hora_entrada_salida, fecha_entrada_salida, codigo_caseta, tipo, codigo_proveedor, codigo_ruta_transporte) VALUES (:p_codigo_entrada_salida, :p_dni_conductor, :p_placa_entrada_salida, :p_nombre_conductor, :p_codigo_usuario, :p_hora_entrada_salida, :p_fecha_entrada_salida, :p_codigo_caseta, :p_tipo, :codigo_proveedor, :codigo_ruta_transporte);";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_codigo_entrada_salida", $nuevoNumerosalida);
                    $sentencia->bindParam(":p_dni_conductor", $dni_conductor);
                    $sentencia->bindParam(":p_placa_entrada_salida", $placa);
                    $sentencia->bindParam(":p_nombre_conductor", $nombre_conductor);
                    $sentencia->bindParam(":p_codigo_usuario",  $codigo_usuario);
                    $sentencia->bindParam(":p_hora_entrada_salida",  $hora_entrada_salida);
                    $sentencia->bindParam(":p_fecha_entrada_salida", $fecha_entrada_salida);
                    $sentencia->bindParam(":p_codigo_caseta", $codigo_garita);
                    $sentencia->bindParam(":p_tipo", $tipo);
                    $sentencia->bindParam(":codigo_proveedor", $codigo_proveedor);
                    $sentencia->bindParam(":codigo_ruta_transporte", $codigo_ruta_transporte);
                } else {
                    $sql = "INSERT INTO garita_entrada_salida_control(codigo_entrada_salida, dni_conductor, placa_entrada_salida, nombre_conductor, codigo_usuario, hora_entrada_salida, fecha_entrada_salida, codigo_caseta, tipo, codigo_ruta, codigo_proveedor, codigo_ruta_transporte) VALUES (:p_codigo_entrada_salida, :p_dni_conductor, :p_placa_entrada_salida, :p_nombre_conductor, :p_codigo_usuario, :p_hora_entrada_salida, :p_fecha_entrada_salida, :p_codigo_caseta, :p_tipo,:p_codigo_ruta, :codigo_proveedor, :codigo_ruta_transporte);";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_codigo_entrada_salida", $nuevoNumerosalida);
                    $sentencia->bindParam(":p_dni_conductor", $dni_conductor);
                    $sentencia->bindParam(":p_placa_entrada_salida", $placa);
                    $sentencia->bindParam(":p_nombre_conductor", $nombre_conductor);
                    $sentencia->bindParam(":p_codigo_usuario",  $codigo_usuario);
                    $sentencia->bindParam(":p_hora_entrada_salida",  $hora_entrada_salida);
                    $sentencia->bindParam(":p_fecha_entrada_salida", $fecha_entrada_salida);
                    $sentencia->bindParam(":p_codigo_caseta", $codigo_garita);
                    $sentencia->bindParam(":p_tipo", $tipo);
                    $sentencia->bindParam(":codigo_proveedor", $codigo_proveedor);
                    $sentencia->bindParam(":codigo_ruta_transporte", $codigo_ruta_transporte);
                    $sentencia->bindParam(":p_codigo_ruta", $codigo_ruta);
                }
                //Ejecutar la sentencia preparada
                $sentencia->execute();

                $sql = "UPDATE correlativo_control SET numero = numero + 1 WHERE tabla = 'garita_entrada_salida_control'";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->execute();

                $sql = "INSERT into garita_movil values(:p_codigo_entrada, :p_codigo_caseta);";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_codigo_entrada", $nuevoNumerosalida);
                $sentencia->bindParam(":p_codigo_caseta", $codigo_garita);
                $sentencia->execute();

                $sql = "INSERT into monitor_rapido (codigo_empresa, tipo, tipo_empresa, codigo_entrada_salida, revision) values (:codigo_empresa, 'E', 'P', :codigo, :revision);";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":codigo_empresa", $codigo_garita);
                $sentencia->bindParam(":codigo", $nuevoNumerosalida);
                $sentencia->bindParam(":revision", $insertarEnRevision);
                $sentencia->execute();

                $sql = "SELECT nombre_empresa FROM empresa_control where codigo_empresa = :codigo_empresa;";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":codigo_empresa", $codigo_empresa);
                $sentencia->execute();
                $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);

                $this->dblink->commit();
                $datos["conductor"] = $nombre_conductor . "\n". $resultado["nombre_empresa"];
                return $datos;
            } else {
                throw new Exception("No se a configurado el correlativo para Garitas Principales");
            }
            /**SALIDA */
        }catch(Exception $e){
            $this->dblink->rollBack();
            throw $e;
        }
    }

    public function salidaRapidaVehiculo($codigo_vehiculo, $placa, $tipo_vehiculo, $codigo_empresa, $codigo_usuario, $codigo_garita){
        $this->dblink->beginTransaction();
        try{
            /** VALIDAR CASETA DEL USUARIO **/
            $codigo_garita = $this->obtenerEmpresaDeUsuario($codigo_usuario);
            /** VALIDAR CASETA DEL USUARIO **/
            $insertarEnRevision = "N";
            $sql = "SELECT
                                upper(c.apellidos_paternos_conductor||' '||c.apellidos_maternos_conductor||', '||c.nombre_conductor) as nombre_conductor,
                                upper(trim(c.dni_conductor)) as dni
                            FROM
                                detalle_transporte as dt inner join conductor_control as c on(dt.codigo_conductor = c.codigo_conductor)
                                inner join vehiculo as v on (dt.codigo_vehiculo = v.codigo_vehiculo)
                            WHERE
                                dt.codigo_vehiculo = :p_codigo_vehiculo
                            ORDER BY
                                dt.fecha_registro desc;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_codigo_vehiculo", $codigo_vehiculo);
            $sentencia->execute();

            if (!$sentencia->rowCount()) {
                $insertarEnRevision = "S";
                $nombre_conductor = "";
                $dni_conductor = "";
            }else{
                $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
                $nombre_conductor = $resultado["nombre_conductor"];
                $dni_conductor = $resultado["dni"];
            }

            $hora_entrada_salida = date("H:i:s");
            $fecha_entrada_salida = date('Y-m-d');

            /**SALIDA */
            $sql = "SELECT codigo_ruta as id FROM ruta WHERE dni = lower(trim(:p_dni)) and estado = 'A' order by timestamp_registro desc;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $dni_conductor);
            $sentencia->execute();
            $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);

            $codigo_ruta = "";

            if ($sentencia->rowCount()) {
                $codigo_ruta = $resultado['id'];
                $sql = "UPDATE ruta SET estado = 'C' WHERE dni = lower(trim(:p_dni));";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_dni", $dni_conductor);
                $sentencia->execute();
            }

            $sql = "SELECT * FROM f_generar_correlativo('garita_entrada_salida_control') AS nc";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->execute();
            $resultado = $sentencia->fetch();

            if ($sentencia->rowCount()) {
                $nuevoNumerosalida = $resultado["nc"];
                $tipo = "S";
                if ($codigo_ruta == "") {
                    $sql = "INSERT INTO garita_entrada_salida_control(codigo_entrada_salida, dni_conductor, placa_entrada_salida, nombre_conductor, codigo_usuario, hora_entrada_salida, fecha_entrada_salida, codigo_caseta, tipo) VALUES (:p_codigo_entrada_salida, :p_dni_conductor, :p_placa_entrada_salida, :p_nombre_conductor, :p_codigo_usuario, :p_hora_entrada_salida, :p_fecha_entrada_salida, :p_codigo_caseta, :p_tipo);";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_codigo_entrada_salida", $nuevoNumerosalida);
                    $sentencia->bindParam(":p_dni_conductor", $dni_conductor);
                    $sentencia->bindParam(":p_placa_entrada_salida", $placa);
                    $sentencia->bindParam(":p_nombre_conductor", $nombre_conductor);
                    $sentencia->bindParam(":p_codigo_usuario",  $codigo_usuario);
                    $sentencia->bindParam(":p_hora_entrada_salida",  $hora_entrada_salida);
                    $sentencia->bindParam(":p_fecha_entrada_salida", $fecha_entrada_salida);
                    $sentencia->bindParam(":p_codigo_caseta", $codigo_garita);
                    $sentencia->bindParam(":p_tipo", $tipo);
                } else {
                    $sql = "INSERT INTO garita_entrada_salida_control(codigo_entrada_salida, dni_conductor, placa_entrada_salida, nombre_conductor, codigo_usuario, hora_entrada_salida, fecha_entrada_salida, codigo_caseta, tipo, codigo_ruta) VALUES (:p_codigo_entrada_salida, :p_dni_conductor, :p_placa_entrada_salida, :p_nombre_conductor, :p_codigo_usuario, :p_hora_entrada_salida, :p_fecha_entrada_salida, :p_codigo_caseta, :p_tipo,:p_codigo_ruta);";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_codigo_entrada_salida", $nuevoNumerosalida);
                    $sentencia->bindParam(":p_dni_conductor", $dni_conductor);
                    $sentencia->bindParam(":p_placa_entrada_salida", $placa);
                    $sentencia->bindParam(":p_nombre_conductor", $nombre_conductor);
                    $sentencia->bindParam(":p_codigo_usuario",  $codigo_usuario);
                    $sentencia->bindParam(":p_hora_entrada_salida",  $hora_entrada_salida);
                    $sentencia->bindParam(":p_fecha_entrada_salida", $fecha_entrada_salida);
                    $sentencia->bindParam(":p_codigo_caseta", $codigo_garita);
                    $sentencia->bindParam(":p_tipo", $tipo);
                    $sentencia->bindParam(":p_codigo_ruta", $codigo_ruta);
                }
                //Ejecutar la sentencia preparada
                $sentencia->execute();

                $sql = "UPDATE correlativo_control SET numero = numero + 1 WHERE tabla = 'garita_entrada_salida_control'";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->execute();

                $sql = "INSERT into garita_movil values(:p_codigo_entrada, :p_codigo_caseta);";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_codigo_entrada", $nuevoNumerosalida);
                $sentencia->bindParam(":p_codigo_caseta", $codigo_garita);
                $sentencia->execute();

                $sql = "INSERT into monitor_rapido (codigo_empresa, tipo, tipo_empresa, codigo_entrada_salida, revision) values (:codigo_empresa, 'E', 'P', :codigo, :revision);";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":codigo_empresa", $codigo_garita);
                $sentencia->bindParam(":codigo", $nuevoNumerosalida);
                $sentencia->bindParam(":revision", $insertarEnRevision);
                $sentencia->execute();

                $sql = "SELECT nombre_empresa FROM empresa_control where codigo_empresa = :codigo_empresa;";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":codigo_empresa", $codigo_empresa);
                $sentencia->execute();
                $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);

                $this->dblink->commit();
                $datos["conductor"] = $nombre_conductor . "\n". $resultado["nombre_empresa"];
                return $datos;
            } else {
                throw new Exception("No se a configurado el correlativo para Garitas Principales");
            }
            /**SALIDA */
        }catch(Exception $e){
            $this->dblink->rollBack();
            throw $e;
        }
    }

    public function salidaRapidaDni($dni_conductor, $codigo_garita, $codigo_usuario){
        $this->dblink->beginTransaction();
        try{
            /** VALIDAR CASETA DEL USUARIO **/
            $codigo_garita = $this->obtenerEmpresaDeUsuario($codigo_usuario);
            /** VALIDAR CASETA DEL USUARIO **/
            $insertarEnRevision = "N";
            $sql = "SELECT
                        upper(trim(placa_entrada_salida)) as placa,
                        codigo_empresa as cod_emp_pertenece,
                        codigo_empresa2 as cod_emp_dirige,
                        codigo_tipo_vehiculo,
                        fecha_entrada_salida,
                        (
                            coalesce((SELECT upper(c.apellidos_paternos_conductor||' '||c.apellidos_maternos_conductor||', '||c.nombre_conductor) FROM conductor_control as c WHERE lower(trim(c.dni_conductor)) = lower(trim(:p_dni))), 'CONDUCTOR NO ENCONTRADO')
                        ) as nombre_completo
                    from
                        garita_entrada_salida_control
                    where
                        lower(trim(dni_conductor)) = lower(trim(:p_dni)) and tipo='E'
                        order by fecha_entrada_salida desc, hora_entrada_salida desc
                        limit 1;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $dni_conductor);
            $sentencia->execute();
            $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);

            if ($sentencia->rowCount()) {
                $placa =  $resultado["placa"];
                $empresa_pert = $resultado["cod_emp_pertenece"];
                $empresa_dir = $resultado["cod_emp_dirige"];
                $tipo_vehiculo = $resultado["codigo_tipo_vehiculo"];
                $conductor = $resultado["nombre_completo"];
                $insertar_conductor = $conductor;
            } else {
                $placa = "";
                $empresa_pert = 20;
                $empresa_dir = 20;
                $tipo_vehiculo = 13;

                $token = 'apis-token-1.aTSI1U7KEuT-6bbbCguH-4Y8TI6KS73N';

                   // Iniciar llamada a API
                   $curl = curl_init();
                
                   // Buscar dni
                   curl_setopt_array($curl, array(
                   CURLOPT_URL => 'https://api.apis.net.pe/v1/dni?numero=' . $dni_conductor,
                   CURLOPT_RETURNTRANSFER => true,
                   CURLOPT_ENCODING => '',
                   CURLOPT_MAXREDIRS => 2,
                   CURLOPT_TIMEOUT => 0,
                   CURLOPT_FOLLOWLOCATION => true,
                   CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                   CURLOPT_CUSTOMREQUEST => 'GET',
                   CURLOPT_HTTPHEADER => array(
                   'Referer: https://apis.net.pe/consulta-dni-api',
                   'Authorization: Bearer ' . $token
                   ),
                   ));
                
                   $respuesta = curl_exec($curl);
                
                   $obj = json_decode($respuesta);

                   if($respuesta === "Not Found"){
                        /* Si no fue encontrado en RENIEC se regresa un array vacío */
                        $conductor = "CONDUCTOR NO ENCONTRADO";
                        $insertar_conductor = "";
                        /* Si no fue encontrado en RENIEC se regresa un array vacío */
                    }else if($obj->{'error'} === "Nro DNI debe contener 8 digitos"){
                        /* Si no fue encontrado en RENIEC se regresa un array vacío */
                        $conductor = "CONDUCTOR NO ENCONTRADO";
                        $insertar_conductor = "";
                        /* Si no fue encontrado en RENIEC se regresa un array vacío */
                    }
                    else {
                        $conductor = $obj->{'nombre'};
                        $insertar_conductor = $obj->{'nombre'};
                    }
            
            }

            if($placa == ""){
                $insertarEnRevision = "S";
            }

            $hora_entrada_salida = date("H:i:s");
            $fecha_entrada_salida = date('Y-m-d');

            /**SALIDA */
            $sql = "SELECT codigo_ruta as id FROM ruta WHERE dni = lower(trim(:p_dni)) and estado = 'A' order by timestamp_registro desc;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $dni_conductor);
            $sentencia->execute();
            $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);

            $codigo_ruta = "";

            if ($sentencia->rowCount()) {
                $codigo_ruta = $resultado['id'];
                $sql = "UPDATE ruta SET estado = 'C' WHERE dni = lower(trim(:p_dni));";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_dni", $dni_conductor);
                $sentencia->execute();
            }

            $sql = "SELECT * FROM f_generar_correlativo('garita_entrada_salida_control') AS nc";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->execute();
            $resultado = $sentencia->fetch();

            if ($sentencia->rowCount()) {
                $nuevoNumerosalida = $resultado["nc"];
                $tipo = "S";
                if ($codigo_ruta == "") {
                    $sql = "INSERT INTO garita_entrada_salida_control(codigo_entrada_salida, dni_conductor, placa_entrada_salida, nombre_conductor, codigo_usuario, hora_entrada_salida, fecha_entrada_salida, codigo_caseta, tipo) VALUES (:p_codigo_entrada_salida, :p_dni_conductor, :p_placa_entrada_salida, :p_nombre_conductor, :p_codigo_usuario, :p_hora_entrada_salida, :p_fecha_entrada_salida, :p_codigo_caseta, :p_tipo);";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_codigo_entrada_salida", $nuevoNumerosalida);
                    $sentencia->bindParam(":p_dni_conductor", $dni_conductor);
                    $sentencia->bindParam(":p_placa_entrada_salida", $placa);
                    $sentencia->bindParam(":p_nombre_conductor", $insertar_conductor);
                    $sentencia->bindParam(":p_codigo_usuario",  $codigo_usuario);
                    $sentencia->bindParam(":p_hora_entrada_salida",  $hora_entrada_salida);
                    $sentencia->bindParam(":p_fecha_entrada_salida", $fecha_entrada_salida);
                    $sentencia->bindParam(":p_codigo_caseta", $codigo_garita);
                    $sentencia->bindParam(":p_tipo", $tipo);
                } else {
                    $sql = "INSERT INTO garita_entrada_salida_control(codigo_entrada_salida, dni_conductor, placa_entrada_salida, nombre_conductor, codigo_usuario, hora_entrada_salida, fecha_entrada_salida, codigo_caseta, tipo, codigo_ruta) VALUES (:p_codigo_entrada_salida, :p_dni_conductor, :p_placa_entrada_salida, :p_nombre_conductor, :p_codigo_usuario, :p_hora_entrada_salida, :p_fecha_entrada_salida, :p_codigo_caseta, :p_tipo,:p_codigo_ruta);";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_codigo_entrada_salida", $nuevoNumerosalida);
                    $sentencia->bindParam(":p_dni_conductor", $dni_conductor);
                    $sentencia->bindParam(":p_placa_entrada_salida", $placa);
                    $sentencia->bindParam(":p_nombre_conductor", $insertar_conductor);
                    $sentencia->bindParam(":p_codigo_usuario",  $codigo_usuario);
                    $sentencia->bindParam(":p_hora_entrada_salida",  $hora_entrada_salida);
                    $sentencia->bindParam(":p_fecha_entrada_salida", $fecha_entrada_salida);
                    $sentencia->bindParam(":p_codigo_caseta", $codigo_garita);
                    $sentencia->bindParam(":p_tipo", $tipo);
                    $sentencia->bindParam(":p_codigo_ruta", $codigo_ruta);
                }
                //Ejecutar la sentencia preparada
                $sentencia->execute();

                $sql = "UPDATE correlativo_control SET numero = numero + 1 WHERE tabla = 'garita_entrada_salida_control'";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->execute();

                $sql = "INSERT into garita_movil values(:p_codigo_entrada, :p_codigo_caseta);";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_codigo_entrada", $nuevoNumerosalida);
                $sentencia->bindParam(":p_codigo_caseta", $codigo_garita);
                $sentencia->execute();

                $sql = "INSERT into monitor_rapido (codigo_empresa, tipo, tipo_empresa, codigo_entrada_salida, revision) values (:codigo_empresa, 'E', 'P', :codigo, :revision);";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":codigo_empresa", $codigo_garita);
                $sentencia->bindParam(":codigo", $nuevoNumerosalida);
                $sentencia->bindParam(":revision", $insertarEnRevision);
                $sentencia->execute();

                $this->dblink->commit();
                $datos["conductor"] = $conductor;
                return $datos;
            } else {
                throw new Exception("No se a configurado el correlativo para Garitas Principales");
            }
            /**SALIDA */
        }catch(Exception $e){
            $this->dblink->rollBack();
            throw $e;
        }
    }

    public function reportarInfraccion($dni, $cod_usu, $caseta, $tipo_caseta, $desc_inc)
    {
        $this->dblink->beginTransaction();
        try{
            /** VALIDAR CASETA DEL USUARIO **/
            $caseta = $this->obtenerEmpresaDeUsuario($cod_usu);
            /** VALIDAR CASETA DEL USUARIO **/
            $sql = "SELECT numero + 1 as nc from correlativo_control where tabla = 'reporte_infraccion_conductor';";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->execute();

            if (!$sentencia->rowCount()) {
                $sql = "INSERT into correlativo_control(numero, tabla) values(0, 'reporte_infraccion_conductor');";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->execute();
                $codigo = 1;
            } else {
                $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
                $codigo = $resultado["nc"];
            }

            $fecha_registro = date("Y-m-d H:i:s");
            $sql = "INSERT into reporte_infraccion_conductor(codigo_reporte_infraccion, dni_conductor, codigo_usuario, codigo_caseta, tipo_caseta, descripcion, fecha_registro)
                                    values(?, ?, ?, ?, ?, ?, ?);";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(1, $codigo);
            $sentencia->bindParam(2, $dni);
            $sentencia->bindParam(3, $cod_usu);
            $sentencia->bindParam(4, $caseta);
            $sentencia->bindParam(5, $tipo_caseta);
            $sentencia->bindParam(6, $desc_inc);
            $sentencia->bindParam(7, $fecha_registro);
            $sentencia->execute();

            $sql = "UPDATE correlativo_control SET numero = numero + 1 where tabla = 'reporte_infraccion_conductor'";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->execute();
            $this->dblink->commit();
            return "Registradoo y Reportado";
        }catch(Exception $e){
            $this->dblink->rollBack();
            throw $e;
        }
    }

    public function obtenerEmpresaDeUsuario($codigo_usuario){
        try{
            $sql = "SELECT
                        COALESCE(codigo_empresa, '0') as codigo_empresa
                    from
                        usuario_control
                    where codigo_usuario = ?";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(1, $codigo_usuario);
            $sentencia->execute();
            if(!$sentencia->rowCount()){
                return 0;
            }
            $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
            return $resultado["codigo_empresa"];
        }catch(Exception $e){
            throw $e;
        }
    }
    
    
    
// nuevas funciones para calulo de hora

     function ObtenerFechaHoraActual(){
		try{
					$sql = "select to_char (now()::date, 'YYYY-MM-DD') AS FECHA, to_char(now()::time without time zone, 'HH24:MI:SS') as hora;";

						$sentencia = $this->dblink->prepare($sql);
                        $sentencia->execute();
                        $resultado = $sentencia->fetchAll(PDO::FETCH_ASSOC);
			return $resultado;
		}catch(Exception $exc){
				throw $exc;
        }
	}
	
	
    function ObtenerTipoES($dni){
		try{
		    
					$sql = "SELECT codigo_caseta, tipo 
                            FROM public.garita_entrada_salida_control 
                            WHERE 
                            fecha_entrada_salida = current_date 
                            and lower(trim(dni_conductor)) = lower(trim(:p_dni_conductor)) 
                              AND estado_entrada = 'E' 
                              AND codigo_caseta IN (1, 2, 7)
                            ORDER BY codigo_entrada_salida DESC 
                            LIMIT 1;";
                 	$sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_dni_conductor", $dni);
                    $sentencia->execute();
                    $resultado = $sentencia->fetchAll(PDO::FETCH_ASSOC);
			return $resultado;
		}catch(Exception $exc){
				throw $exc;
        }
	}
	
	function obtenerTiempoRuta($tipo){
		try{
					$sql = "select tiempo from recorrido_garitas_principales where tipo = :p_tipo";
                 	$sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_tipo", $tipo);
                    $sentencia->execute();
                    $resultado = $sentencia->fetchAll(PDO::FETCH_ASSOC);
			return $resultado;
		}catch(Exception $exc){
				throw $exc;
        }
	}

     function ObtenerVelocidadEntrada($dni){
		try{
					$sql = "
					SELECT 
					g1.fecha_entrada_salida, 
					g1.dni_conductor,
					g1.nombre_conductor, 
					g1.vehiculo1,
					g1.placa1,
					g1.km,
					g1.tiempo_permitido,
					g1.hora_inicial, 
					g1.codigo_tipo_vehiculo,
					g1.garita_inicial as garita_ingreso, 
				    g2.hora_final, 
					g2.garita_final as garita_llegada, 
					g2.vehiculo2,
					g2.placa2,
					(g2.hora_final-g1.hora_inicial) as tiempo, 
					(case when (g2.hora_final-g1.hora_inicial) < g1.tiempo_permitido then 'Exceso de Velocidad' else 'Velocidad Permitida' end ) as limite_velocidad
					
					from (
					Select
					g.fecha_entrada_salida,
					g.dni_conductor,
					g.nombre_conductor,
					g.placa_entrada_salida as placa1,
					g.hora_entrada_salida as hora_inicial,
					t.codigo_tipo_vehiculo,
					t.descripcion_tipo_vehiculo as vehiculo1,
					t.descripcion_velocidad as km,
				    t.tiempo_permitido,	
					c.descripcion_caseta as garita_inicial
					from
					garita_entrada_salida_control g
					inner join caseta_control c on (g.codigo_caseta=c.codigo_caseta)
					inner join tipo_vehiculo_control t on (g.codigo_tipo_vehiculo = t.codigo_tipo_vehiculo)
					where
					g.dni_conductor = :p_dni_conductor and 
					g.codigo_caseta = 1 and g.tipo='E' and fecha_entrada_salida >= '2023-01-01'
					group by g.fecha_entrada_salida,g.dni_conductor,g.nombre_conductor,	g.placa_entrada_salida,g.hora_entrada_salida,t.descripcion_tipo_vehiculo,t.codigo_tipo_vehiculo,t.descripcion_velocidad, t.tiempo_permitido,c.descripcion_caseta
					order by fecha_entrada_salida desc, hora_entrada_salida desc	
					limit 1
					)g1
					
					LEFT JOIN (
					
					select
					g.fecha_entrada_salida,
					g.dni_conductor,
					g.placa_entrada_salida as placa2 ,
					t.descripcion_tipo_vehiculo as vehiculo2,
					g.hora_entrada_salida as hora_final,
					c.descripcion_caseta as garita_final
					from
					garita_entrada_salida_control g
					inner join caseta_control c on (g.codigo_caseta=c.codigo_caseta)
					inner join tipo_vehiculo_control t on (g.codigo_tipo_vehiculo = t.codigo_tipo_vehiculo)
					where
					g.dni_conductor = :p_dni_conductor and 
					g.codigo_caseta = 4 and g.tipo='E' and fecha_entrada_salida >= '2023-01-01'
					group by g.fecha_entrada_salida, g.dni_conductor,	g.placa_entrada_salida,g.hora_entrada_salida,t.descripcion_tipo_vehiculo,c.descripcion_caseta
					order by fecha_entrada_salida desc, hora_entrada_salida desc
					limit 1
					) g2
					on
					g1.dni_conductor=g2.dni_conductor;
					
					";

						$sentencia = $this->dblink->prepare($sql);
                        $sentencia->bindParam(":p_dni_conductor", $dni);
                        $sentencia->execute();
                        $resultado = $sentencia->fetchAll(PDO::FETCH_ASSOC);
			return $resultado;
		}catch(Exception $exc){
				throw $exc;
        }
	}
	
	 function obtenerUltimoDato($dni){
		try{
					$sql = "SELECT
                                ges.codigo_entrada_salida,
                                ges.fecha_entrada_salida,
                                ges.hora_entrada_salida as hora_inicial,
                                ges.dni_conductor,
                                ges.nombre_conductor,
                                ges.tipo,
                                ges.estado_entrada,
                                ges.codigo_tipo_vehiculo,
                                tvc.descripcion_tipo_vehiculo as vehiculo1,
                                ges.placa_entrada_salida,
                                tvc.tiempo_permitido,
                                ges.codigo_caseta
                            FROM
                                public.garita_entrada_salida_control ges
                            JOIN
                                public.tipo_vehiculo_control tvc ON ges.codigo_tipo_vehiculo = tvc.codigo_tipo_vehiculo
                            WHERE
                                ges.dni_conductor = :p_dni_conductor and estado_entrada = 'E' and fecha_entrada_salida >= '2023-01-01'
                            ORDER BY
                                ges.codigo_entrada_salida desc
                            LIMIT 1;
                            ";

						$sentencia = $this->dblink->prepare($sql);
                        $sentencia->bindParam(":p_dni_conductor", $dni);
                        $sentencia->execute();
                        $resultado = $sentencia->fetchAll(PDO::FETCH_ASSOC);
			return $resultado;
		}catch(Exception $exc){
				throw $exc;
        }
	}
	
	function ObtenerVelocidadEntradagam_gm($dni){
		try{
					$sql = "
					SELECT 
					g1.fecha_entrada_salida, 
					g1.dni_conductor,
					g1.nombre_conductor, 
					g1.vehiculo1,
					g1.placa1,
					g1.km,
					g1.tiempo_permitido,
					g1.hora_inicial, 
					g1.codigo_tipo_vehiculo,
					g1.garita_inicial as garita_ingreso, 
				    g2.hora_final, 
					g2.garita_final as garita_llegada, 
					g2.vehiculo2,
					g2.placa2,
					(g2.hora_final-g1.hora_inicial) as tiempo, 
					(case when (g2.hora_final-g1.hora_inicial) < g1.tiempo_permitido then 'Exceso de Velocidad' else 'Velocidad Permitida' end ) as limite_velocidad
					
					from (
					Select
					g.fecha_entrada_salida,
					g.dni_conductor,
					g.nombre_conductor,
					g.placa_entrada_salida as placa1,
					g.hora_entrada_salida as hora_inicial,
					t.codigo_tipo_vehiculo,
					t.descripcion_tipo_vehiculo as vehiculo1,
					t.descripcion_velocidad as km,
				    t.tiempo_permitido,	
					c.descripcion_caseta as garita_inicial
					from
					garita_entrada_salida_control g
					inner join caseta_control c on (g.codigo_caseta=c.codigo_caseta)
					inner join tipo_vehiculo_control t on (g.codigo_tipo_vehiculo = t.codigo_tipo_vehiculo)
					where
					g.dni_conductor = :p_dni_conductor and 
					g.codigo_caseta = 7 and g.tipo='E' and fecha_entrada_salida >= '2023-01-01'
					group by g.fecha_entrada_salida,g.dni_conductor,g.nombre_conductor,	g.placa_entrada_salida,g.hora_entrada_salida,t.descripcion_tipo_vehiculo,t.codigo_tipo_vehiculo,t.descripcion_velocidad, t.tiempo_permitido,c.descripcion_caseta
					order by fecha_entrada_salida desc, hora_entrada_salida desc	
					limit 1
					)g1
					
					LEFT JOIN (
					
					select
					g.fecha_entrada_salida,
					g.dni_conductor,
					g.placa_entrada_salida as placa2 ,
					t.descripcion_tipo_vehiculo as vehiculo2,
					g.hora_entrada_salida as hora_final,
					c.descripcion_caseta as garita_final
					from
					garita_entrada_salida_control g
					inner join caseta_control c on (g.codigo_caseta=c.codigo_caseta)
					inner join tipo_vehiculo_control t on (g.codigo_tipo_vehiculo = t.codigo_tipo_vehiculo)
					where
					g.dni_conductor = :p_dni_conductor and 
					g.codigo_caseta = 3 and g.tipo='E' and fecha_entrada_salida >= '2023-01-01'
					group by g.fecha_entrada_salida, g.dni_conductor,	g.placa_entrada_salida,g.hora_entrada_salida,t.descripcion_tipo_vehiculo,c.descripcion_caseta
					order by fecha_entrada_salida desc, hora_entrada_salida desc
					limit 1
					) g2
					on
					g1.dni_conductor=g2.dni_conductor;
					
					";

						$sentencia = $this->dblink->prepare($sql);
                        $sentencia->bindParam(":p_dni_conductor", $dni);
                        $sentencia->execute();
                        $resultado = $sentencia->fetchAll(PDO::FETCH_ASSOC);
			return $resultado;
		}catch(Exception $exc){
				throw $exc;
    }
	}
	
	
	 function ObtenerVelocidadSalida($dni){
		try{
					$sql = "
                    SELECT 
					g1.fecha_entrada_salida, 
					g1.dni_conductor,
					g1.nombre_conductor, 
					g1.vehiculo1,
					g1.placa1,
					g1.km,
					g1.tiempo_permitido,
					g1.hora_inicial, 
					g1.codigo_tipo_vehiculo,
					g1.garita_inicial as garita_ingreso, 
				    g2.hora_final, 
					g2.garita_final as garita_llegada, 
					g2.vehiculo2,
					g2.placa2,
					(g2.hora_final-g1.hora_inicial) as tiempo, 
					(case when (g2.hora_final-g1.hora_inicial) < g1.tiempo_permitido then 'Exceso de Velocidad' else 'Velocidad Permitida' end ) as limite_velocidad
					
					from (
					Select
					g.fecha_entrada_salida,
					g.dni_conductor,
					g.nombre_conductor,
					g.placa_entrada_salida as placa1,
					g.hora_entrada_salida as hora_inicial,
					t.codigo_tipo_vehiculo,
					t.descripcion_tipo_vehiculo as vehiculo1,
					t.descripcion_velocidad as km,
				    t.tiempo_permitido,	
					c.descripcion_caseta as garita_inicial
					from
					garita_entrada_salida_control g
					inner join caseta_control c on (g.codigo_caseta=c.codigo_caseta)
					inner join tipo_vehiculo_control t on (g.codigo_tipo_vehiculo = t.codigo_tipo_vehiculo)
					where
					g.dni_conductor = :p_dni_conductor and 
					g.codigo_caseta = 4 and g.tipo='S'  and fecha_entrada_salida >= '2023-01-01' 
					group by g.fecha_entrada_salida,g.dni_conductor,g.nombre_conductor,	g.placa_entrada_salida,g.hora_entrada_salida,t.descripcion_tipo_vehiculo,t.codigo_tipo_vehiculo,t.descripcion_velocidad, t.tiempo_permitido,c.descripcion_caseta
					order by fecha_entrada_salida desc, hora_entrada_salida desc	
					limit 1
					)g1
					
					LEFT JOIN (
					
					select
					g.fecha_entrada_salida,
					g.dni_conductor,
					g.placa_entrada_salida as placa2 ,
					t.descripcion_tipo_vehiculo as vehiculo2,
					g.hora_entrada_salida as hora_final,
					c.descripcion_caseta as garita_final
					from
					garita_entrada_salida_control g
					inner join caseta_control c on (g.codigo_caseta=c.codigo_caseta)
					inner join tipo_vehiculo_control t on (g.codigo_tipo_vehiculo = t.codigo_tipo_vehiculo)
					where
					g.dni_conductor = :p_dni_conductor and 
					g.codigo_caseta = 1 and g.tipo='S' and fecha_entrada_salida >= '2023-01-01'
					group by g.fecha_entrada_salida, g.dni_conductor,	g.placa_entrada_salida,g.hora_entrada_salida,t.descripcion_tipo_vehiculo,c.descripcion_caseta
					order by fecha_entrada_salida desc, hora_entrada_salida desc
					limit 1
					) g2
					on
					g1.dni_conductor=g2.dni_conductor;";

						$sentencia = $this->dblink->prepare($sql);
                        $sentencia->bindParam(":p_dni_conductor", $dni);
                        $sentencia->execute();
                        $resultado = $sentencia->fetchAll(PDO::FETCH_ASSOC);
			return $resultado;
		}catch(Exception $exc){
				throw $exc;
    }
	}
	
	
		 function ObtenerVelocidadSalida_gm_gam($dni){
		try{
					$sql = "
                    SELECT 
					g1.fecha_entrada_salida, 
					g1.dni_conductor,
					g1.nombre_conductor, 
					g1.vehiculo1,
					g1.placa1,
					g1.km,
					g1.tiempo_permitido,
					g1.hora_inicial, 
					g1.codigo_tipo_vehiculo,
					g1.garita_inicial as garita_ingreso, 
				    g2.hora_final, 
					g2.garita_final as garita_llegada, 
					g2.vehiculo2,
					g2.placa2,
					(g2.hora_final-g1.hora_inicial) as tiempo, 
					(case when (g2.hora_final-g1.hora_inicial) < g1.tiempo_permitido then 'Exceso de Velocidad' else 'Velocidad Permitida' end ) as limite_velocidad
					
					from (
					Select
					g.fecha_entrada_salida,
					g.dni_conductor,
					g.nombre_conductor,
					g.placa_entrada_salida as placa1,
					g.hora_entrada_salida as hora_inicial,
					t.codigo_tipo_vehiculo,
					t.descripcion_tipo_vehiculo as vehiculo1,
					t.descripcion_velocidad as km,
				    t.tiempo_permitido,	
					c.descripcion_caseta as garita_inicial
					from
					garita_entrada_salida_control g
					inner join caseta_control c on (g.codigo_caseta=c.codigo_caseta)
					inner join tipo_vehiculo_control t on (g.codigo_tipo_vehiculo = t.codigo_tipo_vehiculo)
					where
					g.dni_conductor = :p_dni_conductor and 
					g.codigo_caseta = 3 and g.tipo='S'  and fecha_entrada_salida >= '2022-01-01' 
					group by g.fecha_entrada_salida,g.dni_conductor,g.nombre_conductor,	g.placa_entrada_salida,g.hora_entrada_salida,t.descripcion_tipo_vehiculo,t.codigo_tipo_vehiculo,t.descripcion_velocidad, t.tiempo_permitido,c.descripcion_caseta
					order by fecha_entrada_salida desc, hora_entrada_salida desc	
					limit 1
					)g1
					
					LEFT JOIN (
					
					select
					g.fecha_entrada_salida,
					g.dni_conductor,
					g.placa_entrada_salida as placa2 ,
					t.descripcion_tipo_vehiculo as vehiculo2,
					g.hora_entrada_salida as hora_final,
					c.descripcion_caseta as garita_final
					from
					garita_entrada_salida_control g
					inner join caseta_control c on (g.codigo_caseta=c.codigo_caseta)
					inner join tipo_vehiculo_control t on (g.codigo_tipo_vehiculo = t.codigo_tipo_vehiculo)
					where
					g.dni_conductor = :p_dni_conductor and 
					g.codigo_caseta = 7 and g.tipo='S' and fecha_entrada_salida >= '2022-01-01'
					group by g.fecha_entrada_salida, g.dni_conductor,	g.placa_entrada_salida,g.hora_entrada_salida,t.descripcion_tipo_vehiculo,c.descripcion_caseta
					order by fecha_entrada_salida desc, hora_entrada_salida desc
					limit 1
					) g2
					on
					g1.dni_conductor=g2.dni_conductor;";

						$sentencia = $this->dblink->prepare($sql);
                        $sentencia->bindParam(":p_dni_conductor", $dni);
                        $sentencia->execute();
                        $resultado = $sentencia->fetchAll(PDO::FETCH_ASSOC);
			return $resultado;
		}catch(Exception $exc){
				throw $exc;
    }
	}
	
	public function registarConductorSegundoPlano($dni_conductor, $paterno, $materno, $nombres){
	    try{
	        
	        /** SI NO ESTÁ REGISTRADO EL CONDUCTOR, LO REGISTRAMOS */
            $sql = "SELECT * FROM conductor_control WHERE lower(trim(dni_conductor)) = lower(trim(:p_dni));";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $dni_conductor);
            $sentencia->execute();
            $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
            
            if (!$sentencia->rowCount()) {
                
             
                        $sql = "SELECT * FROM f_generar_correlativo('conductor_control') as nc";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->execute();
                        $resultado = $sentencia->fetch();
                        $nuevoCodigoConductor = $resultado["nc"];
                        $dni = trim($dni_conductor);
                        $sql = "INSERT INTO conductor_control(
                                            codigo_conductor,
                                            dni_conductor,
                                            nombre_conductor,
                                            apellidos_maternos_conductor,
                                            apellidos_paternos_conductor,
                                            estado_conductor
                                        )VALUES(
                                            :p_codigo_conductor,
                                            :p_dni_conductor,
                                            :p_nombre_conductor,
                                            :p_apellidos_maternos_conductor,
                                            :p_apellidos_paternos_conductor,
                                            'A');";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->bindParam(":p_codigo_conductor", $nuevoCodigoConductor);
                        $sentencia->bindParam(":p_dni_conductor", $dni);
                        $sentencia->bindParam(":p_nombre_conductor", $nombres);
                        $sentencia->bindParam(":p_apellidos_paternos_conductor", $paterno);
                        $sentencia->bindParam(":p_apellidos_maternos_conductor", $materno);
                        $sentencia->execute();
                        $sql = "UPDATE correlativo_control SET numero = numero + 1 where tabla = 'conductor_control'";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->execute();
                        return true;
                
            }else{
                
            }
            
        } catch (Exception $exc) {
            throw $exc;
        }
        
         return false;
	    
	}


    /*inician las pruebas*/
    
     public function agregarEntradaMovilPrueba($fecha_entrada_salida, $placa_entrada_salida, $hora_entrada_salida, $codigo_empresa, $dni_conductor, $codigo_usuario, $nombre_conductor, $codigo_tipo_vehiculo, 
    $codigo_empresa2,$codigo_caseta, $tipo, $punto_control, $codigo_ruta, $codigo_proveedor, $codigo_ruta_transporte, $exceso, $version){
        $placaAlterna = "";
        $this->dblink->beginTransaction();
        try {
            /** VALIDAR CASETA DEL USUARIO **/
            $codigo_caseta = $this->obtenerEmpresaDeUsuario($codigo_usuario);
            /** VALIDAR CASETA DEL USUARIO **/
            
            /** SI NO ESTÁ REGISTRADO EL CONDUCTOR, LO REGISTRAMOS */
           
            /** SI NO ESTÁ REGISTRADO EL CONDUCTOR, LO REGISTRAMOS */

            /* OBTENER EL CÓDIGO DE RUTA */
            if ($punto_control == "N" || $punto_control == "-") {
                $holderDni = trim($dni_conductor);
                // Check out if there is a route open.... if its open then close it.
                $sql = "SELECT codigo_ruta as id, timestamp_registro as hora_registro FROM ruta WHERE dni = :p_dni and estado = 'A';";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_dni", $dni_conductor);
                $sentencia->execute();
                $resultado = $sentencia->fetch();

                if ($sentencia->rowCount()) {
                    $hora_registro_ruta = new DateTime($resultado['hora_registro']);
                    $hora_actual = new DateTime();
                    $d = $hora_actual->diff($hora_registro_ruta);

                    if ($d->y == 0 && $d->m == 0 && $d->d == 0 && $d->h > 5) {
                        $sql = "UPDATE ruta SET estado = 'C' WHERE dni = :p_dni and estado = 'A';";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->bindParam(":p_dni", $dni_conductor);
                        $sentencia->execute();

                        // registrar la ruta
                        $lugar_creacion = "G#" . $codigo_caseta;
                        $sql = "INSERT INTO ruta(dni, lugar_creacion) values(:p_dni, :p_creacion);";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->bindParam(":p_dni", $dni_conductor);
                        $sentencia->bindParam(":p_creacion", $lugar_creacion);
                        $sentencia->execute();
                    }
                } else {
                    // registrar la ruta
                    $lugar_creacion = "G#" . $codigo_caseta;
                    $sql = "INSERT INTO ruta(dni, lugar_creacion) values(:p_dni, :p_creacion);";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_dni", $dni_conductor);
                    $sentencia->bindParam(":p_creacion", $lugar_creacion);
                    $sentencia->execute();
                }

                $sql = "SELECT codigo_ruta as id FROM ruta WHERE dni = :p_dni and estado = 'A' order by codigo_ruta desc limit 1;";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_dni", $dni_conductor);
                $sentencia->execute();
                $resultado = $sentencia->fetch();

                $codigo_ruta = $resultado['id'];
            }
            /* OBTENER EL CÓDIGO DE RUTA */

            /* VERIFICAR SI SE DEBE RECORRER TODOS LOS QR O SOLO LOS DATOS ENVIADOS */
            $resultado = ""; // $this->obtenerQRConductorPorDni($dni_conductor, 'T'); // Parámetro 'T' -> Obtener todas las tarjetas QR y recorrerlas
            // TODO: REVISAR PROCESO SI SE APLICA COMO ENTRADA TODOS LOS QR
            if (!empty($resultado)){
                foreach($resultado as $item){
                    if($item["estado"] == "A"){
                        $sql = "SELECT * FROM f_generar_correlativo('prueba_garita_entrada_salida_control') as nc";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->execute();
                        $resultado = $sentencia->fetch();

                        $nuevoNumeroEntradaSalida = $resultado["nc"];

                        $sql = "INSERT INTO prueba_garita_entrada_salida_control(
                                        codigo_entrada_salida,
                                        placa_entrada_salida,
                                        fecha_entrada_salida,
                                        hora_entrada_salida,
                                        codigo_empresa,
                                        dni_conductor,
                                        codigo_usuario,
                                        nombre_conductor,
                                        codigo_tipo_vehiculo,
                                        codigo_empresa2,
                                        placa_alterna_salida,
                                        tipo,
                                        codigo_caseta,
                                        codigo_ruta,
                                        punto_control,
                                        codigo_proveedor,
                                        codigo_ruta_transporte,
                                        exceso,
                                        version_app
                                    )VALUES (
                                        :p_codigo_entrada_salida,
                                        :p_placa_entrada_salida,
                                        :p_fecha_entrada_salida,
                                        :p_hora_entrada_salida,
                                        :p_codigo_empresa,
                                        :p_dni_conductor,
                                        :p_codigo_usuario,
                                        :p_nombre_conductor,
                                        :p_codigo_tipo_vehiculo,
                                        :p_codigo_empresa2,
                                        :p_placa_alterna_salida,
                                        :p_tipo,
                                        :p_codigo_caseta,
                                        :p_codigo_ruta,
                                        :p_punto_control,
                                        :codigo_proveedor,
                                        :codigo_ruta_transporte,
                                        :p_exceso,
                                        :p_version);";

                        $sentencia = $this->dblink->prepare($sql);

                        $sentencia->bindParam(":p_codigo_entrada_salida", $nuevoNumeroEntradaSalida);
                        $sentencia->bindParam(":p_placa_entrada_salida", $item["placa"]);
                        $sentencia->bindParam(":p_fecha_entrada_salida", $fecha_entrada_salida);
                        $sentencia->bindParam(":p_hora_entrada_salida", $hora_entrada_salida);
                        $sentencia->bindParam(":p_codigo_empresa", $item["cod_emp_pertenece"]);
                        $sentencia->bindParam(":p_dni_conductor", $item["dni"]);
                        $sentencia->bindParam(":p_codigo_usuario", $codigo_usuario);
                        $sentencia->bindParam(":p_nombre_conductor", $item["nombre_conductor"]);
                        $sentencia->bindParam(":p_codigo_tipo_vehiculo", $item["codigo_tipo_vehiculo"]);
                        $sentencia->bindParam(":p_codigo_empresa2", $item["cod_emp_pertenece"]);
                        $sentencia->bindParam(":p_placa_alterna_salida", $placaAlterna);
                        $sentencia->bindParam(":p_tipo", $tipo);
                        $sentencia->bindParam(":p_codigo_caseta", $codigo_caseta);
                        $sentencia->bindParam(":p_codigo_ruta", $codigo_ruta);
                        $sentencia->bindParam(":p_punto_control", $punto_control);
                        $sentencia->bindParam(":codigo_proveedor", $item["codigo_proveedor"]);
                        $sentencia->bindParam(":codigo_ruta_transporte", $item["codigo_ruta"]);
                        $sentencia->bindParam(":p_exceso", $exceso);
                        $sentencia->bindParam(":p_version", $version);

                        $sentencia->execute();

                        $sql = "UPDATE correlativo_control SET numero = numero + 1 WHERE tabla = 'prueba_garita_entrada_salida_control'";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->execute();

                        $sql = "INSERT into garita_movil values(:p_codigo_entrada, :p_codigo_caseta);";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->bindParam(":p_codigo_entrada", $nuevoNumeroEntradaSalida);
                        $sentencia->bindParam(":p_codigo_caseta", $codigo_caseta);
                        $sentencia->execute();
                    }
                }
            }else{
                $sql = "SELECT * FROM f_generar_correlativo('prueba_garita_entrada_salida_control') as nc";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->execute();
                $resultado = $sentencia->fetch();

                $nuevoNumeroEntradaSalida = $resultado["nc"];

                $sql = "INSERT INTO prueba_garita_entrada_salida_control(
                                codigo_entrada_salida,
                                placa_entrada_salida,
                                fecha_entrada_salida,
                                hora_entrada_salida,
                                codigo_empresa,
                                dni_conductor,
                                codigo_usuario,
                                nombre_conductor,
                                codigo_tipo_vehiculo,
                                codigo_empresa2,
                                placa_alterna_salida,
                                tipo,
                                codigo_caseta,
                                codigo_ruta,
                                punto_control,
                                codigo_proveedor,
                                codigo_ruta_transporte,
                                exceso,
                                version_app
                            )VALUES (
                                :p_codigo_entrada_salida,
                                :p_placa_entrada_salida,
                                :p_fecha_entrada_salida,
                                :p_hora_entrada_salida,
                                :p_codigo_empresa,
                                :p_dni_conductor,
                                :p_codigo_usuario,
                                :p_nombre_conductor,
                                :p_codigo_tipo_vehiculo,
                                :p_codigo_empresa2,
                                :p_placa_alterna_salida,
                                :p_tipo,
                                :p_codigo_caseta,
                                :p_codigo_ruta,
                                :p_punto_control,
                                :codigo_proveedor,
                                :codigo_ruta_transporte,
                                :p_exceso,
                                :p_version);";

                $sentencia = $this->dblink->prepare($sql);

                $sentencia->bindParam(":p_codigo_entrada_salida", $nuevoNumeroEntradaSalida);
                $sentencia->bindParam(":p_placa_entrada_salida", $placa_entrada_salida);
                $sentencia->bindParam(":p_fecha_entrada_salida", $fecha_entrada_salida);
                $sentencia->bindParam(":p_hora_entrada_salida", $hora_entrada_salida);
                $sentencia->bindParam(":p_codigo_empresa", $codigo_empresa);
                $sentencia->bindParam(":p_dni_conductor", $dni_conductor);
                $sentencia->bindParam(":p_codigo_usuario", $codigo_usuario);
                $sentencia->bindParam(":p_nombre_conductor", $nombre_conductor);
                $sentencia->bindParam(":p_codigo_tipo_vehiculo", $codigo_tipo_vehiculo);
                $sentencia->bindParam(":p_codigo_empresa2", $codigo_empresa2);
                $sentencia->bindParam(":p_placa_alterna_salida", $placaAlterna);
                $sentencia->bindParam(":p_tipo", $tipo);
                $sentencia->bindParam(":p_codigo_caseta", $codigo_caseta);
                $sentencia->bindParam(":p_codigo_ruta", $codigo_ruta);
                $sentencia->bindParam(":p_punto_control", $punto_control);
                $sentencia->bindParam(":codigo_proveedor", $codigo_proveedor);
                $sentencia->bindParam(":codigo_ruta_transporte", $codigo_ruta_transporte);
                $sentencia->bindParam(":p_exceso", $exceso);
                $sentencia->bindParam(":p_version", $version);

                $sentencia->execute();

                $sql = "UPDATE correlativo_control SET numero = numero + 1 WHERE tabla = 'prueba_garita_entrada_salida_control'";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->execute();

                $sql = "INSERT into garita_movil values(:p_codigo_entrada, :p_codigo_caseta);";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_codigo_entrada", $nuevoNumeroEntradaSalida);
                $sentencia->bindParam(":p_codigo_caseta", $codigo_caseta);
                $sentencia->execute();
            }
            /* VERIFICAR SI SE DEBE RECORRER TODOS LOS QR O SOLO LOS DATOS ENVIADOS */

            $sql = "SELECT * from alertas_empresa where dni = :p_dni;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $holderDni);
            $sentencia->execute();

            $resultado = $sentencia->fetchAll();

            if ($codigo_caseta == 1) {
                        $holderGaritaMensaje = ' Garita km 855 ';
                } else {
                       if ($codigo_caseta == 2) {
                        $holderGaritaMensaje = ' Garita Palo Verde ';
                } else {
                        if ($codigo_caseta == 3) {
                        $holderGaritaMensaje = ' Garita Morrope ';
                        } else {
                        $holderGaritaMensaje = ' Garita Km 32 ';
                        }
                    }
                }
            if ($sentencia->rowCount()) {
                foreach ($resultado as $item) {
                    $telefono = $item['telefono'];
                    $mensaje = date('h:i A') . ' ' . $holderGaritaMensaje . ' ' . $item['mensaje'];

                    $url_holder_sms = str_replace(' ', '%20', "http://servicio.smsmasivos.com.ar/enviar_sms.asp?api=1&usuario=PROOLMOS&clave=PROOLMOS352&tos=" . $telefono . "&texto=" . $mensaje);
                    $ch = curl_init();
                    curl_setopt($ch, CURLOPT_URL, $url_holder_sms);
                    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
                    $data = curl_exec($ch);
                    curl_close($ch);
                }
            }
            $this->dblink->commit();
            
           
            return true;
            
            
            
        } catch (Exception $exc) {
            $this->dblink->rollBack();
            throw  $exc;
        }
        
        return false;
    }
    
        public function agregarSalidaMovilPrueba($dni_conductor, $placa_alterna_salida, $placa_entrada_salida, $nombre_conductor, $hora_actual, $fecha_entrada_salida, $codigo_caseta, $codigo_usuario, $tipo, $contenedor, $codigo_proveedor, 
        $codigo_ruta_transporte, $codigo_tipo_vehiculo, $exceso, $version, $dirige)
    {
        $this->dblink->beginTransaction();
        try {
            /** VALIDAR CASETA DEL USUARIO **/
            $codigo_caseta = $this->obtenerEmpresaDeUsuario($codigo_usuario);
            /** VALIDAR CASETA DEL USUARIO **/
            
            /** SI NO ESTÁ REGISTRADO EL CONDUCTOR, LO REGISTRAMOS */
            
            $sql = "SELECT * FROM conductor_control WHERE lower(trim(dni_conductor)) = lower(trim(:p_dni));";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $dni_conductor);
            $sentencia->execute();
            $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);

           /* if (!$sentencia->rowCount()) {
               
                $porciones = explode(" ",$nombre_conductor);
                $paterno = $porciones[0];
                $materno = $porciones[1];
                $nombres = $porciones[2].' '.$porciones[3];


                        $sql = "SELECT * FROM f_generar_correlativo('conductor_control') as nc";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->execute();
                        $resultado = $sentencia->fetch();
                        $nuevoCodigoConductor = $resultado["nc"];
                        $dni = trim($dni_conductor);
                        $sql = "INSERT INTO conductor_control(
                                            codigo_conductor,
                                            dni_conductor,
                                            nombre_conductor,
                                            apellidos_maternos_conductor,
                                            apellidos_paternos_conductor,
                                            estado_conductor
                                        )VALUES(
                                            :p_codigo_conductor,
                                            :p_dni_conductor,
                                            :p_nombre_conductor,
                                            :p_apellidos_maternos_conductor,
                                            :p_apellidos_paternos_conductor,
                                            'A');";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->bindParam(":p_codigo_conductor", $nuevoCodigoConductor);
                        $sentencia->bindParam(":p_dni_conductor", $dni);
                        $sentencia->bindParam(":p_nombre_conductor", $nombres);
                        $sentencia->bindParam(":p_apellidos_paternos_conductor", $paterno);
                        $sentencia->bindParam(":p_apellidos_maternos_conductor", $materno);
                        $sentencia->execute();
                        $sql = "UPDATE correlativo_control SET numero = numero + 1 where tabla = 'conductor_control'";
                        $sentencia = $this->dblink->prepare($sql);
                        $sentencia->execute();
                        
            }*/
            /** SI NO ESTÁ REGISTRADO EL CONDUCTOR, LO REGISTRAMOS */

            $sql = "SELECT codigo_ruta as id FROM ruta WHERE dni = lower(trim(:p_dni)) and estado = 'A' order by timestamp_registro desc;";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_dni", $dni_conductor);
            $sentencia->execute();
            $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);

            $codigo_ruta = "";

            if ($sentencia->rowCount()) {
                $codigo_ruta = $resultado['id'];
                $sql = "UPDATE ruta SET estado = 'C' WHERE dni = lower(trim(:p_dni));";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_dni", $dni_conductor);
                $sentencia->execute();
            }

            $sql = "SELECT * FROM f_generar_correlativo('prueba_garita_entrada_salida_control') AS nc";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->execute();
            $resultado = $sentencia->fetch();

            if ($sentencia->rowCount()) {
                $nuevoNumerosalida = $resultado["nc"];

                if ($codigo_ruta == "") {
                    $sql = "INSERT INTO prueba_garita_entrada_salida_control(codigo_entrada_salida, dni_conductor, placa_entrada_salida, placa_alterna_salida, nombre_conductor, codigo_usuario, hora_entrada_salida, fecha_entrada_salida, codigo_caseta, tipo, contenedor,
                    codigo_proveedor, codigo_ruta_transporte, codigo_tipo_vehiculo, exceso, version_app, codigo_empresa, codigo_empresa2)
                    VALUES (:p_codigo_entrada_salida, :p_dni_conductor, :p_placa_entrada_salida, :p_placa_alterna_salida, :p_nombre_conductor, :p_codigo_usuario, :p_hora_entrada_salida, :p_fecha_entrada_salida, :p_codigo_caseta, :p_tipo, :p_contenedor, :codigo_proveedor,
                    :codigo_ruta_transporte,  :p_codigo_tipo_vehiculo, :p_exceso, :p_version, :p_codigo_empresa, :p_codigo_empresa2);";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_codigo_entrada_salida", $nuevoNumerosalida);
                    $sentencia->bindParam(":p_dni_conductor", $dni_conductor);
                    $sentencia->bindParam(":p_placa_entrada_salida", $placa_entrada_salida);
                    $sentencia->bindParam(":p_placa_alterna_salida", $placa_alterna_salida);
                    $sentencia->bindParam(":p_nombre_conductor", $nombre_conductor);
                    $sentencia->bindParam(":p_codigo_usuario",  $codigo_usuario);
                    $sentencia->bindParam(":p_hora_entrada_salida",  $hora_actual);
                    $sentencia->bindParam(":p_fecha_entrada_salida", $fecha_entrada_salida);
                    $sentencia->bindParam(":p_codigo_caseta", $codigo_caseta);
                    $sentencia->bindParam(":p_tipo", $tipo);
                    $sentencia->bindParam(":p_contenedor", $contenedor);
                    $sentencia->bindParam(":codigo_proveedor", $codigo_proveedor);
                    $sentencia->bindParam(":codigo_ruta_transporte", $codigo_ruta_transporte);
                    $sentencia->bindParam(":p_codigo_tipo_vehiculo", $codigo_tipo_vehiculo);
                    $sentencia->bindParam(":p_exceso", $exceso);
                    $sentencia->bindParam(":p_version", $version);
                    $sentencia->bindParam(":p_codigo_empresa", $dirige);
                    $sentencia->bindParam(":p_codigo_empresa2", $dirige);

                } else {
                    $sql = "INSERT INTO prueba_garita_entrada_salida_control(codigo_entrada_salida, dni_conductor, placa_entrada_salida, placa_alterna_salida, nombre_conductor, codigo_usuario, hora_entrada_salida, fecha_entrada_salida, codigo_caseta, tipo, contenedor, codigo_ruta,
                    codigo_proveedor, codigo_ruta_transporte, codigo_tipo_vehiculo, exceso, version_app, codigo_empresa, codigo_empresa2)
                    VALUES (:p_codigo_entrada_salida, :p_dni_conductor, :p_placa_entrada_salida, :p_placa_alterna_salida, :p_nombre_conductor, :p_codigo_usuario, :p_hora_entrada_salida, :p_fecha_entrada_salida, :p_codigo_caseta, :p_tipo, :p_contenedor,:p_codigo_ruta, :codigo_proveedor,
                    :codigo_ruta_transporte, :p_codigo_tipo_vehiculo, :p_exceso, :p_version, :p_codigo_empresa, :p_codigo_empresa2);";
                    $sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_codigo_entrada_salida", $nuevoNumerosalida);
                    $sentencia->bindParam(":p_dni_conductor", $dni_conductor);
                    $sentencia->bindParam(":p_placa_entrada_salida", $placa_entrada_salida);
                    $sentencia->bindParam(":p_placa_alterna_salida", $placa_alterna_salida);
                    $sentencia->bindParam(":p_nombre_conductor", $nombre_conductor);
                    $sentencia->bindParam(":p_codigo_usuario",  $codigo_usuario);
                    $sentencia->bindParam(":p_hora_entrada_salida",  $hora_actual);
                    $sentencia->bindParam(":p_fecha_entrada_salida", $fecha_entrada_salida);
                    $sentencia->bindParam(":p_codigo_caseta", $codigo_caseta);
                    $sentencia->bindParam(":p_tipo", $tipo);
                    $sentencia->bindParam(":p_contenedor", $contenedor);
                    $sentencia->bindParam(":p_codigo_ruta", $codigo_ruta);
                    $sentencia->bindParam(":codigo_proveedor", $codigo_proveedor);
                    $sentencia->bindParam(":codigo_ruta_transporte", $codigo_ruta_transporte);
                    $sentencia->bindParam(":p_codigo_tipo_vehiculo", $codigo_tipo_vehiculo);
                    $sentencia->bindParam(":p_exceso", $exceso);
                    $sentencia->bindParam(":p_version", $version);
                    $sentencia->bindParam(":p_codigo_empresa", $dirige);
                    $sentencia->bindParam(":p_codigo_empresa2", $dirige);
                }
                //Ejecutar la sentencia preparada
                $sentencia->execute();

                $sql = "UPDATE correlativo_control SET numero = numero + 1 WHERE tabla = 'prueba_garita_entrada_salida_control'";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->execute();

                $sql = "INSERT into garita_movil values(:p_codigo_entrada, :p_codigo_caseta);";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_codigo_entrada", $nuevoNumerosalida);
                $sentencia->bindParam(":p_codigo_caseta", $codigo_caseta);
                $sentencia->execute();

                $this->dblink->commit();

                return true;
            } else {
                throw new Exception("No se a configurado el correlativo para Garitas Principales");
            }
        } catch (Exception $exc) {
            $this->dblink->rollBack();
            throw  $exc;
        }
        return false;
    }
    
        public function listadoGaritaPrueba($fecha1, $fecha2, $tipo, $codigo_caseta, $tipo_entrada) {
        try {
            if($tipo_entrada == "E"){
                $sql = "
                (
                SELECT distinct
                        g.codigo_entrada_salida,
                        g.dni_conductor as dni,
                        (case g.nombre_conductor
                            when '' then 'SIN REGISTRO'
                            when 'null' then 'SIN REGISTRO'
                            when 'CONDUCTOR NO, ENCONTRADO ' then 'CONDUCTOR NO ENCONTRADO'
                            else coalesce(g.nombre_conductor, 'SIN REGISTRO') end) as nomb,
                        coalesce(c.estado_conductor, 'A') as est,
                        to_char(g.hora_entrada_salida,'HH12:MI AM')AS hora,
                        to_char(g.fecha_entrada_salida,'DD/MM/YYYY')AS fech,
                        (case g.exceso when 1 then 'EXCESO DE VELOCIDAD' else 'VELOCIDAD PERMITIDA' end) AS EXCESO,
                        coalesce(e.nombre_empresa, '') as perte,
                        coalesce(e2.nombre_empresa, '') as diri,
                        upper(g.placa_entrada_salida) as placa,
                        coalesce(v.descripcion_tipo_vehiculo, '') as tipo_vehi
                    FROM prueba_garita_entrada_salida_control as g Left join conductor_control as c on(g.dni_conductor = c.dni_conductor)
                            INNER JOIN empresa_control as e on (g.codigo_empresa = e.codigo_empresa)
                            inner join empresa_control as e2 on (g.codigo_empresa2 = e2.codigo_empresa)
                            inner join tipo_vehiculo_control as v on (g.codigo_tipo_vehiculo = v.codigo_tipo_vehiculo)
                    WHERE
                        g.tipo= 'E' and g.codigo_caseta = :p_codigo_caseta and g.estado_entrada = 'E'
                        and (
                            case :p_tipo
                            when 'H' then g.fecha_entrada_salida = current_date
                            else g.fecha_entrada_salida between :p_fecha1 and :p_fecha2
                            end
                            )
                    order by
                        g.codigo_entrada_salida desc
                        
                )
                
                UNION
                
                 (
                SELECT distinct
                        g.codigo_entrada_salida,
                        g.dni_conductor as dni,
                        (case g.nombre_conductor
                            when '' then 'SIN REGISTRO'
                            when 'null' then 'SIN REGISTRO'
                            when 'CONDUCTOR NO, ENCONTRADO ' then 'CONDUCTOR NO ENCONTRADO'
                            else coalesce(g.nombre_conductor, 'SIN REGISTRO') end) as nomb,
                        coalesce(c.estado_conductor, 'A') as est,
                        to_char(g.hora_entrada_salida,'HH12:MI AM')AS hora,
                        to_char(g.fecha_entrada_salida,'DD/MM/YYYY')AS fech,
                        (case g.exceso when 1 then 'EXCESO DE VELOCIDAD' else 'VELOCIDAD PERMITIDA' end) AS EXCESO,
                        coalesce(e.nombre_empresa, '') as perte,
                        coalesce(e2.nombre_empresa, '') as diri,
                        upper(g.placa_entrada_salida) as placa,
                        coalesce(v.descripcion_tipo_vehiculo, '') as tipo_vehi
                    FROM garita_entrada_salida_control as g Left join conductor_control as c on(g.dni_conductor = c.dni_conductor)
                            INNER JOIN empresa_control as e on (g.codigo_empresa = e.codigo_empresa)
                            inner join empresa_control as e2 on (g.codigo_empresa2 = e2.codigo_empresa)
                            inner join tipo_vehiculo_control as v on (g.codigo_tipo_vehiculo = v.codigo_tipo_vehiculo)
                    WHERE
                        g.tipo= 'E' and g.codigo_caseta = :p_codigo_caseta and g.estado_entrada = 'E'
                        and (
                            case :p_tipo
                            when 'H' then g.fecha_entrada_salida = current_date
                            else g.fecha_entrada_salida between :p_fecha1 and :p_fecha2
                            end
                            )
                    order by
                        g.codigo_entrada_salida desc
                        
                )
                
                ";
            }else{
                $sql = "SELECT
                        distinct
                        g.codigo_entrada_salida,
                        g.dni_conductor as dni,
                        (case g.nombre_conductor
                            when '' then 'SIN REGISTRO'
                            when 'null' then 'SIN REGISTRO'
                            when 'CONDUCTOR NO, ENCONTRADO ' then 'CONDUCTOR NO ENCONTRADO'
                            else coalesce(g.nombre_conductor, 'SIN REGISTRO') end) as nomb,
                        coalesce(c.estado_conductor, 'A') as est,
                        to_char(g.hora_entrada_salida,'HH12:MI AM')AS hora,
                        to_char(g.fecha_entrada_salida,'DD/MM/YYYY') AS fech ,
						(case g.exceso when 1 then 'EXCESO DE VELOCIDAD' else 'VELOCIDAD PERMITIDA' end) AS EXCESO,
                        upper(g.placa_entrada_salida) as placa,
                        upper(g.placa_alterna_salida) as placa_alterna,
                        (case coalesce(g.contenedor, 'N') when '' then 'N' else g.contenedor end) as contenedor
                    FROM prueba_garita_entrada_salida_control as g Left join conductor_control as c on(g.dni_conductor = c.dni_conductor)
                    WHERE
                        g.tipo= 'S' and g.codigo_caseta = :p_codigo_caseta and g.estado_entrada = 'E'
                        and (
                            case :p_tipo
                            when 'H' then g.fecha_entrada_salida = current_date
                            else g.fecha_entrada_salida between :p_fecha1 and :p_fecha2
                            end
                            )
                    order by
                        g.codigo_entrada_salida desc;";
            }


            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_fecha1", $fecha1);
            $sentencia->bindParam(":p_fecha2", $fecha2);
            $sentencia->bindParam(":p_tipo", $tipo);
            $sentencia->bindParam(":p_codigo_caseta", $codigo_caseta);
            $sentencia->execute();

            $resultado = $sentencia->fetchAll(PDO::FETCH_ASSOC);

            return $resultado;

        } catch (Exception $exc) {
            throw $exc;
        }
    }
    
        public function listarSancionados() {
        try {
            $sql = "SELECT * FROM public.conductor_control where estado_conductor = 'I'";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->execute();
            $resultado = $sentencia->fetchAll(PDO::FETCH_ASSOC);

            return $resultado;

        } catch (Exception $exc) {
            throw $exc;
        }
    }
    
        public function listar5diasEntradaSalidaPaloVerde() {
        try {
            $sql = "SELECT * 
                    FROM garita_entrada_salida_control gesc
                    WHERE fecha_entrada_salida  >= NOW() - INTERVAL '1 days' and codigo_caseta = 1 and tipo = 'E'";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->execute();
            $resultado = $sentencia->fetchAll(PDO::FETCH_ASSOC);

            return $resultado;

        } catch (Exception $exc) {
            throw $exc;
        }
    }
    
 public function leerDatosQrConductorCarga($hash)
    {

        $this->dblink->beginTransaction();

        try {
            $sql = "SELECT *
                    FROM public.registro_qr_vehiculos_carga
                    WHERE replace(codigo_hash, ' ', '') = :p_hash
                    ORDER BY idregistroqr DESC
                    LIMIT 1;
                    ";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_hash", $hash);
            $sentencia->execute();
            $resultado = $sentencia->fetch(PDO::FETCH_ASSOC);
            $fecha_salida = $resultado['fecha_salida'];
            $hora_salida = $resultado['hora_registro'];

            $fechaActual = date('Y-m-d');
            $horaActual = date('H:i:s');


            if ($sentencia->rowCount()) {

                $operacion = strtotime("$fechaActual $horaActual") - strtotime("$fecha_salida $hora_salida");

                // Comparar la diferencia con 24 horas (86400 segundos)
                if ($operacion < 0) {
                    // QR expirado
                    $resultado['mensaje'] = 'inaceptable';
                } elseif ($operacion <= 86400) {
                    // Aceptable (dentro de 24 horas)
                    $resultado['mensaje'] = 'aceptable';
                } else {
                    // Aún no puede entrar (> 24 horas)
                    $resultado['mensaje'] = 'expirado';
                }
            } else{
                 $resultado['mensaje'] = 'inexistente';
            }
            
            $this->dblink->commit();

            $resultado['fecha_actual'] = $fechaActual;
            $resultado['hora_actual'] = $horaActual;
            $resultado['tiempo'] = $operacion;
            return $resultado;
        } catch (Exception $exc) {
            $this->dblink->rollBack();
            throw $exc;
        }
    }
    
     function ObtenerWS($paquete){
		try{
		    
					$sql = "select * from ruta_app_ws where descripcion= :p_descripcion and estado = 'A' LIMIT 1;";
                 	$sentencia = $this->dblink->prepare($sql);
                    $sentencia->bindParam(":p_descripcion", $paquete);
                    $sentencia->execute();
                    $resultado = $sentencia->fetchAll(PDO::FETCH_ASSOC);
			return $resultado;
		}catch(Exception $exc){
				throw $exc;
        }
	}


}